/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// BriefcaseSettingsAdvanced.cpp : implementation file
//

#include "stdafx.h"
#include "BriefcaseSettingsAdvanced.h"
//#include "comutil.h"
#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"
#include "tpcshell.h"
#include "ClientUtil.h"

// CBriefcaseSettingsAdvanced dialog

IMPLEMENT_DYNAMIC(CBriefcaseSettingsAdvanced, CDialog)

CBriefcaseSettingsAdvanced::CBriefcaseSettingsAdvanced(CWnd* pParent /*=NULL*/)
	: CDialog(CBriefcaseSettingsAdvanced::IDD, pParent)
{

}

CBriefcaseSettingsAdvanced::~CBriefcaseSettingsAdvanced(){
  #if defined(WIN32_PLATFORM_PSPC)
    editRemote.Detach();
  #endif
}

void CBriefcaseSettingsAdvanced::DoDataExchange(CDataExchange* pDX)
{
    DDX_Control(pDX, IDC_BRIEFCASE_EDIT_REMOTE, editRemote);
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CBriefcaseSettingsAdvanced, CDialog)
#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CBriefcaseSettingsAdvanced::OnHotkey)
#endif
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


BOOL CBriefcaseSettingsAdvanced::OnInitDialog(){
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ADVANCED_BRIEFCASE); SetWindowText(s1);
    CDialog::OnInitDialog();

    // create menubar
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
    #endif
    #endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    #if defined(WIN32_PLATFORM_WFSP)
        // override back key
        LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
            SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
        ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
    #endif

    editRemote.SetLimitText(EDIT_MAXLENGTH);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_REMOTENAME); SetDlgItemText(IDC_BRIEFCASE_STATIC_REMONAME,s1);

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_BRIEFCASE_STATIC_REMONAME)->SetFont(&fontBold);

    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ADVANCED_BRIEFCASE); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif

    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("briefcase");
    s1 = ssconf->getURI(); SetDlgItemText(IDC_BRIEFCASE_EDIT_REMOTE,s1);

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    editRemote.SetSel(LARGE_INT ,LARGE_INT);
    editRemote.SetFocus();

    return FALSE;
}


void CBriefcaseSettingsAdvanced::OnOK(){
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("briefcase");
    CString s1;
    //_bstr_t bst;

    if(checkStartSync()){
        CDialog::OnOK();
    }

    GetDlgItemText(IDC_BRIEFCASE_EDIT_REMOTE, s1);
    wstring s((LPCTSTR)s1);    
    char* tmp = toMultibyte(s.c_str());    
    if(strcmp(tmp,ssconf->getURI()) != 0){
        ssconf->setURI(tmp);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_BRIEFCASE);
        getRegConfig()->saveDirty();
    }
    if (tmp) {
        delete [] tmp; tmp = NULL;
    }
    /*
    bst=s1.AllocSysString();
    if(strcmp((char*)bst,ssconf->getURI()) != 0){
        ssconf->setURI((char*)bst);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_BRIEFCASE);
        getRegConfig()->saveDirty();
    }
    bst.Detach();
    */
    CDialog::OnOK();
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CBriefcaseSettingsAdvanced::OnHotkey(WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == VK_TBACK)
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    return 0;
}
#endif


HBRUSH CBriefcaseSettingsAdvanced::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}