/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "CommunicationSettings.h"

#include "LoglevelSettings.h"
#include "RecoverSettings.h"
#include "About.h"
#include "PushSettings.h"
#include "SyncSettings.h"
#include "pim/ClientSettings.h"
#include "pim/maincpp.h"
#include "shellapi.h"

#include "SynchronizationMessages.h"

#include "Shellapi.h"
#include "aygshell.h"
#include "pim/SettingFunctions.h"
#include "ClientUtil.h"

#include "base/util/utils.h"
#include <windows.h>

#include "comutil.h"
#include "HwndFunctions.h"
#include "notify/timed_msgbox.h"

#include "CustomButton.h"
#include "ClientUtil.h"

#include "MailSettings.h"
#include "ContactSettings.h"
#include "CalendarSettings.h"
#include "TaskSettings.h"
#include "BriefcaseSettings.h"
#include "NotesSettings.h"
#include "DeviceInfo.h"
#include "pim/account.h"
#include "processUtils.h"
#include "Popup.h"


#define ITEMS_FROM_SERVER 20

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

long retValueLastSync=0;
bool isServerConfigured();
void trim(wstring& str);
wstring reformatDate(wstring date);
wstring reformatTime(wstring time);

static bool showNoteSource = false;
static bool showBriefcaseSource = false;

CuiDlg::CuiDlg(CWnd* pParent /*=NULL*/)
: CDialog(CuiDlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
    openUp = false;
}


void CuiDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_LIST1, lstSources);
}


BEGIN_MESSAGE_MAP(CuiDlg, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE)
    ON_WM_SIZE()
#endif
    ON_COMMAND(IDM_COMM, OnMenuComm)
    //}}AFX_MSG_MAP
    ON_COMMAND(IDM_LOGLEVEL,                     &CuiDlg::OnMenuLoglevel)
    ON_COMMAND(ID_SYNC,                          &CuiDlg::OnSync)
    ON_COMMAND(ID_MENU_EXIT,                     &CuiDlg::OnMenuExit)
    ON_COMMAND(ID_RECOVER,                       &CuiDlg::OnMenuRecover)
    ON_COMMAND(ID_MENU_CLEANUP,                  &CuiDlg::OnMenuRecover)
    ON_MESSAGE(ID_MYMSG_CONNECTING,              &CuiDlg::OnMsgConnecting)
    ON_MESSAGE(ID_MYMSG_ENDING,                  &CuiDlg::OnMsgEndingSync)
    ON_MESSAGE(ID_MYMSG_ENDING_SOURCE,           &CuiDlg::OnMsgEndingSource)
    ON_MESSAGE(ID_MYMSG_STARTSYNC_ENDED,         &CuiDlg::OnStartsyncEnded)
    ON_MESSAGE(ID_MYMSG_STARTING_SYNC,           &CuiDlg::OnStartingSync)
    ON_MESSAGE(ID_MYMSG_STARTING_SOURCE,         &CuiDlg::OnStartingSource)
    ON_MESSAGE(ID_MYMSG_ITEM_SYNCED_FROM_SERVER, &CuiDlg::OnMsgItemSyncedFromServer)
    ON_MESSAGE(ID_MYMSG_ITEM_SYNCED_FROM_CLIENT, &CuiDlg::OnMsgItemSyncedFromClient)
    ON_MESSAGE(ID_MYMSG_POPUP,                   &CuiDlg::OnMsgPopup)
    ON_MESSAGE(ID_MYMSG_TOTAL_ITEMS,             &CuiDlg::OnMsgTotalItems)
    ON_MESSAGE(ID_MYMSG_OK, &CuiDlg::OnOKMsg)
    

    ON_MESSAGE(ID_MYMSG_TOTAL_NEW, &CuiDlg::OnMsgTotalItemsNew)
    ON_MESSAGE(ID_MYMSG_TOTAL_MOD, &CuiDlg::OnMsgTotalItemsMod)
    ON_MESSAGE(ID_MYMSG_TOTAL_DEL, &CuiDlg::OnMsgTotalItemsDel)


    ON_COMMAND(ID_MENU_UPDATE_SW, &CuiDlg::OnUpdateProcedure)
    ON_COMMAND(ID_MENU_ABOUT,                    &CuiDlg::OnMenuAbout)
    ON_COMMAND(IDABORT,                          &CuiDlg::OnMenuStop)
    ON_COMMAND(ID_SETTINGS_PUSHSETTINGS,         &CuiDlg::OnMenuPush)
    ON_COMMAND(ID_SETTINGS_SYNC,                 &CuiDlg::OnMenuSync)
    
                    
    ON_WM_CLOSE()
    //ON_WM_CONTEXTMENU() // for context menu
    ON_MESSAGE(ID_MYMSG_SOURCE_STATE,            &CuiDlg::OnMsgSourceState)
    ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST1,        &CuiDlg::OnItemchangedList1)
    #if defined(WIN32_PLATFORM_PSPC)
//        ON_COMMAND(ID_MENU_CONTEXT_SYNC, &CuiDlg::OnContextMenuSync)
//        ON_COMMAND(ID_MENU_CONTEXT_SETTINGS, &CuiDlg::OnContextMenuSettings)
    #endif
    ON_WM_ACTIVATE()
    ON_WM_INITMENUPOPUP()   
    ON_COMMAND(ID_TOOLS_DEVICEINFO, &CuiDlg::OnToolsDeviceinfo)
END_MESSAGE_MAP()


/**
* It deletes a special file created during the installation process to signal
* there is an update in progress. If the user chooses to stop the upgrade of the 
* product, this files remain there and there is no way to remove. So at the start
* of the plug-in we remove the file in any way. See setup.cpp for other detail.
*/
void deleteFileForUpdate() {
    DeleteFile(TEXT("\\__uninstall_Fun_plugin__"));
}


class ParamsForUpdate {
public:
    CuiDlg& dialogWnd;
    bool manual;

    ParamsForUpdate(CuiDlg& wnd, bool val) : 
            dialogWnd(wnd), manual(val) {};
    ~ParamsForUpdate() {};
} ;
 
static bool isThereAnUpdate() {
    
    bool ret = false;
    ClientSettings* cs = getRegConfig();
    cs->readUpdateParams();
    if ((getRegConfig()->getUpdateParams().getLater() != "0"   &&
        getRegConfig()->getUpdateParams().getLater() != "")    ||
        (getRegConfig()->getUpdateParams().getSkipped() != ""  &&
        getRegConfig()->getUpdateParams().getSkipped() != "0") ||
        (getRegConfig()->getUpdateParams().getNow() != ""  &&
        getRegConfig()->getUpdateParams().getNow() != "0")   ) {
            ret = true;
    }
    return ret;
}
/**
* Thread that starts after the sync of the credential if there is 
* an update available
*/
static DWORD WINAPI updateSettingsProcedureWorker(LPVOID lpv) {
    Sleep(1000);
    ParamsForUpdate* p = (ParamsForUpdate*)lpv;         
    int ret = updateProcedure(p->dialogWnd.GetSafeHwnd(), false);       
    return 0;
}

static DWORD WINAPI updateProcedureWorker(LPVOID lpv) {
    ParamsForUpdate* p = (ParamsForUpdate*)lpv;     
    SetCursor(LoadCursor(NULL, IDC_WAIT)); // LoadIconModal();   // 
    checkAvailableUpdate();
    SetCursor(LoadCursor(NULL, IDC_ARROW)); // RemoveIconModal(); // 
    if (p->manual == true) {
        p->dialogWnd.OnOK();        
    } 
    int ret = updateProcedure(p->dialogWnd.GetSafeHwnd(), p->manual);       
    return 0;
}

void CuiDlg::OnUpdateProcedure() {      
    int ret = OnMsgPopup(IDS_BUT_NOW_LATER, IDS_UP_MESSAGE);        
    LOG.debug("Asked if the user want update (mandatory): response (0Ye, 1La) %i", ret);
    if (ret == 0) {        
        ParamsForUpdate* p = new ParamsForUpdate(*this, true);
        if ( !CreateThread(NULL, 0, updateProcedureWorker, (LPVOID)p, 0, NULL) ) {
            LOG.error("OnUpdateProcedure Error creating updateProcedureWorker thread.");           
        }        
    } else {        
        // the user chooses Later. We only update the timer
        updateLastCheckUpdate();
        
    }    
}

void CuiDlg::OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex, BOOL bSysMenu) { 
       
    if (isThereAnUpdate()) {
        CString item;
        pPopupMenu->GetMenuString(3, item, MF_BYPOSITION);
        CString s1; s1.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_UPDATE_SOFTWARE);
        
        if (item != s1) {
            pPopupMenu->InsertMenu(3, MF_BYCOMMAND | MF_BYPOSITION, ID_MENU_UPDATE_SW, s1);
        }
    }
}

// start the sync only for email
static DWORD WINAPI firtsSyncWorker(LPVOID lpv) {
    Sleep(2000);   
    startProgram(SYNCAPP, TEXT("manual mail"));
    return 0;
}

void CuiDlg::OnMenuComm()
{
    isInSettings = true;
    CCommunicationSettings wndCommSett;
    wndCommSett.setIsFromMainMenu(true);
    INT_PTR result = wndCommSett.DoModal();   
    // it returns 1 if the sync process has aborted
    if (result == 1) {
        OnStartsyncEnded(NULL, (LPARAM) -20 );
        bSyncStarted = false;
    }   
    else if (result == 2) {
        if ( !CreateThread(NULL, 0, firtsSyncWorker, NULL, 0, NULL) ) {
            LOG.error("firtsSyncWorker Error creating updateProcedureWorker thread.");           
        }
    }
    
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
    this->Invalidate();
}


// CuiDlg message handlers

BOOL CuiDlg::OnInitDialog()
{
    
    // it is used to decide if reset the counter of the total item.
    resetTheItem = false;
    // it is used to populate the message without the default message
    messageToWrite = "";

    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_FUNAMBOL); SetWindowText(s1);
    CDialog::OnInitDialog();
    LOG.setLevel(getLOGLevel());
    AfxGetMainWnd()->EnableWindow();
    // init settings
    ClientSettings* cs = getRegConfig();
    
    //
    // it defines if we have to show the briefcase source and note source
    //
    showBriefcaseSource = cs->getBrandingParams().getShowBriefcaseSource();
    showNoteSource = cs->getBrandingParams().getShowNoteSource();

    deleteFileForUpdate();
    
    try {
        // Set the icon for this dialog.  The framework does this automatically
        //  when the application's main window is not a dialog
        SetIcon(m_hIcon, TRUE);			// Set big icon
        SetIcon(m_hIcon, FALSE);		// Set small icon
    }
    catch(...) {
        LOG.error("uiDlg: can't set application icon.");
    }

     // try to set the font used in main window
    try {
        LOGFONT lf;
        memset(&lf, 0, sizeof(LOGFONT));
        HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
        GetObject(hFont, sizeof lf, &lf);
        lf.lfWeight = FW_BOLD;
        lf.lfHeight += FONT_HEIGHT_OFFSET;
        VERIFY(lstSources.fontBold.CreateFontIndirect(&lf));
    }
    catch(...) {
        LOG.error("uiDlg: can't set main screen font.");
    }

    // check if another sync is in progress
    if(checkStartSync() != 0)
    {
        CreateMenuBar(IDM_STOP);
        #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            CreateMenuBar(IDR_MENUBAR_STOP);
        #endif
        #endif
        bSyncStarted = true;
    }
    else
    {
        CreateMenuBar(IDR_MENU_MAIN);
       
        #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            CreateMenuBar(IDR_MAINFRAME);
        #endif
        #endif
        bSyncStarted = false;
    }

     // init variables. To put in the constructor
    //**************************
    currentSource = 0;
    resetCurrentItem = true;
    lastItem = 0;
    currentItem = 0;
    totalItems = 0;
    sourceEnds = 0;
    sourceBegins = 0;
    nMailsToSync = 0;

    isInSettings = false;
    firstSourceId = -1;

    // set states
    syncSourceContactState = SYNCSOURCE_STATE_OK;
    syncSourceCalendarState = SYNCSOURCE_STATE_OK;
    syncSourceTaskState = SYNCSOURCE_STATE_OK;
    syncSourceNoteState = SYNCSOURCE_STATE_OK;
    syncSourceBriefcaseState = SYNCSOURCE_STATE_OK;
    syncSourceMailState = SYNCSOURCE_STATE_OK;
    bSyncCanceled = false;
    //**************************

    initSyncControls();
    lstSources.SetItemState(0, LVIS_SELECTED, LVIF_STATE);

    // stretch the list to cover all client area.
    // this is used only for smartphone because it has only one dialog.
    // For pocket pc it is overwritten by the OnSize function
    CRect rect;
    GetClientRect(&rect);
    lstSources.setItemWidth(rect.Width());
    int panesHeight = max(rect.Height()/8, 30);     // min height is 30px
    lstSources.setItemHeight(panesHeight);
    lstSources.SetWindowPos(GetActiveWindow(),0,0, rect.Width(), rect.Height(), SWP_SHOWWINDOW);
    lstSources.Invalidate();          

    CString user;
    CString password;
    user = getRegConfig()->getAccessConfig().getUsername();
    password = getRegConfig()->getAccessConfig().getPassword();
    string syncConfig = getRegConfig()->getToSyncConfiguration();

    if( (!(isServerConfigured())) || user.IsEmpty() || password.IsEmpty() ||
        syncConfig == "1"){

        lstSources.ShowWindow(SW_HIDE);                
        //OnMenuComm();
        isInSettings = true;
        CCommunicationSettings commSettings;
        CString msg;
        if (syncConfig != "1" || user.IsEmpty() || password.IsEmpty()) {
            msg.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_ERROR_ENTER_CREDENTIALS);
            commSettings.showMessage(msg);
        }
        INT_PTR result = commSettings.DoModal();
        refreshSourcePane(SOURCE_MAIL);
        lstSources.ShowWindow(SW_NORMAL);       
        if (result == 2) {
            if ( !CreateThread(NULL, 0, firtsSyncWorker, NULL, 0, NULL) ) {
                LOG.error("firtsSyncWorker Error creating updateProcedureWorker thread.");           
            }
        }
       
        isInSettings = false;
        return TRUE;

    }

    //this is set a true the first time to prevent not refresh
    //the date in on activate function before OnInitDialog 
    openUp = true;
    
    return TRUE;
}
//////////////////////////////////////////////////////////////////////////



void CuiDlg::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    int resId = IDD_UI_DIALOG;

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_DIALOG_WIDE;
        break;
    case DRA::Portrait:
        resId = IDD_UI_DIALOG; break;
    case DRA::Square:
        resId = IDD_UI_DIALOG_SQUARE; break;
    default:
        resId = IDD_UI_DIALOG; break;
    };
#endif

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));

    // stretch the list to cover all client area
    CRect rect;
    GetClientRect(&rect);
    lstSources.setItemWidth(rect.Width());
    int panesHeight = max(rect.Height()/8, 30);
    lstSources.setItemHeight(panesHeight);
    lstSources.SetWindowPos(GetActiveWindow(),0,0, rect.Width(), rect.Height(),
        SWP_SHOWWINDOW );

    // Recalculate the main logo position.
    //ClientSettings* cs = getRegConfig();
    //if(cs->getIsPortal()) {
    //    lstSources.calculateMainLogoPosition();
    //}
    if (!showNoteSource && !showBriefcaseSource) {
        lstSources.calculateMainLogoPosition();
    }

    lstSources.Invalidate();
}


void CuiDlg::OnMenuLoglevel()
{
    isInSettings = true;
    CLoglevelSettings wndLoglevel;
    INT_PTR result = wndLoglevel.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}

void CuiDlg::OnSync()
{
    if (!lstSources.getIsLocked()){
        lockPanes();
    }

    // start a sync of all enabled sources
    lstSources.setStateToAll(ITEM_STATE_TO_SYNC, true);
    lstSources.SetItemState(lstSources.GetCurSelItem(), 0, LVIS_FOCUSED);
    refreshSourcePane(SOURCE_MAIL);
    setFirstSourceToSync();
    StartSync(TEXT("manual"));
}

void CuiDlg::OnClose()
{
    int processID = 0;
    HWND wnd = NULL;
    wnd = ::FindWindow(TEXT("funambolstartsync"), NULL);
    if ((processID = checkStartSync()) != 0)  {
        // sync is in progress
        HwndFunctions::closePreviousMsgBox();
        
        // send msg to startsync to stop
        if(wnd){
            ::PostMessage(wnd, WM_COMMAND, (WPARAM)10, NULL);
        }
        Sleep(500);
        DWORD code = 0;
        GetExitCodeProcess((HANDLE)processID, &code);
        if (code == STILL_ACTIVE) {
            HANDLE hProcess = OpenProcess(0, FALSE, processID);
            if (hProcess) {
                startProgram(SYNCAPP, TEXT("removeNotif"));
                TerminateProcess(hProcess, -6); //-6 code for the user stop sync.
                CloseHandle(hProcess);
            }
        }
    }
    else {        
        startProgram(SYNCAPP, TEXT("removeNotif"));        
    }
    EndDialog(0);
}

void CuiDlg::OnMenuExit()
{
    int ret = EndAppQuery(GetSafeHwnd());
    if((ret == 0) || (ret== 1)) {
        //CDialog::OnOK();
        getRegConfig()->saveDirty();
    }
    EndDialog(0);
}

void CuiDlg::OnMenuRecover()
{
    CRecoverSettings wndRecover;
    // TODO: workaround for isInSettings
    isInSettings = true;
    INT_PTR result = wndRecover.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
     // Just to be sure...
    if (result == RECOVER_SYNC_STARTED) {
        SetCursor(LoadCursor(NULL, IDC_ARROW));
    }

    ClientSettings* cs = getRegConfig();
    cs->read();
    refreshSourcePanes();
            
    /*
    if(result == RECOVER_SYNC_STARTED){
        // the user started a recover sync
    }
    */
    //
    // this is to prevent to loose the push that maybe are come
    // during the cleanup process
    // 
    wchar_t** queue = readQueueList();
    if (queue) {        
        wstring cmdLine(TEXT("manual"));
        for (int count = 0; queue[count] != NULL; count++) {     

            if (count == 0) { cmdLine += L" "; }
            else            { cmdLine += L","; }
            cmdLine += queue[count];                       
        
        } 
        delete [] queue;        
        startProgram(SYNCAPP, cmdLine.c_str());
    }    
    
    isInSettings = false;
        
}


LRESULT CuiDlg::OnStartsyncEnded(WPARAM wParam, LPARAM lParam)
{
    CString s1("");

    lstSources.stopAnim(currentSource);
    lstSources.stopMainLogo();

    ClientSettings* cs = getRegConfig();
    cs->read();

    if(!isInSettings){
        CreateMenuBar(IDR_MENU_MAIN);
        #if defined WIN32_PLATFORM_WFSP
            #if (_WIN32_WCE < 0x500)
                CreateMenuBar(IDR_MAINFRAME);
            #endif
        #endif
    }

    retValueLastSync = lParam;
    bSyncStarted = false;

    // stop sync icon animation
    for(int i=0;i<lstSources.GetItemCount();i++){
        lstSources.stopAnim(lstSources.indexToId(i));
    }

    if( (retValueLastSync == 0) || (retValueLastSync == 10003) ||
        (retValueLastSync == -20) ||  (retValueLastSync == -6) ){  
        // for these errorcodes we don't display error message
    }
    else if (retValueLastSync == ERR_CONFIG_SOURCE_NOT_FOUND) {
        // Code 2100 = "config" source not found on the Server.
        // It's an old Server, that doesn't support config sync. 
        // Don't display messages and disable the source "config"
        // so we won't do config sync again, when UI params are modified.
        LOG.info("configuration syncsource not found on the Server (code 404).");
        LOG.debug("Config syncmode is now set to 'none'.");
        SyncSourceConfig* ssc = cs->getSyncSourceConfig("config");
        if (ssc) {
            ssc->setSync("none");
            cs->setDirty(CS_DIRTY_SOURCE_ENABLED);
            cs->saveDirty();
        }
    }
    else {
        // For smartphones, we don't want a waiting cursor over the msgbox.
        SetCursor(LoadCursor(NULL, IDC_ARROW));

        CString message = SyncErrorMsg(retValueLastSync);
        HwndFunctions::closePreviousMsgBox();
        HWND wndFocus = GetSafeHwnd();
        TimedMessageBox(GetSafeHwnd(), message, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);

        ::SetActiveWindow(wndFocus);
    }

    /* run the update procedure to show if there is one available. The check is done only if there
     * aren't any http error or is not a user stop of the sync      
    */
    if (!isInSettings && retValueLastSync >= 0 && retValueLastSync < ERR_TRANSPORT_BASE) {
        getRegConfig()->readUpdateParams();
        if (getRegConfig()->getUpdateParams().getIsAnUpdateAvailable() == "1") {
            updateProcedure(GetSafeHwnd(), false);
        }
    }

    // reset variables
    currentSource = 0;
    resetCurrentItem = true;
    totalItems = 0;
    lastItem = 0;
    sourceEnds = 0;
    sourceBegins = 0;
    firstSourceId = -1;
    nMailsToSync = 0;

    lstSources.SetItemState(lstSources.GetCurSelItem(), 0, LVIS_FOCUSED);

    //refreshRecoveredSources();
    unlockPanes();

    // for the case of scheduled sync when the user is in a config window
    if(!isInSettings){
        refreshSourcePanes();
    }

    if(! bSyncCanceled) // TODO: is this necessary?
        lstSources.setStateToAll(ITEM_STATE_OK);

    return 0;
}

LRESULT CuiDlg::OnStartingSync(WPARAM wParam, LPARAM lParam)
{
    CString s2;

    if(!isInSettings){
        HwndFunctions::closePreviousMsgBox();

        CreateMenuBar(IDM_STOP);
        #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            CreateMenuBar(IDR_MENUBAR_STOP);
        #endif
        #endif
    }
    lockPanes();

    bSyncStarted = true;

    // Starts the timer for the main logo animation
    lstSources.startMainLogo();

    //getRegConfig()->saveDirty();
    // received start sync message, we check if startsync.exe has given the first source to be synced
    // if wParam !=0, then in lParam we have the id of the 1st source
    if(wParam != 0){
        firstSourceId = (int)lParam;
    }

    return 0;
}

void CuiDlg::OnMenuAbout()
{
    CAbout wndAbout;
    isInSettings = true;
    INT_PTR result = wndAbout.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
    refreshSourcePanes();
}

void CuiDlg::OnMenuStop()
{
    // because the sync is possibly being canceled in EndAppQuery, we set bSyncCanceled to true before calling it
    bSyncCanceled = true;
    int ret = EndAppQuery(GetSafeHwnd());
    // If startsync was not running simulate the message.
    // 1 - sync not running
    // 0 - user choosed to stop the sync, we need to call OnStartsyncEnded
    //       to reset the controls
    if(ret== 0) {
        bSyncCanceled = true;
        refreshCanceledState();
        OnStartsyncEnded(NULL, (LPARAM) -20 ); // nothing to show...
    }
    else if (ret== 1){
        bSyncCanceled = true;
        refreshCanceledState();
        OnStartsyncEnded(NULL, (LPARAM) -6 );
    }
    else{
        bSyncCanceled = false;
    }
}

void CuiDlg::OnMenuPush()
{
    CPushSettings wndPushSettings;

    isInSettings = true;
    INT_PTR result;

    result = wndPushSettings.DoModal();

    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}

void CuiDlg::OnMenuSync()
{
    CSyncSettings wndSyncSettings;
    isInSettings = true;
    INT_PTR result = wndSyncSettings.DoModal();
    isInSettings = false;

    refreshSourcePanes();
}

LRESULT CuiDlg::OnStartingSource(WPARAM wParam, LPARAM lParam)
{
    CString s1,s2;

    if (!lstSources.getIsLocked()){
        lockPanes();
    }

    //LOG.debug("OnStartingSource: lparam=%d, currentSource=%d", lParam, currentSource);
    if (currentSource != lParam) {
        lstSources.stopAnim(currentSource);
        currentSource = lParam;
    }

    if(! (sourceBegins & (int)pow(2.0, currentSource)) ){

        // set the bit for the current source to know that for it we have received a Sync Source Ended event
        sourceBegins |= (int) pow(2.0, currentSource);

        CString msg;
        msg.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHECKING);
        msg+=" ";

        switch(lParam)
        {
        case SOURCE_MAIL:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_MAIL); break;
        case SOURCE_CALENDAR:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CALENDAR); break;
            //s1=getStringLang(IDS_LOWER_CALENDAR); break;
        case SOURCE_CONTACTS:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CONTACTS); break;
        case SOURCE_TASKS:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_TASKS); break;
        case SOURCE_BRIEFCASE:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_FILES); break;
        case SOURCE_NOTES:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_NOTES); break;

        default:
            s1 = "";
        }

        msg+=s1;

        lstSources.setText(lParam, msg);
    };

    // if source fails without sending a message that it has finished
    // then we send it ourselves
    if(lstSources.getIsSyncing(currentSource)){
        OnMsgEndingSource(NULL, NULL);
    }

    // if the source fails without sending 'Starting' message
    //s1 = lstSources.getText(currentSource);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONNECTING); s2 = "("; s2+=s1; s2+= ")";
    s1 = lstSources.getText(firstSourceId);
    if(s1.Compare(s2) == 0){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FAILED);
        lstSources.setText(firstSourceId, s1);
    }

    //currentSource = lParam;
    currentItem = 0;
    //LOG.debug("OnStartingSource: firstSourceId=%d", firstSourceId);
    lstSources.stopAnim(firstSourceId);

    lstSources.setIsSyncing(currentSource, true);
    lstSources.SetCurSelItem(lstSources.idToIndex(currentSource));
    lstSources.startAnim(currentSource);

    return 0;
}

// not a WM_APP message handler
void CuiDlg::OnMsgItemSynced(WPARAM wParam, LPARAM lParam, bool fromClient)
{
    int source=-1;
    CString msg("");
    CString s1;

    if(!bSyncStarted)
        return;
    
    if (resetTheItem) {
        resetTheItem = false;
        currentItem = 0;
    }
    currentItem++;

    if( ((wParam > 0)  && (currentSource != wParam)))  // extra check
    {
        resetCurrentItem = true;
        lastItem = 0;        
    };

    CString s2; s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_UPDATES);
            
    if( fromClient ){
        if (messageToWrite == "") {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SENDING);
            s1 += " ";
            s1 += s2;
        } else {
            s1 = messageToWrite;            
        }
        //msg += s1; msg+=" ";
        msg += s1; 
    }
    else{
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECEIVING);
        msg+=s1; msg+=" ";
        msg+= s2;
    }

    source = wParam;
    currentSource = source;
/*
    if(currentSource == SOURCE_MAIL){
        totalItems = nMailsToSync;
    };
*/
    if( (!resetCurrentItem) && (lParam != 0) )  // the server sent more valid noc's for the same source
        currentItem =lastItem+lParam;

    /* // not used because the last word is always "updates"
    switch(source) {
        case SOURCE_MAIL:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_MAIL); break;
        case SOURCE_CALENDAR:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CALENDAR); break;
        case SOURCE_CONTACTS:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CONTACT); break;
        case SOURCE_TASKS:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_TASK); break;
        case SOURCE_BRIEFCASE:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_FILE); break;
        case SOURCE_NOTES:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_NOTE); break;
        default:
            s1 = "";
    } 
    msg+=s1;
    */

   
    if(msg == "")
        goto finally;
    msg+=" ";

    char* v = ltow(currentItem);
    msg+=v;
    delete [] v; v = NULL;

    if( (totalItems != -1) && (currentItem <= totalItems))
    {
        msg+="/";
        char* v = ltow(totalItems);
        msg+=v;
        delete [] v; v = NULL;
    }

    lstSources.setText(source, msg);

   finally:

    return;
}

LRESULT CuiDlg::OnMsgItemSyncedFromServer( WPARAM wParam, LPARAM lParam)
{
    OnMsgItemSynced(wParam,lParam, false);
    return 0;
}

LRESULT CuiDlg::OnMsgItemSyncedFromClient( WPARAM wParam, LPARAM lParam)
{
    OnMsgItemSynced(wParam, lParam, true);
    return 0;
}

/**
* wParam = source id
* lParam = number of items
* Sending message
*/
LRESULT CuiDlg::OnMsgTotalItemsNew(WPARAM wParam, LPARAM lParam) {
 
    resetTheItem   = true;    
    CString s; s.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SENDING_MESSAGE);
    messageToWrite = s;
    return OnMsgTotalItems(wParam, lParam);
}

/**
* wParam = source id
* lParam = number of items
* Sending updates
*/
LRESULT CuiDlg::OnMsgTotalItemsMod(WPARAM wParam, LPARAM lParam) {
 
    resetTheItem   = true;    
    messageToWrite = ""; // substitute with resource string
    return OnMsgTotalItems(wParam, lParam);
}

/**
* wParam = source id
* lParam = number of items
* Not used at the moment
*/
LRESULT CuiDlg::OnMsgTotalItemsDel(WPARAM wParam, LPARAM lParam) {
 
    resetTheItem   = true;    
    messageToWrite = "Deleting"; // substitute with resource string
    return OnMsgTotalItems(wParam, lParam);
}

// TODO: remove unused code
LRESULT CuiDlg::OnMsgTotalItems(WPARAM wParam, LPARAM lParam)
{
    CString msg;
    CString s1;
    bool changeLabel = false;
    int sourceNumber = wParam;
    totalItems = lParam;

    if (!resetTheItem) {
        messageToWrite = "";
    }
    msg.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECEIVING); msg+=" ";

#if 0
    if ( (sourceNumber != -1) && (sourceNumber > 0) )   // wParam > 0, then the modifs are from server
    {
        if (sourceNumber != currentSource)    // a new source from server
        {
            // begin a new source from server, update the progress bar for the previous source
            changeLabel = true;

            if((currentItem < ITEMS_FROM_SERVER) && (currentItem > 0) && (totalItems == -1))
            {
                /* syncSourceTotalServerItems supplied -1, so if the number of synced
                items is less than ITEMS_FROM_SERVER, step the remaining of the progressbar for
                (ITEMS_FROM_SERVER - currentItem-1) items
                */

                currentItem = 0;
                lastItem = 0;
            };

            totalItems = lParam;
            resetCurrentItem = true;
            lastItem = 0;
            currentItem = 0;
            currentSource = sourceNumber;
        }
        else
        {
            /*
            wParam == currentSource, one of more syncSourceTotalServerItems events for
            the same source, or there is only one source to be synced
            (lastSource == -1) for sources from client
            (lastSource > 0) for sources from server, here lastSource is the source code
            */
            if(lastSource < 0)  // (wParam>0) && (lastSource<0) => first source from server
            {
                resetCurrentItem = true;
                totalItems = lParam;
                lastItem = 0;
                currentItem = 0;
                changeLabel = true;
            }
            else
            {
                resetCurrentItem = false;
                lastItem = currentItem;
            }
        };
    } //if wParam < 0
    else if(wParam == -1)
    {
        totalItems = lParam;    // source from client, total items are ok
    }

#endif

    switch(sourceNumber)
    {
    case SOURCE_MAIL:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_MAIL); break;
    case SOURCE_CALENDAR:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CALENDAR); break;
    case SOURCE_CONTACTS:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CONTACTS); break;
    case SOURCE_TASKS:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_TASKS); break;
    case SOURCE_BRIEFCASE:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_NOTES); break;
    case SOURCE_NOTES:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_FILES); break;

    default:
        s1 = "";
    }

    msg+=s1;
    if( (msg!="") &&  changeLabel) // !
    {
        SetDlgItemText(IDC_MAIN_MSG,msg);
        lstSources.setText(wParam, msg);
        if(lstSources.GetNextItem(-1, LVNI_SELECTED ) != wParam)
            lstSources.SetItemState(wParam, LVIS_SELECTED, LVIF_STATE);
    }

/*
    if(wParam == SOURCE_MAIL){
        nMailsToSync+=totalItems;
    }
*/
    return 0;
}


void CuiDlg::CreateMenuBar(const int id){
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = id;
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!\n\n"));
    }
}

LRESULT CuiDlg::OnOKMsg(WPARAM wParam, LPARAM lParam) {
    this->ShowWindow(SW_MINIMIZE);
    return 0;
}

void CuiDlg::OnOK()
{
    this->ShowWindow(SW_MINIMIZE);
}
bool isServerConfigured() {

    ClientSettings* cs = getRegConfig();
    CString s1;
    s1 = cs->getAccessConfig().getSyncURL();
    if(s1.Find(_T("<")) > 0)
        return false;
    else
        return true;

}

// override back key sph
void CuiDlg::OnCancel(){
    // just hide the window
    this->ShowWindow(SW_MINIMIZE);
}


HBRUSH CuiDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor){
    if(pWnd->GetRuntimeClass() == RUNTIME_CLASS(CCustomButton)){
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetBkColor(RGB(255,255,255));
        return (HBRUSH) GetStockObject(HOLLOW_BRUSH);
    }

    HBRUSH hbr = (HBRUSH) CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
    return hbr;
}


void CuiDlg::initSyncControls(){
    CString s1;
    ClientSettings* cs = getRegConfig();
    s1 = cs->getAccessConfig().getSyncURL();

    // add sync sources to list
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL);
    lstSources.addItem(SOURCE_MAIL, s1, AfxGetApp()->LoadIcon(IDI_MAIL));
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONTACTS);
    lstSources.addItem(SOURCE_CONTACTS, s1, AfxGetApp()->LoadIcon(IDI_CONTACTS));
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CALENDAR);
    lstSources.addItem(SOURCE_CALENDAR, s1, AfxGetApp()->LoadIcon(IDI_CALENDAR));
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TASKS);
    lstSources.addItem(SOURCE_TASKS, s1, AfxGetApp()->LoadIcon(IDI_TASKS));

    // Initialize the main logo only if it's a portal build.
    //if(cs->getIsPortal()) {
    //    lstSources.calculateMainLogoPosition();
    //}
    if (!showNoteSource && !showBriefcaseSource) {
        lstSources.calculateMainLogoPosition();
    }

    //if(!cs->getIsPortal()){
    if (showNoteSource) {
        #if defined(WIN32_PLATFORM_PSPC)
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES);
            lstSources.addItem(SOURCE_NOTES, s1, AfxGetApp()->LoadIcon(IDI_NOTES));
        #endif
    }
    if (showBriefcaseSource) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE);
        lstSources.addItem(SOURCE_BRIEFCASE, s1, AfxGetApp()->LoadIcon(IDI_BRIEFCASE));
    }

    refreshSourcePanes();

    lstSources.Invalidate();
}

void CuiDlg::OnContextMenu(CWnd* /*pWnd*/, CPoint point)
{
    // TODO:  handle 2nd event and close the popup
    CPoint mPointCurrent;
    ::GetCursorPos(&mPointCurrent);
    CMenu mPopupMenu;
//    mPopupMenu.LoadMenu(IDR_MENU_CONTEXT);


    CMenu *pContextMenu = mPopupMenu.GetSubMenu(0);
    /*pContextMenu->TrackPopupMenu(TPM_LEFTALIGN |
    TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD |
    TPM_NONOTIFY,
    mPointCurrent.x, mPointCurrent.y, GetActiveWindow(),
    NULL); */

    pContextMenu->TrackPopupMenu(TPM_LEFTALIGN |
        TPM_RETURNCMD | TPM_NONOTIFY,
        point.x, point.y, GetActiveWindow(),
        NULL);
}

bool isToday(SYSTEMTIME time) {
    SYSTEMTIME st;
    GetLocalTime(&st);
    if (time.wYear  == st.wYear   &&
        time.wMonth == st.wMonth &&
        time.wDay   == st.wDay) {

        return true;
    }
    else {
        return false;
    }
}


wstring CuiDlg::getFormattedDateAndTime(long lastSync) {

    SYSTEMTIME timeDest; wchar_t data[80]; 
    wchar_t formatDate[80]; wchar_t formatTime[80];
    wstring result;

    CTime timeSyncContacts(lastSync);
    
    timeSyncContacts.GetAsSystemTime(timeDest);
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLONGDATE, data, 80);

    if (isToday(timeDest)) {
        CString s1;
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TODAY);
        wcscpy(formatDate, s1);
    } else {
        result = reformatDate(data);   
    GetDateFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, result.c_str(), formatDate, 80); 
    }
    
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, data, 80);
    result = reformatTime(data);   

    GetTimeFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, result.c_str(), formatTime, 80); 
    
    result = formatDate;
    result += TEXT(" ");
    result += formatTime;    
    
    return result;
}


void CuiDlg::refreshSourcePane( int id ){
    CString s1;
    ClientSettings* cs = getRegConfig();
    bool enable;
    unsigned long lastSync=0;
    wstring date_time;

    if(id == SOURCE_MAIL){        
        lastSync=0;

        enable = strcmp(cs->getConfigSourcesParameter("mail","sync"), "none") != 0;
        if(!doesFunambolAccountExist()) {
            enable = false;
        }        
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("mail");

        if(syncSourceMailState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
        }
        else if( syncSourceMailState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
        }
        else {           
            date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
        }
            lstSources.setText(SOURCE_MAIL, s1);
    }

    if(id == SOURCE_CONTACTS){
       lastSync=0;

        enable =  strcmp(cs->getConfigSourcesParameter("contact","sync"), "none") != 0;
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("contact");

        if(syncSourceContactState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
        }
        else if( syncSourceContactState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
        }
        else
        {           
            wstring date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
        }
         lstSources.setText(SOURCE_CONTACTS, s1);
    }


    if(id == SOURCE_CALENDAR){
        lastSync=0;

        enable =  strcmp(cs->getConfigSourcesParameter("calendar","sync"), "none") != 0;
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("calendar");

        if(syncSourceCalendarState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
        }
        else if( syncSourceCalendarState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
        }
        else
        {            
            date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
        }
        lstSources.setText(SOURCE_CALENDAR, s1);
    }

    if(id == SOURCE_TASKS){
        lastSync=0;

        enable =  strcmp(cs->getConfigSourcesParameter("task","sync"), "none") != 0;
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("task");

        if(syncSourceTaskState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
        }
        else if( syncSourceTaskState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
        }
        else
        {           
            date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
        }
        lstSources.setText(SOURCE_TASKS, s1);
    }

    if(id == SOURCE_BRIEFCASE){
        lastSync=0;

        enable =  strcmp(cs->getConfigSourcesParameter("briefcase","sync"), "none") != 0;
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("briefcase");

        if(syncSourceBriefcaseState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
        }
        else if( syncSourceBriefcaseState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
        }
        else
        {           
            date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
        }
        lstSources.setText(SOURCE_BRIEFCASE, s1);
    }

#if defined(WIN32_PLATFORM_PSPC)
    if(id == SOURCE_NOTES){
        lastSync=0;

        enable =  strcmp(cs->getConfigSourcesParameter("note","sync"), "none") != 0;
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("note");

        if(syncSourceNoteState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
        }
        else if( syncSourceNoteState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
        }
        else
        {           
            date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
        }
        lstSources.setText(SOURCE_NOTES, s1);
    }
#endif

    EnableSyncButton(isSyncEnabled());
    lstSources.resetSourceIcon(id);
}

void CuiDlg::refreshSourcePanes(){
    refreshSourcePane(SOURCE_MAIL);
    refreshSourcePane(SOURCE_CONTACTS);
    refreshSourcePane(SOURCE_CALENDAR);
    refreshSourcePane(SOURCE_TASKS);

    //ClientSettings* cs = getRegConfig();
    //if(! cs->getIsPortal()){
    if (showNoteSource) {
        #if defined(WIN32_PLATFORM_PSPC)
            refreshSourcePane(SOURCE_NOTES);
        #endif
    }
    if (showBriefcaseSource) {
        refreshSourcePane(SOURCE_BRIEFCASE);
    }

    lstSources.resetSourceIcons();
}


void CuiDlg::SelectSource( int index ){
 if(index < 0)
    return;

 try{
    CString s1,s2;
    bool refreshPane = false;
    ClientSettings* cs = getRegConfig();

    int id = lstSources.indexToId(index);

    if(id == SOURCE_MAIL){
        if(!doesFunambolAccountExist()){
            refreshSourcePane(SOURCE_MAIL);
        }
       else{

        // check if source is enabled
        if( ( strcmp(cs->getConfigSourcesParameter("mail","sync"), "none") != 0) ){
            // start sync with this source
            lstSources.setState(SOURCE_MAIL, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_MAIL);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual mail"));
        }
        else{
            if(!doesFunambolAccountExist())
                return;
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL); s1+=s2;
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("mail","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_MAIL);
                getRegConfig()->saveDirty();
            }
        }
      }
    };

    if(id == SOURCE_CONTACTS){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("contact","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_CONTACTS, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_CONTACTS);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual contact"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONTACTS); s1+=s2;
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
              MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("contact","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_CONTACTS);
                getRegConfig()->saveDirty();
            }
        }
    };

    if(id == SOURCE_CALENDAR){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("calendar","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_CALENDAR, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_CALENDAR);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual calendar"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CALENDAR); s1+=s2;
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("calendar","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_CALENDAR);
                getRegConfig()->saveDirty();
            }
        }
    };

    if(id == SOURCE_TASKS){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("task","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_TASKS, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_TASKS);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual task"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TASKS); s1+=s2;
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("task","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_TASKS);
                getRegConfig()->saveDirty();
            }
        }
    };

    if(id == SOURCE_BRIEFCASE){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("briefcase","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_BRIEFCASE, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_BRIEFCASE);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual briefcase"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE); s1+=s2;
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("briefcase","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_BRIEFCASE);
                getRegConfig()->saveDirty();
            }
        }
    };

#if defined(WIN32_PLATFORM_PSPC)
    if(id == SOURCE_NOTES){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("note","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_NOTES, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_NOTES);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual note"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES); s1+=s2;
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("note","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_NOTES);
                getRegConfig()->saveDirty();
            }
        }
    };
#endif


    if(refreshPane){
        refreshSourcePane(id);
    }

  }
  catch(...){
      LOG.error("uiDlg: error while selecting source");
  }

  lstSources.UpdateWindow();
}

void CuiDlg::StartSync(CString param){
    CString s1;

    int check  = checkStartSync();
    int ret;

    bSyncCanceled = false;

    // check credentials
    ClientSettings* cs = getRegConfig();
    CString user;
    CString password;
    user = cs->getAccessConfig().getUsername();
    password = cs->getAccessConfig().getPassword();

    if( user.IsEmpty() || password.IsEmpty()){
        HwndFunctions::closePreviousMsgBox();
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_ENTER_CREDENTIALS);
        TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT), MB_OK | MB_ICONHAND, INFINITE);
        OnStartsyncEnded(0,0); // signal sync ended
        OnMenuComm();
        return; // we return from this so we don't lock the panes
    }
    else
      if (!isServerConfigured()) {
        HwndFunctions::closePreviousMsgBox();
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_SET_FUNAMBOL_URL);
        TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT), MB_OK | MB_ICONHAND, INFINITE);
        OnStartsyncEnded(0,0); // signal sync ended
        OnMenuComm();
        return; // we return from this so we don't lock the panes

    } else if(check == 0) {
        // maybe the user deleted the mail account
        // so we refresh the mail and if there is no source to be synced, we return
        if(! isSyncEnabled())
            return; // no sync will be performed

        ret = startProgram(SYNCAPP, param);	 // start sync process
        if ( (ret == 0) || (ret == -5) )
        {
            HwndFunctions::closePreviousMsgBox();
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_STARTSYNC_NOT_FOUND);
            TimedMessageBox(GetSafeHwnd(), s1,getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT), MB_OK | MB_ICONHAND, INFINITE);
            OnStartsyncEnded(0,0); // signal sync ended
            return; // we return from this so we don't lock the panes
        }
        else
        {
            dlgCommandBar.nToolBarId = IDM_STOP;
            #if defined WIN32_PLATFORM_WFSP
            #if (_WIN32_WCE < 0x500)
                CreateMenuBar(IDR_MENUBAR_STOP);
            #endif
            #endif

            if(!SHCreateMenuBar(&dlgCommandBar)){
                TRACE(_T("Cannot create command bar!"));
                return ;
            }
        }
    };

    lstSources.SetItemState(-1, LVIS_SELECTED, LVIF_STATE);
    // lock panes so they cannot be selected
    lockPanes();
}

void CuiDlg::lockPanes(){
    lstSources.lockList();
}

void CuiDlg::unlockPanes(){
    lstSources.unlockList();
}

BOOL CuiDlg::PreTranslateMessage(MSG* pMsg){

#if defined(WIN32_PLATFORM_PSPC)
    // This msg is received to close the plugin from some task-manager applications
    // like "X-Button" (included in HTC Touch).
    if(pMsg->message == WM_COMMAND) {
        if (pMsg->wParam == 0x0001) {
            OnClose();
            return TRUE;
        }
    }
#endif

    bool bProcessed =false;
    if( ( (pMsg->message == WM_KEYDOWN) || (pMsg->message == WM_KEYUP) ||
        ((pMsg->message == WM_LBUTTONDOWN) && (pMsg->hwnd == lstSources.GetSafeHwnd())) )  // click in source list
        && (lstSources.getIsLocked()) ){
            bProcessed = true;
            return TRUE;
    };

    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            bProcessed = true;
            SelectSource(lstSources.GetNextItem(-1, LVNI_SELECTED ));
        }
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

void CuiDlg::CancelSync(){
    OnMenuStop();
}

/**
* Refresh all the canceled states, so it set all the global variables.
* don't put if {} else if {}
*/
void CuiDlg::refreshCanceledState(){
    if(lstSources.getState(SOURCE_MAIL) == ITEM_STATE_TO_SYNC)
        syncSourceMailState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_CONTACTS) == ITEM_STATE_TO_SYNC)
        syncSourceContactState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_CALENDAR) == ITEM_STATE_TO_SYNC)
        syncSourceCalendarState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_TASKS) == ITEM_STATE_TO_SYNC)
        syncSourceTaskState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_BRIEFCASE) == ITEM_STATE_TO_SYNC)
        syncSourceBriefcaseState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_NOTES) == ITEM_STATE_TO_SYNC)
        syncSourceNoteState = SYNCSOURCE_STATE_CANCELED;
}

// source id as wParam, state as lParam
LRESULT CuiDlg::OnMsgSourceState( WPARAM wParam, LPARAM lParam){
    if (wParam == SOURCE_MAIL) {
        syncSourceMailState = lParam;
    } else if (wParam == SOURCE_CONTACTS) {
        syncSourceContactState = lParam;
    } else if (wParam == SOURCE_CALENDAR) {
        syncSourceCalendarState = lParam;
    } else if (wParam == SOURCE_TASKS) {
        syncSourceTaskState = lParam;
    } else if (wParam == SOURCE_BRIEFCASE) {
        syncSourceBriefcaseState = lParam;
    } else if (wParam == SOURCE_NOTES) {
        syncSourceNoteState = lParam;
    }
    return 0;
}

void CuiDlg::setFirstSourceToSync(){
    CArray<int> sources;
    bool found = false;
    sources.Add(SOURCE_MAIL); sources.Add(SOURCE_CONTACTS);
    sources.Add(SOURCE_CALENDAR); sources.Add(SOURCE_TASKS);
    #if defined(WIN32_PLATFORM_PSPC)
        sources.Add(SOURCE_NOTES);
    #endif
    sources.Add(SOURCE_BRIEFCASE);
    for(int i=0; (i<sources.GetCount()) && (!found); i++){
        if(lstSources.isEnabledItem(sources[i])){
            firstSourceId = sources[i];
            found = true;
        }
    }

}

LRESULT CuiDlg::OnMsgEndingSource( WPARAM , LPARAM ){
    // sync source ended event
    CString s1;
    lastItem = currentItem;
    currentItem = 0;

    if(sourceEnds & (int)pow(2.0, currentSource) ){
        // done receiving
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DONE_RECEIVING);
        lstSources.setText(currentSource, s1);
    }
    else{
        // done sending
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DONE_SENDING);
        lstSources.setText(currentSource, s1);
        // set the bit for the current source to know that for it we have received a Sync Source Ended event
        sourceEnds |= (int) pow(2.0, currentSource);
        // reset nr of mails
        nMailsToSync =0;
    }

    lstSources.setIsSyncing(currentSource, false);
    return 0;
}

//
LRESULT CuiDlg::OnMsgConnecting( WPARAM , LPARAM ){
    CString s1;

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONNECTING);
    if(firstSourceId == -1){
        setFirstSourceToSync();
    }

    // check if we have a valid first source
    if(firstSourceId != -1){
        lstSources.SetCurSelItem(lstSources.idToIndex(firstSourceId));
        lstSources.setText(firstSourceId, s1);
        lstSources.startAnim(firstSourceId);
    }

    return 0;
}

// 'Ending...' sync message
LRESULT CuiDlg::OnMsgEndingSync( WPARAM , LPARAM ){
    CString s1;

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ENDING);
    lstSources.setText(currentSource, s1);
    lstSources.startAnim(currentSource);

    return 0;
}

LRESULT CuiDlg::OnMsgPopup(WPARAM wParam, LPARAM lParam){

    CString button1;
    CString button2;
    CString button3;
    CString swap;
    CString msg;
    CString buttonval;
    msg.LoadString(getLocalizationUtils()->getLocaleResource(), lParam);
    buttonval.LoadString(getLocalizationUtils()->getLocaleResource(), (UINT)wParam);
    int b1 =buttonval.Find(L"*");
    int b2 =buttonval.Find(L"*",b1+1);
    if ( b1 == -1 && b2 == -1){
        button1 = buttonval;
        button2 = L"";
    }else if ( b2 == -1 ){ //just 2 buttons
        button1 = buttonval.Left(b1);
        button2 = buttonval.Right(buttonval.GetLength() - b1 -1);
        button3 = L"";
    } else { //3 buttons
        button1 = buttonval.Left(b1);
        swap = buttonval.Right(buttonval.GetLength() - b1 -1);
        int s = swap.Find(L"*");
        button3 = swap.Right(swap.GetLength() - s -1);
        button2 = swap.Left(s);
    }

    return CMessageBox(msg, button1, button2, button3);
    //return 1;

}

void CuiDlg::OnItemchangedList1(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    int id = lstSources.indexToId(pNMListView->iItem);
    // if item is disabled we do not change icon

    if(lstSources.isEnabledItem(id)){
        // Only interested in state changes
        if (pNMListView->uChanged & LVIF_STATE)
            if (pNMListView->uNewState & LVIS_FOCUSED)
            {
                // new selection
                lstSources.setIcon(id, AfxGetApp()->LoadIcon(IDI_ARROWS22A));
            }
            else if (pNMListView->uOldState & LVIS_FOCUSED)
            {
                // deselection
                lstSources.resetSourceIcon(id);
            }
    }

    lstSources.InvalidatePane(id);

   *pResult = 0;
}

// context menu handlers
void CuiDlg::OnContextMenuSync(){
    // TODO: implementation
}

void CuiDlg::OnContextMenuSettings(){
    // TODO: implementation
}

void CuiDlg::OnToolsDeviceinfo()
{
    CDeviceInfo wndDeviceInfo;
    isInSettings = true;
    INT_PTR result = wndDeviceInfo.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}


void CuiDlg::EnableSyncButton(bool bEnable){
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB)
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, ID_SYNC, (LPARAM)&tbi);

    if(bEnable){
        tbi.fsState = TBSTATE_ENABLED;
    }
    else{
        tbi.fsState &= ~TBSTATE_ENABLED;
    }

    ::SendMessage (hwndMB, TB_SETBUTTONINFO, ID_SYNC, (LPARAM)&tbi);
}

bool CuiDlg::isSyncEnabled(){
    bool enabled = false;
    enabled = lstSources.isEnabledItem(SOURCE_MAIL) ||
              lstSources.isEnabledItem(SOURCE_CONTACTS) ||
              lstSources.isEnabledItem(SOURCE_CALENDAR) ||
              lstSources.isEnabledItem(SOURCE_TASKS) ||
              lstSources.isEnabledItem(SOURCE_BRIEFCASE);

    #if defined(WIN32_PLATFORM_PSPC)
        enabled |= lstSources.isEnabledItem(SOURCE_NOTES);
    #endif

    return enabled;
}

void CuiDlg::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{

    CDialog::OnActivate(nState, pWndOther, bMinimized);
    if(openUp && !bSyncStarted){
        refreshSourcePanes();
    }

}

void trim(wstring& str)
{
  wstring::size_type pos = str.find_last_not_of(' ');
  if(pos != string::npos) {
    str.erase(pos + 1);
    pos = str.find_first_not_of(' ');
    if(pos != string::npos) str.erase(0, pos);
  }
  else str.erase(str.begin(), str.end());
}

wstring reformatDate(wstring date) {
    
    int found = 0;
    // remove "dddd" or "dddd," or "dddd, "
    if ((found = date.find(TEXT("dddd, "))) != wstring::npos) {
        date.replace(found, 6, TEXT(""));
    } else if ((found = date.find(TEXT("dddd,"))) != wstring::npos) {
        date.replace(found, 5, TEXT(""));
    }else if ((found = date.find(TEXT("dddd"))) != wstring::npos) {
        date.replace(found, 4, TEXT(""));
    }
    
    trim(date);
    // change full month in abbreviation (e.g. September into Sept)    
    if ((found = date.find(TEXT("MMMM, "))) != wstring::npos) {
        date.replace(found, 6, TEXT("MMM, "));
    } else if ((found = date.find(TEXT("MMMM,"))) != wstring::npos) {
        date.replace(found, 5, TEXT("MMM,"));
    } if ((found = date.find(TEXT("MMMM"))) != wstring::npos) {
        date.replace(found, 4, TEXT("MMM"));
    }
    
    trim(date);
    // remove year
    if (((found = date.find(TEXT("yyyy, "))) != wstring::npos) || 
        ((found = date.find(TEXT(", yyyy"))) != wstring::npos)) {
        date.replace(found, 6, TEXT(""));
    } else if (((found = date.find(TEXT("yyyy,"))) != wstring::npos) ||
               ((found = date.find(TEXT(",yyyy"))) != wstring::npos)) {
        date.replace(found, 5, TEXT(""));
    } if ((found = date.find(TEXT("yyyy"))) != wstring::npos) {
        date.replace(found, 4, TEXT(""));
    }
    trim(date);
    
    return date;

}

wstring reformatTime(wstring time) {
    
    int found = 0;
    // remove ":ss" or ":ss," or "ss"
    if ((found = time.find(TEXT(":ss,"))) != wstring::npos ||
        (found = time.find(TEXT(".ss,"))) != wstring::npos) {
        time.replace(found, 4, TEXT(""));
    } else if ((found = time.find(TEXT(":ss"))) != wstring::npos ||
               (found = time.find(TEXT(".ss"))) != wstring::npos) {
        time.replace(found, 3, TEXT(""));
    }else if ((found = time.find(TEXT("ss"))) != wstring::npos) {              
        time.replace(found, 2, TEXT(""));
    }
    
    trim(time);   
    
    return time;

}
