/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
#pragma once

#include "ui.h"
#include "afxwin.h"
#include "customization.h"

// CCommunicationSettings dialog
class CCommunicationSettings : public CDialog
{
   DECLARE_DYNAMIC(CCommunicationSettings)

public:
   CCommunicationSettings(CWnd* pParent = NULL);   // standard constructor
   virtual ~CCommunicationSettings();
   void showMessage(CString message);

// Dialog Data
    enum { IDD = IDD_UI_COMM };

protected:
   SHMENUBARINFO dlgCommandBar;
   CEdit editUrl;
   CEdit editUser;
   CEdit editPassword;
   CFont fontBold;
   CFont fontAlert;
   CString alertMessage;

   /**
    * checks if the input if entered in the dialog is valid
    * @param url : Funambol server url
    * @param username
    * @param password
    * @return  : 0 if all ok, an error code otherwise
    */
   int checkInputParameters(CString url, CString username, CString password);
   
   /**
    * It show the message returned by the isInputValid function
    *
    */
   void showNotValidInputMessage(int code);

   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

   /**
    *  called on dialog init
    */
    virtual BOOL OnInitDialog();

    /**
     * Starts a sync of configuration.
     * UI is disabled, and the sync is started in a separate thread (configWorker).
     * UI will be re-enabled at the end of the thread, sending a ID_MYMSG_NOTIF_SYNC_ENDED message.
     * @return  0 if sync started normally
     *          1 if sync not started, due to errors
     */
    int startConfigSync();

    /**
     * Used to enable/disable UI objects (edit boxes, buttons).
     * @param enable  set to true to enable objects, false to diable objects
     */ 
    void enableUIObjects(bool enable);

    /**
     * It indicates if the form is called by the mainUI->menu->account
     * or it comes from the first time
     */
    bool isFromMainMenu;


 DECLARE_MESSAGE_MAP()

public:
   /**
    * called when saving changes
    */
   afx_msg void OnOK();
   afx_msg LRESULT OnMsgConfigSyncEnded(WPARAM , LPARAM);
   afx_msg LRESULT OnMsgClearAuthenticationLabel(WPARAM , LPARAM);
   afx_msg void OnCancel();

   HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
    
   void changeMailSettingsInfo(CString username);
   #if defined(WIN32_PLATFORM_WFSP)
	LRESULT OnHotkey(WPARAM wParam, LPARAM lParam);
   #endif

#if defined(_DEVICE_RESOLUTION_AWARE)
   afx_msg void OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/);
#endif
    
   void setIsFromMainMenu(bool v) { isFromMainMenu = v; }

   CString currentUsername;
   CString currentPassword;

};
