/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#pragma once

#include "ui.h"


CString getActiveSyncMethod();


// CSyncMethod dialog
class CSyncMethod : public CDialog
{
    DECLARE_DYNAMIC(CSyncMethod)

private:
    HINSTANCE localResource;


protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    virtual BOOL OnInitDialog();

    #if defined(WIN32_PLATFORM_PSPC)
        CComboBox lstSyncMethod;
        CComboBox lstPolling;
    #endif

    SHMENUBARINFO dlgCommandBar;

    #if defined(WIN32_PLATFORM_WFSP)
        CListBox lstSyncMethod;
        CListBox lstPolling;
        CSpinButtonCtrl spinSyncMethod;
        CSpinButtonCtrl spinPolling;
    #endif

    CButton checkEncryption;
    CButton checkCradle;
    CButton checkEnableCompression;
    CFont fontBold;

    /// store boolean value, indicates if cradle was enabled when entering the dialog
    bool cradleInitiallyEnabled;
    
    /// store what was the initial selection in the scheduler settings when entering the dialog
    int initialSchedulerSelection;

    /// store what was the initial selection in the lstSyncMethod list when entering the dialog
    int initialSyncMethodSelection;

    /// Used to know if we have to start push service, at the end of the config sync.
    bool toStartPush;

    /// Used to know if we have to stop push service, at the end of the config sync.
    bool toStopPush;

    /**
     * Starts a sync of configuration.
     * UI is disabled, and the sync is started in a separate thread (configWorker).
     * UI will be re-enabled at the end of the thread, sending a ID_MYMSG_NOTIF_SYNC_ENDED message.
     * @return  0 if sync started normally
     *          1 if sync not started, or aborted (syncmode = none)
     */
    int startConfigSync();

    /**
     * Used to enable/disable UI objects (edit boxes, buttons).
     * @param enable  set to true to enable objects, false to diable objects
     */ 
    void enableUIObjects(bool enable);


   DECLARE_MESSAGE_MAP()
    afx_msg void    OnOK();
    afx_msg LRESULT OnMsgConfigSyncEnded(WPARAM , LPARAM);
    afx_msg void    OnCbnSelchangeSchedulerSyncmethod();
    HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);

#if defined(_DEVICE_RESOLUTION_AWARE)
   afx_msg void     OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/);
#endif


public:
    CSyncMethod(CWnd* pParent = NULL);   // standard constructor
    virtual ~CSyncMethod();

    enum { IDD = IDD_UI_SYNC_METHOD };
    
};
