/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include "vocl/WinItem.h"
#include "pim/ClientSettings.h"
#include "spds/constants.h"
#include "spds/SyncReport.h"
#include "notify/checknet.h"
#include "customization.h"
#include "base/stringUtils.h"
#include "pim/ConfigParams.h"

#define PROPERTY_NOTIFICATION_PORT      ("pushPort")
#define PROPERTY_SMS_WAP_PUSH           ("sms")
#define PROPERTY_SERVER_NOTIFIED        ("svrNotified")
#define PROPERTY_SOURCE_USE_SIF         ("useSIF")
#define PROPERTY_SOURCE_DIR             ("dir")
#define PROPERTY_PATH                   ("path")
#define PROPERTY_SCHEDULE               ("schedule")
#define PROPERTY_CRADLE_NOTIFICATION    ("cradleNotif")
#define PROPERTY_CLIENT_PUSH_INTERVAL   ("clientPushInterval")
#define PROPERTY_EMAIL_FILTER           ("emailFilter")
#define PROPERTY_LAST_EMAIL_ADDRESS     ("lastEmailAddress")
#define PROPERTY_LAST_EMAIL_VISIBLENAME ("lastEmailVisibleName")
#define PROPERTY_CONFIGSYNC_DIRTY       ("configSyncDirty")


// to decide if the plug-in version is for portal or not
#define PROPERTY_IS_PORTAL              ("isPortal")
#define PROPERTY_ASK_USER               ("askUser")
#define PROPERTY_TO_SYNC_CONFIGURATION  ("toSyncConfiguration")  // 1 to sync, 0 don't sync

/**
* For the Update process
*/

#define CONTEXT_UPDATE              "/Update"
#define UPDATE_VERSION              "version"
#define UPDATE_MANDATORY            "mandatory"
#define UPDATE_RELEASE_DATE         "releaseDate"
#define UPDATE_URL_UPDATE           "urlUpdate"
#define UPDATE_URL_COMMENT          "urlComment"
#define UPDATE_SIZE                 "size"
#define UPDATE_LAST_CHECK_UPDATE    "lastCheckUpdate"
#define UPDATE_INT_CHECK_UP_HTTP    "intervalCheckUpdateHttp"
#define UPDATE_INT_REMIND           "intervalRemind"
#define UPDATE_SKIPPED              "skipped"
#define UPDATE_LAST_UPDATE          "lastUpdated"
#define UPDATE_LATER                "later"
#define UPDATE_NOW                  "now"
#define UPDATE_IS_AN_UPDATE_AVAILABLE "isUpdateAvailable"

/**
* For branding parameters. It is a context under Funambol/SyncClient
* The logic is boolean to choose to SHOW the corrispondend property (field, label...)
*/

#define CONTEXT_BRAND               "/brand"
#define BRAND_URL_FIELD             "showUrlField"
#define BRAND_NOTE_SOURCE           "showNoteSource"
#define BRAND_BRIEFCASE_SOURCE      "showBriefcaseSource"
#define BRAND_ADV_SETTING_MENU      "showAdvSettingMenu"


//
// Init static pointer.
//
ClientSettings* ClientSettings::pinstance = NULL;

/*
* Method to get the sole instance
*/
ClientSettings* ClientSettings::getInstance() {
    if (pinstance == NULL) {
        // TODO: use #define
        pinstance = new ClientSettings(ROOT_CONTEXT);
        pinstance->read();
    }
    return pinstance;
}

ClientSettings::ClientSettings(const char* application_uri)
    : DMTClientConfig(application_uri), updateParams(0) {

        winSourceConfigs      = NULL;
        winSourceConfigsCount = 0;
        dirty                 = FALSE;
        mailssconfig          = NULL;
        configSyncDirty       = 0;
}

ClientSettings::~ClientSettings() {
    if (winSourceConfigs) {
        delete [] winSourceConfigs;
        winSourceConfigs = NULL;
    }

    if (mailssconfig) {
        delete mailssconfig;
        mailssconfig = NULL;
    }
    pinstance = NULL;
}

void ClientSettings::setIp(string extip) {
    ipaddress = extip;
}

string ClientSettings::getIp() {
    refreshIP();
    return ipaddress;
}

void ClientSettings::setPush(string extpush) {
    push = extpush;
}
string ClientSettings::getPush() {
    return push;
}

void ClientSettings::setEmailFilter(string filter) {
    emailFilter = filter;
}
string ClientSettings::getEmailFilter() {
    return emailFilter;
}

void ClientSettings::setPushPost(string extpushPort) {
    pushPort = extpushPort;
}
string ClientSettings::getPushPort() {
    return pushPort;
}

void ClientSettings::setSms(string extsms){
    sms = extsms;
}
string ClientSettings::getSms() {
    return sms;
}

void ClientSettings::setPolling(string extpolling) {
    polling = extpolling;
}

string ClientSettings::getPolling() {
    return polling;
}

void ClientSettings::setSvrNotified(string extsvrNotified) {
    svrNotified = extsvrNotified;
}
string ClientSettings::getSvrNotified() {
    return svrNotified;
}

void ClientSettings::setPath(string extpath) {
    path = extpath;
}
string ClientSettings::getPath() {
    return path;
}

string ClientSettings::getAppPath(){
    return getPath();
}

string ClientSettings::getClientPushInterval() {
    return clientPushInterval;
}

void ClientSettings::setClientPushInterval(string extclientPushInterval) {
    clientPushInterval = extclientPushInterval;
}

void ClientSettings::setToSyncConfiguration(string exttoSyncConfiguration) {
    toSyncConfiguration = exttoSyncConfiguration;
}
string ClientSettings::getToSyncConfiguration() {
    return toSyncConfiguration;
}

// for the sources

// TODO: to be removed all common source-related properties
// when they will be no longer used in the code, use instead the methods from WindowsSyncSourceConfig
void ClientSettings::setDir(string extdir) {
    dir = extdir;
}

string  ClientSettings::getDir() {
    return dir;
}

void ClientSettings::setIsPortal(string extportal) {
    isPortal = extportal;
}
bool ClientSettings::getIsPortal() {
    return isPortal == "1" ? true : false;
}

void ClientSettings::setCradleNotification(string v) {
    cradleNotif = v;
}
string ClientSettings::getCradleNotification() {
    return cradleNotif;
}

void ClientSettings::setAskUser(string extaskUser){
    askUser = extaskUser;
}
string ClientSettings::getAskUser() {
    return askUser;
}


StringBuffer ClientSettings::encodePassword(const char* password) {
    
    StringBuffer buffer("");
    if (password && strlen(password) > 0) {
        char* encoded = encryptData(password, PASS_KEY);        
        buffer = encoded;        
    }        
    return buffer;   
}

char* ClientSettings::decodePassword(const char* password) {
    
    char* decripted = NULL;
    if (password && strlen(password) > 0) {
         decripted = decryptData(password, PASS_KEY); 
    }    

    if (decripted == NULL || getLastErrorCode() == 801) {
        decripted = new char[1];
        decripted[0] = 0;
        setError(ERR_NONE, getLastErrorMsg());
    }
    return decripted;       
}



/**
 * Read the custom parameters that are in the SyncClient node.
 *
 */
bool ClientSettings::readConfig() {

    bool ret = FALSE;
    LOG.debug("ClientSettings: readConfig");
    //
    // Reading syncml node
    //
    if (!sourcesNode) {
        if (!open()) {
            return FALSE;
        }
    }

    ManagementNode* node;

    // read parameter of the client only
    node = dmt->readManagementNode(rootContext);
    if (node) {
        char* tmp;
        string t;
        tmp = node->readPropertyValue(PROPERTY_PUSH_NOTIFICATION);
        t = tmp;
        setPush(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_NOTIFICATION_PORT);
        t = tmp;
        setPushPost(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_SMS_WAP_PUSH);
        t = tmp;
        setSms(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_POLLING_NOTIFICATION);
        t = tmp;
        setPolling(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_SERVER_NOTIFIED);
        t = tmp;
        setSvrNotified(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_PATH);
        t = tmp;
        setPath(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_IS_PORTAL);
        t = tmp;
        setIsPortal(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_CRADLE_NOTIFICATION);
        t = tmp;
        setCradleNotification(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_ASK_USER);
        t = tmp;
        setAskUser(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_CLIENT_PUSH_INTERVAL);
        t = tmp;
        setClientPushInterval(t);
        delete [] tmp;
        
        tmp = node->readPropertyValue(PROPERTY_TO_SYNC_CONFIGURATION);
        t = tmp;
        setToSyncConfiguration(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_EMAIL_FILTER);
        t = tmp;
        setEmailFilter(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_LAST_EMAIL_ADDRESS);
        t = tmp;
        setLastEmailAdress(t);
        delete [] tmp;
        
        tmp = node->readPropertyValue(PROPERTY_LAST_EMAIL_VISIBLENAME);
        t = tmp;
        setLastEmailVisibleName(t);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_CONFIGSYNC_DIRTY);
        if (strlen(tmp)) {
            configSyncDirty = atoi(tmp);
        }
        delete [] tmp;

        delete node;
        node = NULL;
    }

    return ret;
}

/**
 * It saves only the parameter that can be changed by the configuration
 * in the different Acc, Ext, Conn DevInfo nodes
 * username/password in Auth
 * syncURL in Conn
 * log/encryption in Ext
 */
bool ClientSettings::saveConfig() {

    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    if (!sourcesNode) {
        open();
    }

    char syncMLContext[DIM_MANAGEMENT_PATH];
    char* fn = syncMLNode->createFullName();
    sprintf(syncMLContext, "%s", fn);
    delete [] fn;

    //
    // Auth properties
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_AUTH);
    node = dmt->readManagementNode(nodeName);
    if (node) {

        node->setPropertyValue(PROPERTY_USERNAME, accessConfig.getUsername());
        //node->setPropertyValue(PROPERTY_PASSWORD, accessConfig.getPassword());

        // modification of the password here        
        StringBuffer buffer = encodePassword(accessConfig.getPassword());
        if (buffer == NULL) {
            buffer = "";
        }
        node->setPropertyValue(PROPERTY_PASSWORD, buffer.c_str());

        delete node;
        node = NULL;
    }

    //
    // DevInfo properties
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_DEV_INFO);
    node = dmt->readManagementNode(nodeName);
    if (node) {

        node->setPropertyValue(PROPERTY_DEVICE_ID, deviceConfig.getDevID());

        delete node;
        node = NULL;
    }


    //
    // Conn properties
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_CONN);
    node = dmt->readManagementNode(nodeName);
    if (node) {
        node->setPropertyValue(PROPERTY_SYNC_URL, accessConfig.getSyncURL());
        node->setPropertyValue(PROPERTY_ENABLE_COMPRESSION,accessConfig.getCompression() ? "1": "0" );

        delete node;
        node = NULL;
    }

    //
    // Ext properties (other misc props)
    //
    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_EXT);
    node = dmt->readManagementNode(nodeName);
    if (node) {
        char buf[512];

        sprintf(buf, "%lu", deviceConfig.getLogLevel());
        node->setPropertyValue(PROPERTY_LOG_LEVEL, buf);
        node->setPropertyValue(PROPERTY_DEVINF_HASH, deviceConfig.getDevInfHash());

        delete node;
        node = NULL;
    }

    // read parameter of the client only
    node = dmt->readManagementNode(rootContext);
    if (node) {
        node->setPropertyValue(PROPERTY_PUSH_NOTIFICATION,     getPush().data());
        node->setPropertyValue(PROPERTY_NOTIFICATION_PORT,     getPushPort().data());
        node->setPropertyValue(PROPERTY_SMS_WAP_PUSH,          getSms().data());
        node->setPropertyValue(PROPERTY_POLLING_NOTIFICATION,  getPolling().data());
        node->setPropertyValue(PROPERTY_SERVER_NOTIFIED,       getSvrNotified().data());
        node->setPropertyValue(PROPERTY_PATH,                  getPath().data());
        node->setPropertyValue(PROPERTY_CRADLE_NOTIFICATION,   getCradleNotification().data());
        node->setPropertyValue(PROPERTY_ASK_USER,              getAskUser().data());
        node->setPropertyValue(PROPERTY_CLIENT_PUSH_INTERVAL,  getClientPushInterval().data());
        node->setPropertyValue(PROPERTY_TO_SYNC_CONFIGURATION, getToSyncConfiguration().data());
        node->setPropertyValue(PROPERTY_EMAIL_FILTER,          getEmailFilter().data());
        node->setPropertyValue(PROPERTY_LAST_EMAIL_ADDRESS,    getLastEmailAdress().data());
        node->setPropertyValue(PROPERTY_LAST_EMAIL_VISIBLENAME,getLastEmailVisibleName().data());

        // Used by ConfigSyncSource during config sync.
        StringBuffer val;
        val.sprintf("%d", getConfigSyncDirty());
        node->setPropertyValue(PROPERTY_CONFIGSYNC_DIRTY,      val.c_str());

        delete node;
        node = NULL;
    }

    close();
    return TRUE;

}

const char* ClientSettings::getConfigSourcesParameter(const char* sourceName, const char* parameter) {

    for (unsigned int i=0; i<winSourceConfigsCount; ++i) {
        if (strcmp(winSourceConfigs[i].getName(), sourceName) == 0) {
            if (strcmp(parameter, "sync") == 0) {
                return winSourceConfigs[i].getSync();
            }
            else if (strcmp(parameter, "encryption") == 0) {
                return winSourceConfigs[i].getEncryption();
            }
            else {
                return "to implement";
            }
        }
    }
    return "";
}


bool ClientSettings::setConfigSourcesParameter(const char* sourceName, const char* parameter, const char* value) {

    for (unsigned int i=0; i<winSourceConfigsCount; ++i) {
        if (strcmp(winSourceConfigs[i].getName(), sourceName) == 0) {
            if (strcmp(parameter, "sync") == 0) {
                winSourceConfigs[i].setSync(value);
            }
            else if (strcmp(parameter, "encryption") == 0) {
                winSourceConfigs[i].setEncryption(value);
            }
            return true;
        }
    }
    return false;
}



int ClientSettings::saveSyncSourceConfig(const char* name) {

    int ret = false;
    //lastErrorCode = ERR_NONE;
    resetError();

    for (unsigned int i=0; i<winSourceConfigsCount; ++i) {

        if (strcmp(name, winSourceConfigs[i].getName()) == 0) {
            saveWinSourceConfig(i);
            break;
        }
    }
    resetError();
    ret = getLastErrorCode();
    
    //ret = (lastErrorCode == ERR_NONE);

    return ret;
}

// method used to get the singleton instance
ClientSettings* getRegConfig(){
    return ClientSettings::getInstance();
}

int ClientSettings::getConfigSourcesLastTimestamp(const char* sourceName) {
    for (unsigned int i=0; i<sourceConfigsCount; ++i) {
        if (strcmp(sourceConfigs[i].getName(), sourceName) == 0) {
            return sourceConfigs[i].getLast();
        }
    }
    return 0;
}

// ------------------------- Read properties from win registry -------------------------
/**
* Read the configuration from Windows registry into this object.
* This method overrides 'DMTClientConfig::read()'.
* 'DMTClientConfig::read()' is first called to read all common properties, then
* specific SyncSource properties are retrieved.
*
* A separate 'winSourceConfigs' array is used to store all specific SS config, common
* props are linked to original 'sourceConfigs' array (no copy!).
*
* @return TRUE if no errors
*/

bool ClientSettings::read() {

    bool ret = FALSE;
    unsigned int i=0;

    //
    // Read common properties
    //
    //lastErrorCode = ERR_NONE;
    resetError();
    ret = DMTClientConfig::read();
    
    char* passDecoded = decodePassword(accessConfig.getPassword());
    accessConfig.setPassword(passDecoded);
    if (passDecoded) { delete [] passDecoded; }
    
    if (getLastErrorCode() != ERR_NONE) {
        return FALSE;
    }

    if (sourceConfigsCount < 1) {
        return FALSE;
    }
    winSourceConfigsCount = sourceConfigsCount;

    //
    // Read additional properties for SyncSources (use winSyncSourceConfig)
    //
    if (winSourceConfigs) {
        delete [] winSourceConfigs;
    }

    if (!open()) {
        return FALSE;
    }

    readConfig(); // this 1st
    //readConfigSources("mail");

    winSourceConfigs = new WindowsSyncSourceConfig[sourceConfigsCount];
    for (i=0; i<sourceConfigsCount; i++) {

        // Link internal pointer to sourceConfigs array
        winSourceConfigs[i].setCommonConfig(DMTClientConfig::getSyncSourceConfig(i));

        int t=0;

        // Read specific properties
        readWinSourceConfig(i);

    }
    
    // read the branding parameters
    readBrandingParams();

    close();
    return ret;
}


// ---------------------------- Save properties to win registry ----------------------------
/**
* Save the configuration from this object into Windows registry.
* If SyncReport pointer is passed not NULL, each SyncSource configuration
* will be saved ONLY if that source was successfully synced.
* This method overrides 'DMTClientConfig::save()'.
*
* A separate 'winSourceConfigs' array is used to store all SS config, so a
* specific method 'saveWinSourceConfig()' is used to save sources config
* into the windows registry.
*
* @return TRUE if no errors
*/

/**
 * save everything
 */
bool ClientSettings::save() {

    bool ret = FALSE;
    //LOG.debug(DBG_WRITING_CONFIG_TO_DM);

    if (!sourcesNode) {
        open();
    }

    //
    // SyncML management node (TBD: manage dirty flags!)
    //
    // modification of the password here
    StringBuffer buffer = encodePassword(accessConfig.getPassword());
    accessConfig.setPassword(buffer.c_str());
    
    saveAccessConfig(*syncMLNode);
    saveDeviceConfig(*syncMLNode);

    // save root values
    //saveConfig();


    //lastErrorCode = ERR_NONE;
    resetError();
    for(unsigned int i=0; i<sourceConfigsCount; i++) {
           saveWinSourceConfig(i);
    }

    //ret = (lastErrorCode == ERR_NONE);
    resetError();
    ret = (getLastErrorCode() != 0);

    close();

    char* passDecoded = decodePassword(accessConfig.getPassword());
    accessConfig.setPassword(passDecoded);
    if (passDecoded) { delete [] passDecoded; }
    return ret;
}

/**
 * using the flags set in the 'dirty' property, we save sections accordingly
 * used in UI
 */
bool ClientSettings::saveDirty() {

    bool ret = FALSE;
    if(dirty){

        if (!sourcesNode) {
            open();
        }

        if (dirty & CS_DIRTY_ACCESS_CONFIG){
            //saveAccessConfig(*syncMLNode);
            saveConfig();
        }

        if(dirty & CS_DIRTY_DEVICE_CONFIG){
            //saveDeviceConfig(*syncMLNode);
            saveConfig();
        }

        // save root values
        if(dirty & CS_DIRTY_ROOT_CONFIG){
            saveConfig();
        }

        if(dirty & CS_DIRTY_SOURCE_ENABLED){
            saveSyncModes();
        }

        //lastErrorCode = ERR_NONE;
        resetError();
        for(unsigned int i=0; i<sourceConfigsCount; i++) {
            if(dirty & CS_DIRTY_SOURCE_ALL){
                saveWinSourceConfig(i);
                //if(strcmp(winSourceConfigs[i].getName(),"mail") == 0){
                    //saveConfigSources("mail");
                //}
                continue;
            }

            if(strcmp(winSourceConfigs[i].getName(),"mail") == 0){
                if(dirty & CS_DIRTY_SOURCE_MAIL){
                    // copy all the properties on the default
                    saveWinSourceConfig(i);
                    //saveConfigSources("mail");
                    continue;
                }
            }

            if(strcmp(winSourceConfigs[i].getName(),"contact") == 0){
                if(dirty & CS_DIRTY_SOURCE_CONTACTS){
                    saveWinSourceConfig(i);
                    continue;
                }
            }

            if(strcmp(winSourceConfigs[i].getName(),"calendar") == 0){
                if(dirty & CS_DIRTY_SOURCE_CALENDAR){
                    saveWinSourceConfig(i);
                    continue;
                }
            }

            if(strcmp(winSourceConfigs[i].getName(),"task") == 0){
                if(dirty & CS_DIRTY_SOURCE_TASKS){
                    saveWinSourceConfig(i);
                    continue;
                }
            }

            if(strcmp(winSourceConfigs[i].getName(),"note") == 0){
                if(dirty & CS_DIRTY_SOURCE_NOTES){
                    saveWinSourceConfig(i);
                    continue;
                }
            }

            if(strcmp(winSourceConfigs[i].getName(),"briefcase") == 0){
                if(dirty & CS_DIRTY_SOURCE_BRIEFCASE){
                    saveWinSourceConfig(i);
                    continue;
                }
            }

            if(strcmp(winSourceConfigs[i].getName(),"config") == 0){
                if(dirty & CS_DIRTY_SOURCE_CONFIG){
                    saveWinSourceConfig(i);
                    continue;
                }
            }
        }
        //ret = (lastErrorCode == ERR_NONE);
        resetError();
        ret = (getLastErrorCode() != 0);
        

        //Setting dirty to false
        dirty = FALSE;
        close();
    }
    return ret;
}

/**
* Here we are sure the management node is not NULL and the timestamp
* has a value
*/
void ClientSettings::saveTimeStamp(ManagementNode* node, unsigned long stamp) {
    
    char buf[512];
    timestampToAnchor(stamp, buf);
    node->setPropertyValue(PROPERTY_SOURCE_LAST_SYNC, buf);      
    LOG.debug("Saved node: %s, last: %s", node->getName(), buf);
}


int ClientSettings::saveWinSourceLastTimeStamp(unsigned int i) {
    
    LOG.debug("ClientSettings::saveWinSourceLastTimeStamp by index");
    
    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    if (!sourcesNode) {
        open();
    }

    //
    // If node not found, create node from Source name.
    //
    if (sourcesNode->getChild(i) == NULL) {
        char* fn = sourcesNode->createFullName();
        sprintf(nodeName, "%s/%s", fn, winSourceConfigs[i].getName());
        LOG.debug("ClientSettings: saveWinSourceLastTimeStamp create the node for the %s: ", nodeName);
        delete [] fn;
        node = dmt->readManagementNode(nodeName);
    }
    else {
        node = (ManagementNode*)sourcesNode->getChild(i)->clone();
    }
    //
    // Save Last Time Stamp
    //
    LOG.debug("ClientSettings::saveWinSourceLastTimeStamp of %s: ", winSourceConfigs[i].getName());    
    saveTimeStamp(node, winSourceConfigs[i].getLast());
    delete node;
    return 0;
   
}

int ClientSettings::saveWinSourceLastTimeStamp(const char* name) {
    
    LOG.debug("ClientSettings::saveWinSourceLastTimeStamp by name: %s", name);
    
    if ((name == NULL) || (strlen(name) == 0)) {
        return 0;
    }
    
    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    if (!sourcesNode) {
        open();
    }
    
    for (unsigned int i = 0; i < winSourceConfigsCount; ++i) {

        if (strcmp(name, winSourceConfigs[i].getName()) == 0) {            
            if (sourcesNode->getChild(i) == NULL) {
                char* fn = sourcesNode->createFullName();
                sprintf(nodeName, "%s/%s", fn, winSourceConfigs[i].getName());
                delete [] fn;
                node = dmt->readManagementNode(nodeName);
            }
            else {
                node = (ManagementNode*)sourcesNode->getChild(i)->clone();
            }
            
            LOG.debug("ClientSettings::saveWinSourceLastTimeStamp of %s: ", winSourceConfigs[i].getName());        
            saveTimeStamp(node, winSourceConfigs[i].getLast());
            delete node;
            break;
        }
    }
    return 0;
}


/*
* Save WindowsSyncSourceConfig properties in DMTree for the desired Source.
* Source properties are placed in specific node under sources node.
* Notes:
* if the node for the current source is not found, it is created!
* if we are under a restore sync (slow/refresh), 'sync' property will
* be skipped (keep previous value from registry)
*
* @param i : the index of SyncSource node
*/
void ClientSettings::saveWinSourceConfig(unsigned int i) {

    LOG.debug("ClientSettings: entering the saveWinSourceConfig");
    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    if (!sourcesNode) {
        open();
    }

    //
    // If node not found, create node from Source name.
    //
    if (sourcesNode->getChild(i) == NULL) {
        char* fn = sourcesNode->createFullName();
        sprintf(nodeName, "%s/%s", fn, winSourceConfigs[i].getName());
        LOG.debug("ClientSettings: create the node for the %s: ", nodeName);
        delete [] fn;
        node = dmt->readManagementNode(nodeName);
    }
    else {
        node = (ManagementNode*)sourcesNode->getChild(i)->clone();
    }
    LOG.debug("ClientSettings: saving the configuration...");
    //
    // Save source properties
    //
    if (node) {
        LOG.debug("ClientSettings: valid node...");       

        // Common props:
        node->setPropertyValue(PROPERTY_SOURCE_NAME,        winSourceConfigs[i].getName          ());
        node->setPropertyValue(PROPERTY_SOURCE_URI,         winSourceConfigs[i].getURI           ());
        node->setPropertyValue(PROPERTY_SOURCE_TYPE,        winSourceConfigs[i].getType          ());
        node->setPropertyValue(PROPERTY_SOURCE_VERSION,     winSourceConfigs[i].getVersion       ());
        node->setPropertyValue(PROPERTY_SOURCE_SYNC_MODES,  winSourceConfigs[i].getSyncModes     ());
        node->setPropertyValue(PROPERTY_SOURCE_ENCODING,    winSourceConfigs[i].getEncoding      ());
        node->setPropertyValue(PROPERTY_SOURCE_SUPP_TYPES,  winSourceConfigs[i].getSupportedTypes());
        node->setPropertyValue(PROPERTY_SOURCE_ENCRYPTION,  winSourceConfigs[i].getEncryption    ());

        node->setPropertyValue(PROPERTY_SOURCE_SCHEDULE,  winSourceConfigs[i].getSchedule().c_str());

        //timestampToAnchor(winSourceConfigs[i].getLast(), buf);
        //LOG.debug("ClientSettings: setting the last %s: ", buf);
        //node->setPropertyValue(PROPERTY_SOURCE_LAST_SYNC, buf);

        node->setPropertyValue(PROPERTY_SOURCE_SYNC,    winSourceConfigs[i].getSync          ());
        node->setPropertyValue(PROPERTY_SOURCE_USE_SIF, winSourceConfigs[i].getUseSif().c_str());

        char* sifVersion = toMultibyte(SIF_VERSION);

        if (strcmp(winSourceConfigs[i].getName(), "contact")  == 0) {
            if(winSourceConfigs[i].getUseSif() == "1"){
                node->setPropertyValue(PROPERTY_SOURCE_TYPE,     "text/x-s4j-sifc");
                node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, sifVersion);
            }
            else {
                node->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-vcard");
                if(strcmp(winSourceConfigs[i].getEncryption(), "0") != 0) {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                } else {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
                }
                char* vCardVersion = toMultibyte(VCARD_VERSION);
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, vCardVersion);
                delete [] vCardVersion;
            }
        }

        else if (strcmp(winSourceConfigs[i].getName(), "calendar")  == 0) {
            if(winSourceConfigs[i].getUseSif() == "1"){
                node->setPropertyValue(PROPERTY_SOURCE_TYPE,     "text/x-s4j-sife");
                node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, sifVersion);
            }
            else {
                node->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-vcalendar");
                if(strcmp(winSourceConfigs[i].getEncryption(), "0") != 0) {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                } else {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
                }
                char* vCalendarVersion = toMultibyte(VCALENDAR_VERSION);
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, vCalendarVersion);
                delete [] vCalendarVersion;
            }
        }

        else if (strcmp(winSourceConfigs[i].getName(), "task")  == 0) {
            if(winSourceConfigs[i].getUseSif() == "1"){
                node->setPropertyValue(PROPERTY_SOURCE_TYPE,     "text/x-s4j-sift");
                node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, sifVersion);
            }
            else {
                node->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-vcalendar");
                if(strcmp(winSourceConfigs[i].getEncryption(), "0") != 0) {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                } else {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
                }
                char* vCalendarVersion = toMultibyte(VCALENDAR_VERSION);
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, vCalendarVersion);
                delete [] vCalendarVersion;
            }
        }

        else if (strcmp(winSourceConfigs[i].getName(), "note")  == 0) {
            if(winSourceConfigs[i].getUseSif() == "1"){
                node->setPropertyValue(PROPERTY_SOURCE_TYPE,     "text/x-s4j-sifn");
                node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, sifVersion);
            }
            else {
                node->setPropertyValue(PROPERTY_SOURCE_TYPE, "text/x-vnote");
                if(strcmp(winSourceConfigs[i].getEncryption(), "0") != 0) {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
                } else {
                    node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
                }
                char* vNoteVersion = toMultibyte(VNOTE_VERSION);
                node->setPropertyValue(PROPERTY_SOURCE_VERSION, vNoteVersion);
                delete [] vNoteVersion;
            }
            node->setPropertyValue(PROPERTY_SOURCE_DIR, winSourceConfigs[i].getSyncDir().data());
        }

        else if(strcmp(winSourceConfigs[i].getName(), "briefcase") == 0) {

            if(strcmp(winSourceConfigs[i].getEncryption(), "0") != 0) {
                node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
            } else {
                node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
            }
            node->setPropertyValue(PROPERTY_SOURCE_TYPE, winSourceConfigs[i].getType()); // required?
            node->setPropertyValue(PROPERTY_SOURCE_DIR, winSourceConfigs[i].getSyncDir().data());
        } 
        
        else if( (strcmp(winSourceConfigs[i].getName(), "mail") == 0)) {

            char t[128];

            sprintf(t, "%d", mailssconfig->getDownloadAge());
            node->setPropertyValue(PROPERTY_SOURCE_DOWNLOAD_AGE, t);
            sprintf(t, "%d", mailssconfig->getBodySize());
            node->setPropertyValue(PROPERTY_SOURCE_BODY_SIZE, t);
            sprintf(t, "%d", mailssconfig->getAttachSize());
            node->setPropertyValue(PROPERTY_SOURCE_ATTACH_SIZE, t);

            sprintf(t, "%d", mailssconfig->getInbox());
            node->setPropertyValue(PROPERTY_SOURCE_INBOX, t);
            sprintf(t, "%d", mailssconfig->getOutbox());
            node->setPropertyValue(PROPERTY_SOURCE_OUTBOX, t);
            sprintf(t, "%d", mailssconfig->getTrash());
            node->setPropertyValue(PROPERTY_SOURCE_TRASH, t);
            sprintf(t, "%d", mailssconfig->getSent());
            node->setPropertyValue(PROPERTY_SOURCE_SENT, t);
            sprintf(t, "%d", mailssconfig->getDraft());
            node->setPropertyValue(PROPERTY_SOURCE_DRAFT, t);
            sprintf(t, "%d", mailssconfig->getSchedule());
            node->setPropertyValue(PROPERTY_SOURCE_SCHEDULE, t);


            if(strcmp(winSourceConfigs[i].getEncryption(), "0") != 0) {
                mailssconfig->setEncoding("b64");
            } else {
                mailssconfig->setEncoding("bin");
            }
            node->setPropertyValue(PROPERTY_SOURCE_ENCODING, mailssconfig->getEncoding());

        }

        if (sifVersion) {
            delete [] sifVersion;
        }
        delete node;
    }
    LOG.debug("ClientSettings: exiting the saveWinSourceConfig");
}

/**
* Save only "sync" properties of each source, to win registry.
*/
void ClientSettings::saveSyncModes() {

    if (!sourcesNode) {
        open();
    }

    ManagementNode* node = NULL;
    for(unsigned int i=0; i<sourceConfigsCount; ++i) {
        node = sourcesNode->getChild(i);
        if (node) {
            node->setPropertyValue(PROPERTY_SOURCE_SYNC, winSourceConfigs[i].getSync());
        }
        node = NULL;
    }

    close();
}


//////////////////////////////////////////////////////////////////////////

// ------------------------------ Get/Set objects ----------------------------------

/**
* Return a pointer to the internal WindowsSyncSourceConfig object from
* its name (must NOT be freed by caller).
* This method replaces 'getSyncSourceConfig()' of DMTClientConfig.
*
* @param name : the source name
* @return     : the correspondent WindowsSyncSourceConfig pointer
*/

WindowsSyncSourceConfig* ClientSettings::getWinSyncSourceConfig(const char* name) {
    if ((name == NULL) || (strlen(name) == 0)) {
        return 0;
    }

    for (unsigned int i=0; i<sourceConfigsCount; i++) {
        if (strcmp(winSourceConfigs[i].getName(), name) == 0) {
            return &winSourceConfigs[i];
        }
    }

    return 0;
}
/**
* Set the passed WindowsSyncSourceConfig object into the correspondent object
* inside 'winSourceConfigs' array. The values are copied into the object that
* matches the same name of the passed one.
* This method replaces the 'SyncManagerConfig::setSyncSourceConfig()'.
* Note:
* If a WindowsSyncSourceConfig with the same name is not found, the passed
* object is added at the end of the 'winSourceConfig' array.
*
* @param wsc : the WindowsSyncSourceConfig passed by reference
* @return    : TRUE if no errors
*/
bool ClientSettings::setSyncSourceConfig(WindowsSyncSourceConfig& wsc) {

    unsigned int i=0;
    for (i=0; i<winSourceConfigsCount; ++i) {
        if (strcmp(wsc.getName(), winSourceConfigs[i].getName()) == 0) {
            break;
        }
    }
    if (i >= winSourceConfigsCount) {
        // Not found! -> add the WindowsSyncSourceConfig.
        return addSyncSourceConfig(wsc);
    }

    // copy all values
    winSourceConfigs[i] = wsc;

    return TRUE;
}


/*
* Adds the passed WindowsSyncSourceConfig.
* It is added at the end of the 'winSourceConfig' array.
* This method replaces the 'SyncManagerConfig::addSyncSourceConfig()'.
*
* @param wsc : the WindowsSyncSourceConfig passed by reference
* @return    : TRUE if no errors
*/
bool ClientSettings::addSyncSourceConfig(WindowsSyncSourceConfig& wsc) {

    unsigned int i = 0;
    WindowsSyncSourceConfig* s = NULL;

    // Copy array in a tmp buffer
    if (winSourceConfigsCount>0) {
        s = new WindowsSyncSourceConfig[winSourceConfigsCount];
        for (i=0; i<winSourceConfigsCount; i++) {
            s[i] = winSourceConfigs[i];
        }
    }

    // Delete old one, create new (+1 element)
    if (winSourceConfigs) {
        delete [] winSourceConfigs;
    }
    winSourceConfigs = new WindowsSyncSourceConfig[winSourceConfigsCount+1];

    // Copy back.
    for (i=0; i<winSourceConfigsCount; i++)
        winSourceConfigs[i] = s[i];
    // Copy the new one.
    winSourceConfigs[winSourceConfigsCount] = wsc;

    if (s) {
        delete [] s;
        s = NULL;
    }

    winSourceConfigsCount ++;
    return TRUE;
}

// ------------------------------ Other methods ----------------------------------

/*
* Returns the value of the given property, from rootKey tree (read only).
* The value is returned as a new char array and must be freed by the user.
*
* @param context      : full context of key, under rootKey
* @param propertyName : name of property to retrieve
* @param rootKey      : one of  HKEY_LOCAL_MACHINE
*                               HKEY_CLASSES_ROOT
*                               HKEY_CURRENT_USER
*                               HKEY_USERS
*                               ...
* @return             : the property value (new allocated buffer)
*                       if key not found, returns an empty string.
*/
char* ClientSettings::readPropertyValue(const char* context, const char* propertyName, HKEY rootKey) {

    DWORD res = 0;
    long  err = 0;
    ULONG dim = 0;
    HKEY  key = NULL;
    char* ret = NULL;

    char* fullContext = new char[strlen(context) + 10];
    sprintf(fullContext, "%s/%s", "Software", context);

    //RegCreateKeyExA(
    WCHAR* wideFullContext = toWideChar(fullContext);
    RegCreateKeyEx(
        rootKey,
        wideFullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_READ,                           // Read only: could be from a limited rights user.
        NULL,
        &key,
        &res
        );

    if (key == 0) {
        //lastErrorCode = ERR_INVALID_CONTEXT;
        //sprintf(lastErrorMsg, ERR_INVALID_REG_PATH, fullContext);
        setErrorF(ERR_INVALID_CONTEXT, "Invalid context path: %s", fullContext);
        goto finally;
    }

    // Get value length
    //err = RegQueryValueExA(
    WCHAR* widePropertyName = toWideChar(propertyName);
    err = RegQueryValueEx(
        key,
        widePropertyName,
        NULL,
        NULL,  // we currently support only strings
        NULL,
        &dim
        );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            char* buf = new char[dim + 1];

            //err = RegQueryValueExA(
            err = RegQueryValueEx(
                key,
                widePropertyName,
                NULL,
                NULL,  // we currently support only strings
                (UCHAR*)buf,
                &dim
                );
            if (err == ERROR_SUCCESS) {
                ret = stringdup(buf);
            }
            delete [] buf;
        }
    }

    finally:

    if(widePropertyName){
        delete [] widePropertyName;
        widePropertyName = NULL;
    }

    if(wideFullContext){
        delete [] wideFullContext;
        wideFullContext = NULL;
    }

    if (!ret) {
        // Always return an empty string if key not found!
        ret = stringdup(EMPTY_STRING);
    }
    if (fullContext) {
        delete [] fullContext;
    }
    if (key != 0) {
        RegCloseKey(key);
    }
    return ret;
}

void ClientSettings::readWinSourceConfig(unsigned int i) {

    char* tmp;
    if (!sourcesNode) {
        open();
    }

    ManagementNode* node = NULL;
    node = sourcesNode->getChild(i);

    if (node) {
        if (strcmp(node->getName(), "mails") == 0) {

            const char* context = NULL;
            char t[255];
            char* fn = node->createFullName();
            strcpy(t, fn);
            delete [] fn;

            context = strstr(t, node->getName());
            t[context - t - 1] = 0;

            MailSourceManagementNode* mmn = new MailSourceManagementNode(t, node->getName());

            if (mmn) {
                if (mailssconfig) {
                    delete mailssconfig;
                }
                mailssconfig = new MailSyncSourceConfig(mmn->getMailSourceConfig(true));
                delete mmn;
            }

        }

        tmp = node->readPropertyValue(PROPERTY_SOURCE_USE_SIF);
        winSourceConfigs[i].setUseSif(tmp);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_SOURCE_DIR);
        winSourceConfigs[i].setSyncDir(tmp);
        delete [] tmp;

        tmp = node->readPropertyValue(PROPERTY_SCHEDULE);
        winSourceConfigs[i].setSchedule(tmp);
        delete [] tmp;

    }

}

void ClientSettings::refreshIP(){

    // get IP
    checkNetwork();
    setIp(getLocalAddress());
}

void ClientSettings::setDirty(int flag){
    dirty |= flag;
}



void ClientSettings::setConfigSyncDirty(int flag) {
    configSyncDirty |= flag;
}

void ClientSettings::clearConfigSyncDirty(int flag) {
    configSyncDirty = configSyncDirty & (~flag);
}


/**
* Load data from the update tree all the configuration parameters.
* It populates also the currentVersion of the UpdateConfig class.
* At the moment "refresh" is not used
*/
BOOL ClientSettings::readUpdateParams(bool refresh){
    
    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    if (!sourcesNode) {
        if (!open()) {
            return FALSE;
        }
    }
    
    char syncMLContext[DIM_MANAGEMENT_PATH];
    char* fn = syncMLNode->createFullName();
    sprintf(syncMLContext, "%s", fn);
    delete [] fn;

    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_UPDATE);
    node = dmt->readManagementNode(nodeName);
    
    char* tmp = NULL;
    string t;

    if (node) {

        //updateParams = new UpdateParams();              
        string val;

        tmp = node->readPropertyValue(UPDATE_VERSION);
        t = tmp; 
        val = updateParams.getVersion();
        updateParams.setVersion((t.length()) > 0 ? t : val);
        delete [] tmp;

        tmp = node->readPropertyValue(UPDATE_MANDATORY);
        t = tmp; 
        updateParams.setMandatory((t.length()) > 0 ? t : "0");
        delete [] tmp;

        tmp = node->readPropertyValue(UPDATE_RELEASE_DATE);
        t = tmp; 
        updateParams.setReleaseDate((t.length()) > 0 ? t : "0");
        delete [] tmp;
        
        // default is ""
        tmp = node->readPropertyValue(UPDATE_URL_UPDATE);
        t = tmp; 
        updateParams.setUrlUpdate(t);
        delete [] tmp;
               
        tmp = node->readPropertyValue(UPDATE_URL_COMMENT);
        t = tmp; 
        updateParams.setUrlComment(t);
        delete [] tmp;      
       
        tmp = node->readPropertyValue(UPDATE_SIZE);
        t = tmp; 
        updateParams.setSize((t.length()) > 0 ? t : "0");
        delete [] tmp;

        tmp = node->readPropertyValue(UPDATE_LAST_CHECK_UPDATE);
        t = tmp; 
        updateParams.setLastCheckUpdate((t.length()) > 0 ? t : "0");
        delete [] tmp;

        tmp = node->readPropertyValue(UPDATE_INT_CHECK_UP_HTTP);
        t = tmp; 
        val = updateParams.getIntervalCheckUpdateHttp();
        updateParams.setIntervalCheckUpdateHttp((t.length()) > 0 ? t : val);
        delete [] tmp;

        tmp = node->readPropertyValue(UPDATE_INT_REMIND);
        t = tmp; 
        val = updateParams.getIntervalRemind();
        updateParams.setIntervalRemind((t.length()) > 0 ? t : val);
        delete [] tmp;

        tmp = node->readPropertyValue(UPDATE_SKIPPED);
        t = tmp; 
        updateParams.setSkipped((t.length()) > 0 ? t : "0");
        delete [] tmp;
        
        tmp = node->readPropertyValue(UPDATE_LAST_UPDATE);
        t = tmp; 
        val = updateParams.getVersion();
        updateParams.setLastUpdated((t.length()) > 0 ? t : val);
        delete [] tmp;                
        
        tmp = node->readPropertyValue(UPDATE_LATER);
        t = tmp; 
        updateParams.setLater((t.length()) > 0 ? t : "0");
        delete [] tmp; 

        tmp = node->readPropertyValue(UPDATE_NOW);
        t = tmp; 
        updateParams.setNow((t.length()) > 0 ? t : "0");
        delete [] tmp; 

        tmp = node->readPropertyValue(UPDATE_IS_AN_UPDATE_AVAILABLE);
        t = tmp; 
        updateParams.setIsAnUpdateAvailable((t.length()) > 0 ? t : "0");
        delete [] tmp; 
        
        updateParams.setCurrentVersion(deviceConfig.getOem());
        
        delete node;
        node = NULL;
    }
    return TRUE;
}

/**
* Save data into the registry
*/
BOOL ClientSettings::storeUpdateParams(){
    
    if (updateParams.getCurrentVersion() == "0") {
        LOG.debug("The udpateParams object doesn't exists: don't save");
        return false;
    }

    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];

    if (!sourcesNode) {
        if (!open()) {
            return FALSE;
        }
    }
    
    char syncMLContext[DIM_MANAGEMENT_PATH];
    char* fn = syncMLNode->createFullName();
    sprintf(syncMLContext, "%s", fn);
    delete [] fn;

    sprintf(nodeName, "%s%s", syncMLContext, CONTEXT_UPDATE);
    node = dmt->readManagementNode(nodeName);
    
    char* tmp = NULL;
    string t;

    if (node) {
        
        node->setPropertyValue(UPDATE_VERSION, updateParams.getVersion().c_str());                                   
        node->setPropertyValue(UPDATE_MANDATORY, updateParams.getMandatory().c_str());                                   
        node->setPropertyValue(UPDATE_RELEASE_DATE, updateParams.getReleaseDate().c_str());                                   
        node->setPropertyValue(UPDATE_URL_UPDATE, updateParams.getUrlUpdate().c_str());                                   
        node->setPropertyValue(UPDATE_URL_COMMENT, updateParams.getUrlComment().c_str());   
        node->setPropertyValue(UPDATE_SIZE, updateParams.getSize().c_str());                                   
        node->setPropertyValue(UPDATE_LAST_CHECK_UPDATE, updateParams.getLastCheckUpdate().c_str());                                   
        node->setPropertyValue(UPDATE_INT_CHECK_UP_HTTP, updateParams.getIntervalCheckUpdateHttp().c_str());                                   
        node->setPropertyValue(UPDATE_INT_REMIND, updateParams.getIntervalRemind().c_str());                                   
        node->setPropertyValue(UPDATE_SKIPPED, updateParams.getSkipped().c_str());                                   
        node->setPropertyValue(UPDATE_LAST_UPDATE, updateParams.getLastUpdated().c_str());  
        node->setPropertyValue(UPDATE_LATER, updateParams.getLater().c_str()); 
        node->setPropertyValue(UPDATE_NOW, updateParams.getNow().c_str()); 
        node->setPropertyValue(UPDATE_IS_AN_UPDATE_AVAILABLE, updateParams.getIsAnUpdateAvailable().c_str()); 
               
    }
    return TRUE;
}


UpdateParams& ClientSettings::getUpdateParams() {
    return updateParams;
}


/**
* Load data from the Brand tree all the configuration parameters.
* It is used to load in the config the needed stuff about the branding of the 
* plugin.
*
*/
BOOL ClientSettings::readBrandingParams(){
    
    ManagementNode* node;
    char nodeName[DIM_MANAGEMENT_PATH];
    
    sprintf(nodeName, "%s%s", rootContext, CONTEXT_BRAND);
    if (dmt == NULL) {
        dmt = DMTreeFactory::getDMTree(rootContext);
    }
    node = dmt->readManagementNode(nodeName);
          
    char* tmp = NULL;
    string t;

    if (node) {
        string t;
        tmp = node->readPropertyValue(BRAND_URL_FIELD);        
        brandingParams.setShowUrlField(strlen(tmp) > 0 ? tmp : "0");
        delete [] tmp;

        tmp = node->readPropertyValue(BRAND_ADV_SETTING_MENU);
        brandingParams.setShowAdvSettingMenu(strlen(tmp) > 0 ? tmp : "0");
        delete [] tmp;

        tmp = node->readPropertyValue(BRAND_BRIEFCASE_SOURCE);
        brandingParams.setShowBriefcaseSource(strlen(tmp) > 0 ? tmp : "0");
        delete [] tmp;
        
        tmp = node->readPropertyValue(BRAND_NOTE_SOURCE);
        brandingParams.setShowNoteSource(strlen(tmp) > 0 ? tmp : "0");
        delete [] tmp;

        delete node;
        node = NULL;
    }
    return TRUE;
}


/*
bool ClientSettings::readConfigSources(const char* sourceType, BOOL toClose) {

    int n = 0, i = 0; // number of sync sources

    bool ret = FALSE;
    ManagementNode* nn = NULL;

    if (!open()) {
        return FALSE;
    }

    n = sourcesNode->getChildrenMaxCount();

    //
    // Let's remove previously created config objects and reinitialize
    // sourceConfigs
    //
    if (sourceConfigs) {
        delete [] sourceConfigs;
    }
    sourceConfigs = new SyncSourceConfig[n];
    sourceConfigsCount = n;

    for (i=0; i<n; ++i) {
        // node owns children, we must not delete them
        readSourceConfig(i, *(sourcesNode) );
    }

    if (sourceType && strcmp(sourceType, "all") == 0) {
         if (toClose)
            close();
        return true;
    }

    for (unsigned int i=0; i<sourceConfigsCount; ++i) {
        if (strcmp(sourceConfigs[i].getName(), sourceType) == 0) {
            ssconfig.assign(sourceConfigs[i]);
            nn = sourcesNode->getChild(i);
            break;
        }
    }

    char* tmp;

    // TODO: this method is used in ClientSettings for "mail" to init mailssconfig, but seems not used anywhere else,
    // so the code for the other sources below should be removed
    if (strcmp(sourceType, "contact") == 0) {

        tmp = nn->readPropertyValue(PROPERTY_SOURCE_USE_SIF);
        //~setUseSif(string(tmp));
        delete [] tmp;
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        //~setSchedule(string(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "calendar") == 0) {

        tmp = nn->readPropertyValue(PROPERTY_SOURCE_USE_SIF);
        //~setUseSif(string(tmp));
        delete [] tmp;
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        //~setSchedule(string(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "task") == 0) {

        tmp = nn->readPropertyValue(PROPERTY_SOURCE_USE_SIF);
        //~setUseSif(string(tmp));
        delete [] tmp;
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        //~setSchedule(string(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "briefcase") == 0) {

        tmp = nn->readPropertyValue(PROPERTY_SOURCE_DIR);
        setDir(string(tmp));
        delete [] tmp;
         tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        //~setSchedule(string(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "note") == 0) {

        tmp = nn->readPropertyValue(PROPERTY_SOURCE_DIR);
        setDir(string(tmp));
        delete [] tmp;
        tmp = nn->readPropertyValue(PROPERTY_SCHEDULE);
        //~setSchedule(string(tmp));
        delete [] tmp;
    } else if (strcmp(sourceType, "mail") == 0) {
        const char* context = NULL;
        char t[255];
        char* fn = nn->createFullName();
        strcpy(t, fn);
        delete [] fn;

        context = strstr(t, nn->getName());
        t[context - t - 1] = 0;

        MailSourceManagementNode* mmn = new MailSourceManagementNode(t, nn->getName());

        mailssconfig = mmn->getMailSourceConfig(true);

        if (mmn) {
            delete mmn;
        }

    }
    ret = TRUE;

    if (toClose)
        close();

    return ret;

}
*/
/*
// TODO: called only for "mail", the other sources could be removed if not needed anymore
bool ClientSettings::saveConfigSources(const char* saveAll) {

    ManagementNode* nn;

    // to save all the sources config. It is used by the form
    // in which there are all the sources whith the list of the sync property
    // and to reset the last timestamp in the recover form. it is set to 1
    //

    if (saveAll && strcmp(saveAll, "all") == 0) {

        for (unsigned int i=0; i<sourceConfigsCount; ++i) {

            nn = sourcesNode->getChild(i);
            saveSourceConfig(i, *sourcesNode, *nn);

        }
        close();
        return true;
    }


    for (unsigned int i=0; i<sourceConfigsCount; ++i) {

        if (strcmp(sourceConfigs[i].getName(), ssconfig.getName()) == 0) {
            nn = sourcesNode->getChild(i);
            break;
        }
    }

    if (strcmp(ssconfig.getName(), "contact")  == 0 ||
        strcmp(ssconfig.getName(), "calendar") == 0 ||
        strcmp(ssconfig.getName(), "task")  == 0) {

        if(strcmp(ssconfig.getEncryption(), "0") != 0 || strcmp(ssconfig.getName(), "note") == 0) {
            nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
        } else {
            nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
        }
        //nn->setPropertyValue(PROPERTY_SOURCE_ENCODING, ssconfig.getEncoding());

    } else if (strcmp(ssconfig.getName(), "mail") == 0) {

        //setMailSourceConfig(mailssconfig, nn);

        const char* context = NULL;
        char* tt = nn->createFullName();
        char t[255];
        strcpy(t, tt);
        context = strstr(t, nn->getName());
        t[context - t - 1] = 0;

        MailSourceManagementNode* mmn = new MailSourceManagementNode(t, nn->getName());

        if(strcmp(mailssconfig.getEncryption(), "0") != 0) {
            mailssconfig.setEncoding("b64");
        } else {
            mailssconfig.setEncoding("bin");
        }

        //mmn->setMailSourceConfig(mailssconfig);
        if (tt) { delete [] tt; }
        if (mmn){ delete [] mmn;}

    }

    close();
    return TRUE;

}
*/

/**
* Reads all sources timestamps from win registry and set
* all values into configuration.

void ClientSettings::readSourcesTimestamps() {

    if (!open()) {
        return;
    }

    for (unsigned int i=0; i<sourceConfigsCount; i++) {
        ManagementNode* node = sourcesNode->getChild(i);
        if (node) {
            // This sets only variables that the library uses internally, like anchors
            readSourceVars(i, *sourcesNode, *node);
        }
    }

    close();
}
*/

/**
* Reads only "sync" properties of each source, to win registry.

void ClientSettings::readSyncModes() {

    if (!open()) {
        return;
    }

    for (unsigned int i=0; i<sourceConfigsCount; i++) {
        ManagementNode* node = sourcesNode->getChild(i);
        if (node) {
            winSourceConfigs[i].setSync(node->readPropertyValue(PROPERTY_SOURCE_SYNC));
        }
    }

    close();
}
*/