/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "pim/ContactBuilder.h"
#include "pim/Utils.h"
#include "pim/MAPIUtils.h"
#include "base/quoted-printable.h"
#include "base/util/utils.h"
#include "base/util/WString.h"
#include "spds/spdsutils.h"
#include "base/util/StringBuffer.h"
#include "syncml/core/PropParam.h"

#include <initguid.h>
#define INITGUID
#include <imaging.h>
#include <aygshell.h>


/**
 * Enumeration of contact fields used by WM plugin.
 * @note Don't change the order of these properties!
 *       This list MUST be synchronized with PIMPR_props array of CEPROPID fields.
 */
enum {
    ePIMPR_FILEAS                          ,
    ePIMPR_TITLE                           ,
    ePIMPR_FIRST_NAME                      ,
    ePIMPR_MIDDLE_NAME                     ,
    ePIMPR_LAST_NAME                       ,
    ePIMPR_SUFFIX                          ,
    ePIMPR_NICKNAME                        ,
    ePIMPR_YOMI_FIRSTNAME                  ,
    ePIMPR_YOMI_LASTNAME                   ,
    ePIMPR_YOMI_COMPANY                    ,

    ePIMPR_COMPANY_NAME                    ,
    ePIMPR_DEPARTMENT                      ,
    ePIMPR_JOB_TITLE                       ,
    ePIMPR_MANAGER                         ,
    ePIMPR_OFFICE_LOCATION                 ,
    ePIMPR_ASSISTANT_NAME                  ,
    ePIMPR_EMAIL1_ADDRESS                  ,
    ePIMPR_EMAIL2_ADDRESS                  ,
    ePIMPR_EMAIL3_ADDRESS                  ,
    ePIMPR_IM1_ADDRESS                     ,
    ePIMPR_IM2_ADDRESS                     ,
    ePIMPR_IM3_ADDRESS                     ,
    ePIMPR_MOBILE_TELEPHONE_NUMBER         ,
    ePIMPR_BUSINESS_TELEPHONE_NUMBER       ,
    ePIMPR_BUSINESS2_TELEPHONE_NUMBER      ,
    ePIMPR_HOME_TELEPHONE_NUMBER           ,
    ePIMPR_HOME2_TELEPHONE_NUMBER          ,
    ePIMPR_BUSINESS_FAX_NUMBER             ,
    ePIMPR_HOME_FAX_NUMBER                 ,
    ePIMPR_PAGER_NUMBER                    ,
    ePIMPR_CAR_TELEPHONE_NUMBER            ,
    ePIMPR_RADIO_TELEPHONE_NUMBER          ,
    ePIMPR_COMPANY_TELEPHONE_NUMBER        ,
    ePIMPR_ASSISTANT_TELEPHONE_NUMBER      ,
    //ePIMPR_SMS                             ,
    //ePIMPR_MMS                             ,
    //ePIMPR_DISPLAY_NAME                    ,
    ePIMPR_SPOUSE                          ,
    ePIMPR_CHILDREN                        ,
    ePIMPR_WEB_PAGE                        ,
    //ePIMPR_RINGTONE                        ,
    ePIMPR_CUSTOMERID                      ,
    ePIMPR_GOVERNMENTID                    ,
    //ePIMPR_ACCOUNT_NAME                    ,
    //ePIMPR_BUSINESS_ADDRESS                ,
    ePIMPR_BUSINESS_ADDRESS_STREET         ,
    ePIMPR_BUSINESS_ADDRESS_CITY           ,
    ePIMPR_BUSINESS_ADDRESS_STATE          ,
    ePIMPR_BUSINESS_ADDRESS_POSTAL_CODE    ,
    ePIMPR_BUSINESS_ADDRESS_COUNTRY        ,
    //ePIMPR_HOME_ADDRESS                    ,
    ePIMPR_HOME_ADDRESS_STREET             ,
    ePIMPR_HOME_ADDRESS_CITY               ,
    ePIMPR_HOME_ADDRESS_STATE              ,
    ePIMPR_HOME_ADDRESS_POSTAL_CODE        ,
    ePIMPR_HOME_ADDRESS_COUNTRY            ,
    //ePIMPR_OTHER_ADDRESS                   ,
    ePIMPR_OTHER_ADDRESS_STREET            ,
    ePIMPR_OTHER_ADDRESS_CITY              ,
    ePIMPR_OTHER_ADDRESS_STATE             ,
    ePIMPR_OTHER_ADDRESS_POSTAL_CODE       ,
    ePIMPR_OTHER_ADDRESS_COUNTRY           ,
    ePIMPR_BIRTHDAY                        ,
    ePIMPR_ANNIVERSARY                     ,
    //ePIMPR_SMARTPROP                       ,
    ePIMPR_PICTURE                         , 
    //ePIMPR_YOMI_FILEAS                     ,
    //ePIMPR_SIM_PHONE                       ,
    ePIMPR_CATEGORIES                      ,
    //ePIMPR_BODY_BINARY                     ,
    ePIMPR_BODY_TEXT                       ,

    // This is the total number of contact properties supported
    NUM_COLS
};


/**
 * Array of contact fields used by WM plugin.
 * ******************************************
 *
 * @note Don't change the order of these properties!
 */
static CEPROPID PIMPR_props [] = {    
    PIMPR_FILEAS                          ,
    PIMPR_TITLE                           ,
    PIMPR_FIRST_NAME                      ,
    PIMPR_MIDDLE_NAME                     ,
    PIMPR_LAST_NAME                       ,    
    PIMPR_SUFFIX                          ,
    PIMPR_NICKNAME                        ,
    PIMPR_YOMI_FIRSTNAME                  ,
    PIMPR_YOMI_LASTNAME                   ,
    PIMPR_YOMI_COMPANY                    ,
    PIMPR_COMPANY_NAME                    ,
    PIMPR_DEPARTMENT                      ,
    PIMPR_JOB_TITLE                       ,
    PIMPR_MANAGER                         ,
    PIMPR_OFFICE_LOCATION                 ,
    PIMPR_ASSISTANT_NAME                  ,
    PIMPR_EMAIL1_ADDRESS                  ,
    PIMPR_EMAIL2_ADDRESS                  ,
    PIMPR_EMAIL3_ADDRESS                  ,
    PIMPR_IM1_ADDRESS                     ,
    PIMPR_IM2_ADDRESS                     ,
    PIMPR_IM3_ADDRESS                     ,
    PIMPR_MOBILE_TELEPHONE_NUMBER         ,
    PIMPR_BUSINESS_TELEPHONE_NUMBER       ,
    PIMPR_BUSINESS2_TELEPHONE_NUMBER      ,
    PIMPR_HOME_TELEPHONE_NUMBER           ,
    PIMPR_HOME2_TELEPHONE_NUMBER          ,
    PIMPR_BUSINESS_FAX_NUMBER             ,
    PIMPR_HOME_FAX_NUMBER                 ,
    PIMPR_PAGER_NUMBER                    ,
    PIMPR_CAR_TELEPHONE_NUMBER            ,
    PIMPR_RADIO_TELEPHONE_NUMBER          ,
    PIMPR_COMPANY_TELEPHONE_NUMBER        ,
    PIMPR_ASSISTANT_TELEPHONE_NUMBER      ,
    //PIMPR_SMS                             ,
    //PIMPR_MMS                             ,
    //PIMPR_DISPLAY_NAME                    ,
    PIMPR_SPOUSE                          ,
    PIMPR_CHILDREN                        ,
    PIMPR_WEB_PAGE                        ,
    //PIMPR_RINGTONE                        ,
    PIMPR_CUSTOMERID                      ,
    PIMPR_GOVERNMENTID                    ,
    //PIMPR_ACCOUNT_NAME                    ,
    //PIMPR_BUSINESS_ADDRESS                ,
    PIMPR_BUSINESS_ADDRESS_STREET         ,
    PIMPR_BUSINESS_ADDRESS_CITY           ,
    PIMPR_BUSINESS_ADDRESS_STATE          ,
    PIMPR_BUSINESS_ADDRESS_POSTAL_CODE    ,
    PIMPR_BUSINESS_ADDRESS_COUNTRY        ,
    //PIMPR_HOME_ADDRESS                    ,
    PIMPR_HOME_ADDRESS_STREET             ,
    PIMPR_HOME_ADDRESS_CITY               ,
    PIMPR_HOME_ADDRESS_STATE              ,
    PIMPR_HOME_ADDRESS_POSTAL_CODE        ,
    PIMPR_HOME_ADDRESS_COUNTRY            ,
    //PIMPR_OTHER_ADDRESS                   ,
    PIMPR_OTHER_ADDRESS_STREET            ,
    PIMPR_OTHER_ADDRESS_CITY              ,
    PIMPR_OTHER_ADDRESS_STATE             ,
    PIMPR_OTHER_ADDRESS_POSTAL_CODE       ,
    PIMPR_OTHER_ADDRESS_COUNTRY           ,
    PIMPR_BIRTHDAY                        ,
    PIMPR_ANNIVERSARY                     ,
    //PIMPR_SMARTPROP                       ,
    PIMPR_PICTURE                         ,
    //PIMPR_YOMI_FILEAS                     ,
    //PIMPR_SIM_PHONE                      , 
    PIMPR_CATEGORIES                      ,
    //PIMPR_BODY_BINARY                    ,
    PIMPR_BODY_TEXT                       

};



/**
 * Array of contact fields names for SIF format.
 * @note Don't change the order of these properties!
 *       This list MUST be synchronized with PIMPR_props array of CEPROPID fields.
 */
const wchar_t* contactFields [] = {    
    L"FileAs"                          ,
    L"Title"                           ,    
    L"FirstName"                       ,                // Returns or sets the first name for the contact.
    L"MiddleName"                     ,
    L"LastName"                       ,
    L"Suffix"                          ,
    L"NickName"                        ,
    L"YomiFirstName"                  ,
    L"YomiLastName"                   ,
    L"YomiCompanyName"                    ,
    L"CompanyName"                    ,
    L"Department"                      ,
    L"JobTitle"                       ,
    L"ManagerName"                         ,
    L"OfficeLocation"                 ,
    L"AssistantName"                  ,
    L"Email1Address"                  ,
    L"Email2Address"                  ,
    L"Email3Address"                  ,
    L"IMAddress"                     ,
    L"IM2Address"                     ,
    L"IM3Address"                     ,
    L"MobileTelephoneNumber"         ,
    L"BusinessTelephoneNumber"       ,
    L"Business2TelephoneNumber"      ,
    L"HomeTelephoneNumber"           ,
    L"Home2TelephoneNumber"          ,
    L"BusinessFaxNumber"             ,
    L"HomeFaxNumber"                 ,
    L"PagerNumber"                    ,
    L"CarTelephoneNumber"            ,
    L"RadioTelephoneNumber"          ,
    L"CompanyMainTelephoneNumber"        ,
    L"AssistantTelephoneNumber"      ,
    //PIMPR_SMS                             ,
    //PIMPR_MMS                             ,
    //PIMPR_DISPLAY_NAME                    ,
    L"Spouse"                          ,
    L"Children"                        ,
    L"WebPage"                        ,         // outlook WebPage
    //PIMPR_RINGTONE                        ,
    L"CustomerID"                      ,
    L"GovernmentIDNumber"                    ,
    //PIMPR_ACCOUNT_NAME                    ,
    //PIMPR_BUSINESS_ADDRESS                ,
    L"BusinessAddressStreet"         ,
    L"BusinessAddressCity"           ,
    L"BusinessAddressState"          ,
    L"BusinessAddressPostalCode"    ,
    L"BusinessAddressCountry"        ,
    //PIMPR_HOME_ADDRESS                    ,
    L"HomeAddressStreet"             ,
    L"HomeAddressCity"               ,
    L"HomeAddressState"              ,
    L"HomeAddressPostalCode"        ,
    L"HomeAddressCountry"            ,
    //PIMPR_OTHER_ADDRESS                   ,
    L"OtherAddressStreet"            ,
    L"OtherAddressCity"              ,
    L"OtherAddressState"             ,
    L"OtherAddressPostalCode"       ,
    L"OtherAddressCountry"           ,
    L"Birthday"                        ,
    L"Anniversary"                     ,
    //PIMPR_SMARTPROP                       ,
    L"Photo"                         ,// use this not associated to the property. ADDED MANUALLY!! PIMPR_PICTURE 
    //PIMPR_YOMI_FILEAS                     ,
    //PIMPR_SIM_PHONE                   ,
    L"Categories"                     ,
    //PIMPR_BODY_BINARY                    ,
    L"Body"                       ,
    NULL
};



/*
* Method to populate the contact XML structure.
* It get every property name and value and build the XML structure to exchange with server.
*
* @param contactStringItem  : the string that will contain the xml structure
* @param pAppointment       : the contact object that contain every value
*
*/
void populateContactStringItem(wstring &contactStringItem, IContact *pContact) {

    BSTR element = NULL;
    DATE date = NULL;
    wchar_t tempDate[50];
    wstring element_s;

    contactStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    contactStringItem += TEXT("<contact>");

    pContact->get_FirstName(&element);
    element_s.clear();
    element_s = wstring(element);
    encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<FirstName>") + element_s + TEXT("</FirstName>");
    SysFreeString(element);

    pContact->get_LastName(&element);
    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<LastName>") + element_s + TEXT("</LastName>");
    SysFreeString(element);

    pContact->get_CompanyName(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<CompanyName>") + element_s + TEXT("</CompanyName>");
    SysFreeString(element);

    pContact->get_BusinessFaxNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessFaxNumber>") + element_s + TEXT("</BusinessFaxNumber>");
    SysFreeString(element);

    pContact->get_Department(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Department>") + element_s + TEXT("</Department>");
    SysFreeString(element);

    pContact->get_Email1Address(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Email1Address>") + element_s + TEXT("</Email1Address>");
    SysFreeString(element);

    pContact->get_MobileTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<MobileTelephoneNumber>") + element_s + TEXT("</MobileTelephoneNumber>");
    SysFreeString(element);

    pContact->get_OfficeLocation(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OfficeLocation>") + element_s + TEXT("</OfficeLocation>");
    SysFreeString(element);

    pContact->get_PagerNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<PagerNumber>") + element_s + TEXT("</PagerNumber>");
    SysFreeString(element);

    pContact->get_BusinessTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessTelephoneNumber>") + element_s + TEXT("</BusinessTelephoneNumber>");
    SysFreeString(element);

    pContact->get_JobTitle(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<JobTitle>") + element_s + TEXT("</JobTitle>");
    SysFreeString(element);

    pContact->get_HomeTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeTelephoneNumber>") + element_s + TEXT("</HomeTelephoneNumber>");
    SysFreeString(element);

    pContact->get_Email2Address(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Email2Address>") + element_s + TEXT("</Email2Address>");
    SysFreeString(element);

    pContact->get_Spouse(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Spouse>") + element_s + TEXT("</Spouse>");
    SysFreeString(element);

    pContact->get_Email3Address(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Email3Address>") + element_s + TEXT("</Email3Address>");
    SysFreeString(element);

    pContact->get_Home2TelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Home2TelephoneNumber>") + element_s + TEXT("</Home2TelephoneNumber>");
    SysFreeString(element);

    pContact->get_HomeFaxNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeFaxNumber>") + element_s + TEXT("</HomeFaxNumber>");
    SysFreeString(element);

    pContact->get_CarTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<CarTelephoneNumber>") + element_s + TEXT("</CarTelephoneNumber>");
    SysFreeString(element);

    pContact->get_AssistantName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<AssistantName>") + element_s + TEXT("</AssistantName>");
    SysFreeString(element);

    pContact->get_AssistantTelephoneNumber(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<AssistantTelephoneNumber>") + element_s + TEXT("</AssistantTelephoneNumber>");
    SysFreeString(element);

    pContact->get_Children(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Children>") + element_s + TEXT("</Children>");
    SysFreeString(element);

    pContact->get_Categories(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Categories>") + element_s + TEXT("</Categories>");
    SysFreeString(element);

    pContact->get_WebPage(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<WebPage>") + element_s + TEXT("</WebPage>");
    SysFreeString(element);

    pContact->get_Business2TelephoneNumber(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Business2TelephoneNumber>") + element_s + TEXT("</Business2TelephoneNumber>");
    SysFreeString(element);

    pContact->get_RadioTelephoneNumber(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<RadioTelephoneNumber>") + element_s + TEXT("</RadioTelephoneNumber>");
    SysFreeString(element);

    pContact->get_FileAs(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<FileAs>") + element_s + TEXT("</FileAs>");
    SysFreeString(element);

    pContact->get_YomiCompanyName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<YomiCompanyName>") + element_s + TEXT("</YomiCompanyName>");
    SysFreeString(element);

    pContact->get_YomiFirstName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<YomiFirstName>") + element_s + TEXT("</YomiFirstName>");
    SysFreeString(element);

    pContact->get_YomiLastName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<YomiLastName>") + element_s + TEXT("</YomiLastName>");
    SysFreeString(element);

    pContact->get_Title(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Title>") + element_s + TEXT("</Title>");
    SysFreeString(element);

    pContact->get_MiddleName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<MiddleName>") + element_s + TEXT("</MiddleName>");
    SysFreeString(element);

    pContact->get_Suffix(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Suffix>") + element_s + TEXT("</Suffix>");
    SysFreeString(element);

    pContact->get_HomeAddressStreet(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressStreet>") + element_s + TEXT("</HomeAddressStreet>");
    SysFreeString(element);

    pContact->get_HomeAddressCity(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressCity>") + element_s + TEXT("</HomeAddressCity>");
    SysFreeString(element);

    pContact->get_HomeAddressState(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressState>") + element_s + TEXT("</HomeAddressState>");
    SysFreeString(element);

    pContact->get_HomeAddressPostalCode(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressPostalCode>") + element_s + TEXT("</HomeAddressPostalCode>");
    SysFreeString(element);

    pContact->get_HomeAddressCountry(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressCountry>") + element_s + TEXT("</HomeAddressCountry>");
    SysFreeString(element);

    pContact->get_OtherAddressStreet(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressStreet>") + element_s + TEXT("</OtherAddressStreet>");
    SysFreeString(element);

    pContact->get_OtherAddressCity(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressCity>") + element_s + TEXT("</OtherAddressCity>");
    SysFreeString(element);

    pContact->get_OtherAddressPostalCode(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressPostalCode>") + element_s + TEXT("</OtherAddressPostalCode>");
    SysFreeString(element);

    pContact->get_OtherAddressCountry(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressCountry>") + element_s + TEXT("</OtherAddressCountry>");
    SysFreeString(element);

    pContact->get_OtherAddressState(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressState>") + element_s + TEXT("</OtherAddressState>");
    SysFreeString(element);

    pContact->get_BusinessAddressStreet(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressStreet>") + element_s + TEXT("</BusinessAddressStreet>");
    SysFreeString(element);

    pContact->get_BusinessAddressCity(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressCity>") + element_s + TEXT("</BusinessAddressCity>");
    SysFreeString(element);

    pContact->get_BusinessAddressState(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressState>") + element_s + TEXT("</BusinessAddressState>");
    SysFreeString(element);

    pContact->get_BusinessAddressPostalCode(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressPostalCode>") + element_s + TEXT("</BusinessAddressPostalCode>");
    SysFreeString(element);

    pContact->get_BusinessAddressCountry(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressCountry>") + element_s + TEXT("</BusinessAddressCountry>");
    SysFreeString(element);

    pContact->get_Body(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Body>") + element_s + TEXT("</Body>");
    SysFreeString(element);

    pContact->get_Birthday(&date);

    wsprintf(tempDate, TEXT(""));

    double toCompare = MAX_DATE_DOUBLE;

    if (date != toCompare) { // MAX_DATE_DOUBLE is set to 949999.00000, the nonsense value when in ppc date is not set...
        doubleToSystemTimeBirthday(tempDate, date);
        //doubleToSystemTimeHourZero(tempDate, date, TRUE);
        // doubleToSystemTime(tempDate, date);
    }
    //contactStringItem +=  TEXT("<Birthday>") + tempDate + TEXT("</Birthday>");
    contactStringItem +=  TEXT("<Birthday>");  contactStringItem += tempDate; contactStringItem += TEXT("</Birthday>");

    pContact->get_Anniversary(&date);
    wsprintf(tempDate, TEXT(""));
    if (date != toCompare) {
        doubleToSystemTimeBirthday(tempDate, date);
        // doubleToSystemTimeHourZero(tempDate, date, TRUE);
        // doubleToSystemTime(tempDate, date);
    }
    //contactStringItem +=  TEXT("<Anniversary>") + tempDate + TEXT("</Anniversary>");

    contactStringItem +=  TEXT("<Anniversary>"); contactStringItem += tempDate; contactStringItem += TEXT("</Anniversary>");

    contactStringItem +=  TEXT("</contact>");

}



/*
* The method calculate the CRC hash value for the contact item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pContact : the contact object that contain every value
*/

long calculateContactHash (IContact* pContact) {

    wstring p;

    populateContactStringItem(p, pContact);
    long l = stringCrc32(p.data());

    p.clear();
    return l;

}






int writeToTempFile(FILE* f, char *ptr, int numBytes) {

   int numWritten = fwrite(ptr, sizeof(char), numBytes, f);
   return numWritten;

}



/*
* Use the WinContact object provided by the C++ API.
* This is only for Windows Mobile 5 devices. It uses windows
* api that are a low level.
*/

WinItem* contactToWinContact(IItem *pContact, bool isSif) {

    HRESULT hr                = E_FAIL;
    CEPROPVAL *prgPropvalPoom = NULL;
    ULONG cbBuffer            = 0;
    HANDLE hHeap              = GetProcessHeap();
    wstring value, imagew = L"";

    WinContact* winC          = NULL;
    if (isSif) { winC = new WinContactSIF(); } 
    else       { winC = new WinContact();    }

    cbBuffer = 0;
    hr = pContact->GetProps(PIMPR_props, CEDB_ALLOWREALLOC, NUM_COLS, &prgPropvalPoom, &cbBuffer, hHeap);

    if(FAILED(hr) || NULL == prgPropvalPoom || 0 == cbBuffer) {
        goto Exit;
    }

    for (int i = 0; i < NUM_COLS; i++) {

        if (i == ePIMPR_BIRTHDAY || i == ePIMPR_ANNIVERSARY) {
            value = getDatePropertyFromMapi(&prgPropvalPoom[i], false, true);
        }
        else if (i == ePIMPR_PICTURE) {
            // currently it is set empty because if there is an image it will be
            // set later with a value
            value = L"";
        }
        else {
            value = getStringPropertyFromMapi(&prgPropvalPoom[i]);
        }
        winC->setProperty(contactFields[i], value);
    }

    // to get the picture of the contact
    imagew = getPicturePropertyFromMapi(pContact, *winC);
    winC->setProperty(PICTURE, imagew);

    hr = S_OK;

Exit:
    // Free memory.
    HeapFree(hHeap, 0, prgPropvalPoom);
    return (WinItem*)winC;

}




HRESULT winContactToContact(WinContact& winC, IItem *pContact) {

    //Handle the picture separately
    wstring picture;
    if (winC.getProperty(PICTURE, picture)) {
        winC.removeElement(PICTURE);
    }
    HRESULT hr                = E_FAIL;
    int numberOfProperty      = winC.getPropertyMapSize();
    int incNumProp            = 0;
    CEPROPVAL *prgPropvalPoom = new CEPROPVAL[numberOfProperty];

    FILETIME date;
    bool exists = false;
    int i = 0;
    for (i = 0; i < NUM_COLS; i++) {
        exists = false;
        wstring& value = winC.getPropertyRef(contactFields[i], &exists);

        if (exists) {
            prgPropvalPoom[incNumProp].wFlags   = 0;
            prgPropvalPoom[incNumProp].wLenData = 0;

            if (i == ePIMPR_BIRTHDAY || i == ePIMPR_ANNIVERSARY) {
                prgPropvalPoom[incNumProp].propid = PIMPR_props[i];
                date = getDatePropertyFromString(value, true);
                prgPropvalPoom[incNumProp].val.filetime = date;
            }
            else if (i == ePIMPR_PICTURE) {
                // Picture is handled later. BTW it should not be here because it is removed
                // before
                incNumProp--;
            }
            else if (i == ePIMPR_BODY_TEXT) {
                convertNewlines(value);
                prgPropvalPoom[incNumProp].propid = PIMPR_props[i];
                prgPropvalPoom[incNumProp].val.lpwstr = (wchar_t*)value.c_str();
            }
            else {
                prgPropvalPoom[incNumProp].propid = PIMPR_props[i];
                prgPropvalPoom[incNumProp].val.lpwstr = (wchar_t*)value.c_str();
            }
            incNumProp++;
        }
    }

    LOG.debug("Setting %d properties into IItem...", incNumProp);
    hr = pContact->SetProps(0, incNumProp, prgPropvalPoom);
    if (FAILED(hr)) {
        LOG.error("Error setting contact properties. Please check if incoming item is correct.");
        goto finally;
    }
    hr = pContact->Save();
    if (FAILED(hr)) {
        LOG.error("Error saving contact.");
        goto finally;
    }

    //
    // Insert the image if exists
    //
    IStream* pStream = NULL;
    ULONG cbWritten = 0;
    if (picture != L"") {
        const wstring charsToRemove = L" \r\n";     // remove spaces, '\r' and '\n'
        size_t len = 0;
        int idx;
        while( (idx=picture.find_first_of(charsToRemove)) >= 0 ) {
            picture.replace( idx, 1, L"" );
        }

        char* valueA = toMultibyte(picture.c_str());
        int rc = b64_decode(valueA, valueA);
        if(rc > 0) {
            hr = pContact->OpenProperty(PIMPR_PICTURE, GENERIC_READ|GENERIC_WRITE, &pStream);
            hr = pStream->Write(valueA, rc, &cbWritten);
            pStream->Commit(STGC_DEFAULT);
            pStream->Release();
        }
        if (valueA) { delete [] valueA;}
    }

    else {
        // Delete existing image (set stream size to 0)
        hr = pContact->OpenProperty(PIMPR_PICTURE, GENERIC_READ|GENERIC_WRITE, &pStream);
        if (pStream && SUCCEEDED(hr)) {
            ULARGE_INTEGER size;
            size.HighPart = size.LowPart = 0;
            hr = pStream->SetSize(size);

            hr = pStream->Commit(STGC_DEFAULT);
            pStream->Release();
        }
    }

    hr = S_OK;

finally:
    if (prgPropvalPoom) {
        delete [] prgPropvalPoom;
    }
    return hr;
}



/*
 * Encode the message in base64, splitting the result in lines of 72 columns
 * each.
 */
char* encodeWithSpaces(const char *msg, int len)
{
    int i, step=54, dlen=0;

    char* res = new char[len*3]; // b64 is 4/3, but we have also the newlines....
    memset(res, 0, len*3);
    res[0] = ' ';
    res[1] = ' ';
    res[2] = ' ';
    res[3] = ' ';
    char* ret = &res[4];
    for(i=0; i<len; i+=step) {
        if(len-i < step) {
            step = len-i;
        }
        dlen += b64_encode(ret+dlen, (void *)(msg+i), step);
        ret[dlen++]='\r';
        ret[dlen++]='\n';
        ret[dlen++]=' ';
        ret[dlen++]=' ';
        ret[dlen++]=' ';
        ret[dlen++]=' ';
    }

    // Terminate the string
    ret[dlen]=0;
    int ll = strlen(res);
    return res;
}
char* loadFromFileAndConvert(const char *filename, const char *encoding)
{
    char *msg = 0;
    bool binary = true;
    size_t msglen=0;
    char *ret = 0;

    if(!filename)
        return 0;

    if( strcmp(encoding, "base64") == 0 ) {
        binary = true;
    }

    // Read file
    if(!readFile(filename, &msg, &msglen, binary))
        return 0;
    // Encode the file
    if( strcmp(encoding, "base64") == 0 ) {
        ret = encodeWithSpaces(msg, msglen);
        delete [] msg;
    }
    return ret;
}


ArrayList* getContactProperties(){

    int i = 0;
    ArrayList* props = new ArrayList();

    while (contactFields[i]){
        Property prop;
        char* field = toMultibyte(contactFields[i]);
        prop.setPropName(field);
        props->add(prop);
        
        delete [] field;

        i++;
    }

    return props;
}

ArrayList* getVCardProperties(){

    ArrayList* props = new ArrayList();
    Property* p = new Property();
    PropParam* pp = new PropParam();
    ArrayList pparams;
    ArrayList valenums;
    StringBuffer val;


    p->setPropName("BEGIN");
    val = "VCARD";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);

    valenums.clear();

    p->setPropName("END");
    val = "VCARD";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);

    valenums.clear();
    
    p->setPropName("VERSION");
    val = "2.1";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);

    p->setPropName("X-FUNAMBOL-ANNIVERSARY");
    props->add(*p);
    
    p->setPropName("BDAY");
    props->add(*p);
    
    p->setPropName("NOTE");
    props->add(*p);
    
    p->setPropName("TEL");
    pp->setParamName("VOICE");
    pparams.add(*pp);
    pp->setParamName("WORK");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("ADR");
    pp->setParamName("WORK");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("TEL");
    pp->setParamName("WORK");
    pparams.add(*pp);
    pp->setParamName("FAX");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("TEL");
    pp->setParamName("CAR");
    pparams.add(*pp);
    pp->setParamName("VOICE");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("CATEGORIES");
    props->add(*p);

    p->setPropName("X-FUNAMBOL-CHILDREN");
    props->add(*p);

    p->setPropName("TEL");
    pp->setParamName("WORK");
    pparams.add(*pp);
    pp->setParamName("PREF");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("ORG");
    props->add(*p);

    p->setPropName("X-FUNAMBOL-CUSTOMERID");
    props->add(*p);

    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    pp->setParamName("HOME");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    pp->setParamName("WORK");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("ADR");
    pp->setParamName("HOME");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("TEL");
    pp->setParamName("HOME");
    pparams.add(*pp);
    pp->setParamName("FAX");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("TEL");
    pp->setParamName("VOICE");
    pparams.add(*pp);
    pp->setParamName("HOME");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    pp->setParamName("HOME");
    pparams.add(*pp);
    pp->setParamName("X-FUNAMBOL-INSTANTMESSENGER");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("TITLE");
    props->add(*p);

    p->setPropName("N");
    props->add(*p);

    p->setPropName("X-FUNAMBOL-MANAGER");
    props->add(*p);

    p->setPropName("TEL");
    pp->setParamName("CELL");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("NICKNAME");
    props->add(*p);

    p->setPropName("ADR");
    props->add(*p);

    p->setPropName("TEL");
    pp->setParamName("PAGER");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("PHOTO");
    props->add(*p);

    p->setPropName("TEL");
    pp->setParamName("X-FUNAMBOL-RADIO");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);

    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();

    p->setPropName("X-FUNAMBOL-SPOUSE");
    props->add(*p);

    p->setPropName("URL");
    props->add(*p);

    p->setPropName("X-FUNAMBOL-YOMICOMPANYNAME");
    props->add(*p);

    p->setPropName("X-FUNAMBOL-YOMIFIRSTNAME");
    props->add(*p);

    p->setPropName("X-FUNAMBOL-YOMILASTNAME");
    props->add(*p);

    delete p; p = NULL;
    delete pp; pp = NULL;

    return props;
}