/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

//
// @author Marco Magistrali
//
#include <windows.h>
#include "Winbase.h"
#include "pim/Utils.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/FILEFileManagement.h"
#include "pim/SettingFunctions.h"
#include "pim/ClientSettings.h"

///// **********************************
//#include "RichInk.h"
// Defines and typedefs needed for the conversion:

#define EM_STREAMIN          (WM_USER + 73)
#define EM_STREAMOUT         (WM_USER + 74)

typedef DWORD (CALLBACK *EDITSTREAMCALLBACK)
      (DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);

typedef struct _editstream
{
    DWORD dwCookie; /* user value passed to callback as first parameter */
    DWORD dwError;  /* last error */
    EDITSTREAMCALLBACK pfnCallback;
} EDITSTREAM;

typedef struct tagCOOKIE
{
    HANDLE hFile;
    LPBYTE pbStart;
    LPBYTE pbCur;
    LONG  bCount;
    DWORD dwError;
} COOKIE, * PCOOKIE;

// Stream formats:
#define SF_TEXT         0x0001
#define SF_RTF          0x0002
#define SF_UNICODE      0x0010      // Unicode data of some kind
#define SF_UTEXT        SF_TEXT | SF_UNICODE   // Unicode text file
#define SFF_PWI         0x0800
#define SF_PWI          ( SF_RTF | SFF_PWI | 0x010000 ) // PWI format

static int recursiveLevel = 1;
static int currentLevel   = 0;


static DWORD CALLBACK ReadCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb) {

    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    // Zero the bytes read.

    *pcb = 0;

    // If there is an error return it.
    if (ReadFile(pCookie->hFile, pbBuff, cb, (unsigned long *)pcb, 0) == 0)

    return (pCookie->dwError = GetLastError());

    // Return no error.
    return 0;

} // ReadCallback()

static DWORD CALLBACK  WriteCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb) {
    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    if (WriteFile(pCookie->hFile, pbBuff, cb,(unsigned long *) pcb, 0) == 0) {
        return (pCookie->dwError = GetLastError());
    } else {
        }

    return 0;
} // WriteCallback()


static DWORD CALLBACK  BufferReadCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    LONG   bytesLeft;
    LONG   bytesRead;

    // Calc the bytes read.
    bytesRead = pCookie->pbCur - pCookie->pbStart;

    // Calc bytes left to read.
    if (bytesRead < pCookie->bCount) {
      // Calc the bytes left to read.
        bytesLeft = pCookie->bCount - bytesRead;
    } else{
        bytesLeft = 0;
    }

    if (cb > bytesLeft)
        cb = bytesLeft;

    *pcb = cb;
    // Copy any bytes.
    if (cb) {
        memcpy(pbBuff, pCookie->pbCur, cb);
        pCookie->pbCur += cb;
    }

    return 0;
} // BufferReadCallback()

static DWORD CALLBACK BufferWriteCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    // Do not overwrite the end of the buffer.
    // If there is no output buffer, then were
    // only here to determine the space required
    // for the stream out.

    if (pCookie->pbStart && (pCookie->pbCur + cb > pCookie->pbStart + pCookie->bCount)) {
        // Writing all this data would overflow the buffer.
        // So only write what will fit.
        cb = pCookie->pbStart + pCookie->bCount - pCookie->pbCur;
    }

    *pcb = cb;

    if (pCookie->pbStart)
        memcpy(pCookie->pbCur, pbBuff, cb); pCookie->pbCur += cb;
    return 0;
} // BufferWriteCallback()


int writeTextToFile(const wchar_t* filename, const wchar_t* text) {

    HINSTANCE   hLib;
    HWND        hwndInk     = NULL;
    LPTSTR      lpszClass   = TEXT("richink") ;
    EDITSTREAM  es;
    COOKIE      cookie;
    HLOCAL      sz;
    UINT        cbSz;
    
    HANDLE file = CreateFile(filename, GENERIC_WRITE, 0, //FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

    DWORD out = 0;
    if (wcscmp(text, TEXT("")) == 0) {
        BOOL ret = WriteFile(file, "", 1, &out, NULL);
        return ret;
    }

    hLib = LoadLibrary( TEXT("richink.dll") );
    if (!hLib)
      return -1;

    hwndInk = CreateWindow(lpszClass, NULL, WS_BORDER, 0, 0, 0, 0, NULL, 0, NULL, NULL);
    if (!hwndInk)
      return -1;

    /* write back to note */

    memset(&cookie, 0, sizeof(cookie));
    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)text;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       =  wcslen(text) * sizeof(wchar_t);
    // cookie.bCount   =  5 * sizeof(wchar_t);
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferReadCallback;
    SendMessage (hwndInk, EM_STREAMIN, (WPARAM)SF_UTEXT, (LPARAM)&es);

    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = 0;
    cookie.bCount       = 0;
    cookie.hFile        = file;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_PWI, (LPARAM)&es);

    sz = (LPWSTR)LocalAlloc(LPTR, (cbSz = cookie.pbCur - cookie.pbStart) + 2);

    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)sz;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = cbSz;
    //cookie.hFile = file;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = WriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_PWI, (LPARAM)&es);

    CloseHandle(file);

    return 0;
}

wchar_t* readTextFromBinaryFile(wchar_t* filename) {

    HINSTANCE   hLib;
    HWND   hwndInk = NULL;
    LPTSTR lpszClass = TEXT("richink") ;
    EDITSTREAM   es;
    COOKIE      cookie;
    HLOCAL sz;
    UINT cbSz;
    wchar_t* ret = NULL;

    // open file if only if exists
    HANDLE hfile = CreateFile(filename, GENERIC_READ, 0,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    hLib = LoadLibrary( TEXT("richink.dll") );
    if (!hLib)
      return NULL;

    hwndInk = CreateWindow(lpszClass, NULL, WS_BORDER, 0, 0, 0, 0, NULL, 0, NULL, NULL);

    if (!hwndInk)
      return NULL;

    // Stream in the Ink Data from a memory block.
    memset (&cookie, 0, sizeof(cookie));
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = 0;
    cookie.hFile        = hfile;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = ReadCallback;
    SendMessage (hwndInk, EM_STREAMIN, (WPARAM)SF_PWI, (LPARAM)&es);

    // Next call stream out to get the size of the Unicode text.
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = 0;
    cookie.bCount       = 0;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);

    // Allocate memory and stream out the Unicode text to it.
    sz = (LPWSTR)LocalAlloc(LPTR, (cbSz = cookie.pbCur - cookie.pbStart) + 2);

    // Check for alloc err.
    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)sz;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = cbSz;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);

    ret = toWideChar((char*)sz, "ascii"); // TODO: is it possible to avoid some casts?

    CloseHandle(hfile);

    return ret;
}


void writeTextToTextFile(wchar_t* fName, wchar_t* text) {

    if (text == NULL)
        return;

    FILE* f;

    // specific for favorites. If file doesn't end with .url, it is a directory. So create it.
    if (wcsstr(fName, URL_EXTENSION) == NULL) {
        if (CreateDirectory(fName, NULL) == TRUE) {
        } else {
            LOG.error("Dir NOT correctly created");
        }
        return;
    }

    f = _wfopen(fName, TEXT("w+"));

    if( f != NULL ) {
        fwprintf(f, text);
    } else {
        wchar_t localDir     [DIM_LOCAL_TEMP];
        wchar_t internalPath [DIM_LOCAL_TEMP];
        getDirName(fName, localDir);

        SHGetSpecialFolderPath(NULL, internalPath, CSIDL_FAVORITES, FALSE);
        wcscat(internalPath, TEXT("\\"));
        wcscat(internalPath, localDir);

        CreateDirectory(internalPath, NULL);

        f = _wfopen(fName, TEXT("w+"));
        fwprintf(f, text);

    }

    fflush(f);
    fclose(f);
}


void writeByteToFile(wchar_t* fName, byte *ptr, int numBytes) {

    wchar_t wfilename[DIM_FILE];
    wchar_t* pt = NULL;
    char filename [DIM_FILE];
    BOOL usedDefaultChar;

    wcscpy(wfilename, fName);

    WideCharToMultiByte (CP_ACP, 0, wfilename, DIM_FILE -1, filename, DIM_FILE -1, "?", &usedDefaultChar);

    int numwritten = 0;
    FILE* f;

    f = fopen(filename, "w+b");

    if( f != NULL ) {
        numwritten = fwrite( ptr, sizeof( byte ), numBytes, f );
    }

    fflush(f);
    fclose(f);
}

void readByteFromFile(wchar_t* fName, byte *ptr, int* numBytes) {

    wchar_t wfilename[DIM_FILE];
    wchar_t* pt = NULL;
    char filename [DIM_FILE];
    BOOL usedDefaultChar;

    wcscpy(wfilename, fName);

    WideCharToMultiByte (CP_ACP, 0, wfilename, DIM_FILE -1, filename, DIM_FILE -1, "?", &usedDefaultChar);

    wchar_t* element = NULL;

    byte line[DIMLINE];
    int numread = 0;
    int total = 0;
    FILE* f;

    f = fopen(filename, "rb");

    while( !feof( f ) ) {

        numread = fread( line, sizeof( byte ), DIMLINE, f );

        memcpy(ptr + total, line, numread);
        total = total + numread;
    }

    fflush(f);
    fclose(f);

    *numBytes = total;

}

/*
* Method to write current file name and time of writing in file to use to
* compare with others in order to search for modified files...
*/


void writeCurrentFileItems(int dataType, const wchar_t* path, const wchar_t* dir) {

    wchar_t localTemp[DIM_LOCAL_TEMP];
    wchar_t wfilename[DIM_FILE];
    wchar_t internalPath[DIM_LOCAL_TEMP];
    wchar_t filterPattern[256];

    wchar_t** fileList = NULL;
    int count = 0;
    int i = 0;
    FILE* f;

    /*
    * It avoids to send object greater then MaxObjSize.
    * In next releases it must me inserted into the
    * source.
    */
    //ClientSettings cs(APPLICATION_URI);
    //cs.readConfig();
    int fileRemoved = 0;

    BOOL recursiveInDirectory = FALSE;
    filterPattern [0] = 0;

    if (dataType == constFiles) {
        wstring complete = appendToPath((wchar_t*)dir, NULL);
        wcscpy(internalPath, complete.c_str()); 
        // wsprintf(internalPath, TEXT("%s\\"), dir);
        wsprintf(wfilename, TEXT("%s\\%s"), path, TEXT(FILENAME_FILES));
    }
    else if (dataType == constNotes) {
        wstring complete = appendToPath((wchar_t*)dir, NULL);
        wcscpy(internalPath, complete.c_str()); 
        // wsprintf(internalPath, TEXT("%s\\"), dir);
        wsprintf(wfilename, TEXT("%s\\%s"), path, TEXT(FILENAME_NOTES));
        wcscpy(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {
        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wstring complete = appendToPath((wchar_t*)localTemp, NULL);
        wcscpy(internalPath, complete.c_str()); 
        //wsprintf(internalPath, TEXT("%s\\"), localTemp);
        wsprintf(wfilename, TEXT("%s\\%s"), path, TEXT(FILENAME_FAVORITES));
        recursiveInDirectory = TRUE;
    }

    f = _wfopen(wfilename, TEXT("w+"));

    getFileList(internalPath, fileList, &count, TRUE, recursiveInDirectory, filterPattern);
    fileList = new wchar_t*[count];
    int m = 0;
    for (m = 0; m < count ; m++) {
        fileList[m] = NULL;
    }
    // to retrieve fileName
    getFileList(internalPath, fileList, &count, FALSE, recursiveInDirectory, filterPattern);

    wsprintf(localTemp, TEXT("%f\n"), getSystemTime());
    fwprintf(f, TEXT("%s"), localTemp);

    for (i = 0; i < count; i++) {
        unsigned long sizeFile = 0;
        getFileSize(fileList[i], &sizeFile);
        if (sizeFile > getMaxObjectSizeToSend()) {
            LOG.debug("writeCurrentFileItems: found file greater then maxObjectSize(%lu): sizefile = %lu", getMaxObjectSizeToSend(), sizeFile);
            continue;
        }

        fwprintf(f, TEXT("%s"), fileList[i]);
        fwprintf(f, TEXT("\n"));
    }
    fflush(f);
    fclose(f);


    for (i = 0; i < count; i++) {
        if (fileList[i] != NULL)
            delete [] fileList[i];
    }

    if (fileList != NULL)
        delete [] fileList;

}

int readFilenameFromFile(int dataType, const wchar_t* path, wchar_t*** ptrArrayFilename) {

    wchar_t wfilename [DIM_FILE];
    wchar_t pt        [DIM_FILE];
    wchar_t** ptr     = NULL;
    int i             = 0;

    if (dataType == constFiles)
        wsprintf(pt, TEXT(FILENAME_FILES));
    else if (dataType == constNotes)
        wsprintf(pt, TEXT(FILENAME_NOTES));
    else if (dataType == constFavorites)
        wsprintf(pt, TEXT(FILENAME_FAVORITES));

    wsprintf(wfilename, TEXT("%s\\%s"),  path, pt);

    wchar_t* element = NULL;
    wchar_t line[256];
    FILE* f;
    wchar_t* tmp = NULL;

    f = _wfopen(wfilename, TEXT("r"));

    if (f == NULL) {
        return 0;
    }

    while(fgetws(line, 255, f) != NULL) {
        i++;
    }
    fflush(f);

    ptr = new wchar_t* [i];

    i = 0;
    f = _wfreopen(wfilename, TEXT("r"), f);

    while(fgetws(line, 255, f) != NULL) {

        ptr[i] = new wchar_t[255];
        wsprintf(ptr[i], TEXT("%s"), line);

        //
        // to remove carriage/return at the end of file. CarriageReturn was introduced
        // by method writeCurrentFileItems to insert new line
        //
        removeEndCarriage(&ptr[i]);

        i++;
    }
    fflush(f);
    fclose(f);

    *ptrArrayFilename = ptr;
    return i - 1;
}
/*
* get file size
*/

BOOL getFileSize(wchar_t* filename, unsigned long *size) {


   HANDLE file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

   DWORD lpFileSizeHigh;
   DWORD dwSize;

   if( file ) {
        dwSize = GetFileSize(file, &lpFileSizeHigh);

        if (dwSize != 0xFFFFFFFF) {
            *size = dwSize;

        }
        else {
            *size = 0;
        }
   }

   if( file ) CloseHandle( file );

   return TRUE;
}

/*
*
*/

BOOL getLastWriteTime(wchar_t* filename, wchar_t* wtime, DATE* dtime) {


    HANDLE file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    // FILETIME time;

    FILETIME ftCreate, ftAccess, ftWrite, stUTC;
    SYSTEMTIME  stLocal;
    BOOL isUTC = TRUE;

    //wchar_t propertyValue[10];
    //getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME, propertyValue, FALSE);

    //if (wcscmp(propertyValue, TEXT("1")) == 0) {
    //    isUTC = TRUE;
    //}

    if( file ) {

        if (!GetFileTime(file, &ftCreate, &ftAccess, &ftWrite)) {
            return FALSE;
        }
        else {
        }

        if (isUTC) {
            FileTimeToSystemTime(&ftWrite, &stLocal);
        }
        else {
            FileTimeToLocalFileTime(&ftWrite, &stUTC);
            FileTimeToSystemTime   (&stUTC, &stLocal);
        }

        wchar_t month [10];
        wchar_t day   [10];
        wchar_t hour  [10];
        wchar_t minute[10];
        wchar_t second[10];

        // month
        if (stLocal.wMonth < 10)
            wsprintf(month, TEXT("0%i"), stLocal.wMonth);
        else
            wsprintf(month, TEXT("%i"), stLocal.wMonth);

        // day
        if (stLocal.wDay  < 10)
            wsprintf(day, TEXT("0%i"), stLocal.wDay);
        else
            wsprintf(day, TEXT("%i"), stLocal.wDay);

        // hour
        if (stLocal.wHour  < 10)
            wsprintf(hour, TEXT("0%i"), stLocal.wHour);
        else
            wsprintf(hour, TEXT("%i"), stLocal.wHour);

        // minute
        if (stLocal.wMinute < 10)
            wsprintf(minute, TEXT("0%i"), stLocal.wMinute);
        else
            wsprintf(minute, TEXT("%i"), stLocal.wMinute);

        // second
        if (stLocal.wSecond < 10)
            wsprintf(second, TEXT("0%i"), stLocal.wSecond);
        else
            wsprintf(second, TEXT("%i"), stLocal.wSecond);


        if (isUTC) {
            wsprintf(wtime, TEXT("%i%s%sT%s%s%sZ"), stLocal.wYear, month, day, hour, minute, second);
        } else {
            wsprintf(wtime, TEXT("%i%s%sT%s%s%s"), stLocal.wYear, month, day, hour, minute, second);
        }

        if (dtime != NULL)
            systemTimeToDouble(wtime, dtime, NULL);

   }
   else {
 }
   if( file ) CloseHandle( file );

   return TRUE;
}

BOOL getFileList(wchar_t *dir, wchar_t** ptrArray, int *count, BOOL onlyFilesNumber,
                 BOOL isRecursiveInDirectory) {

    return getFileList(dir, ptrArray, count, onlyFilesNumber, isRecursiveInDirectory, NULL);
}

/*
* method to retrieve number or filename directory dir.
* it may be recursive if isRecursiveDirectory is TRUE.
* if onlyFilesNumber is TRUE, count contains the number of file to process
*/

BOOL getFileList(wchar_t *dir, wchar_t** ptrArray, int *count, BOOL onlyFilesNumber,
                 BOOL isRecursiveInDirectory, wchar_t* filter) {

    WIN32_FIND_DATA FileData;

    HANDLE hFind;
    wchar_t localTemp [512];
    wchar_t toFind    [512];
    wchar_t szNewPath [512];
    wchar_t filterPattern [256];

    BOOL ret = TRUE;
    szNewPath[0] =   0;
    int lung = 0;

    DWORD dwAttrs;

    BOOL fFinished = FALSE;


    if (filter == NULL || wcslen(filter) == 0)
        wcscpy(filterPattern, TEXT("*.*"));
    else
        wsprintf(filterPattern, TEXT("%s"), filter);


    wsprintf(toFind, TEXT("%s%s"), dir, filterPattern);

    hFind = FindFirstFile(toFind, &FileData);

    int i = 0;

    if (hFind == INVALID_HANDLE_VALUE)
    {
        return FALSE;
    }
    else
    {
        if (onlyFilesNumber == TRUE)  {
            //only to retrieve number of files
            while (!fFinished)
            {
                wsprintf(localTemp, TEXT("%s%s"), dir, FileData.cFileName);

                dwAttrs = GetFileAttributes(localTemp);

                // if (dwAttrs & FILE_ATTRIBUTE_DIRECTORY) {
                if (dwAttrs == FILE_ATTRIBUTE_DIRECTORY) {

                    if (isRecursiveInDirectory == TRUE && currentLevel < recursiveLevel) {

                        currentLevel++;

                        wsprintf(localTemp, TEXT("%s%s%s"), dir, FileData.cFileName, TEXT("\\") );

                        int k = 0;
                        getFileList(localTemp, NULL, &k, TRUE, isRecursiveInDirectory, filterPattern);
                        i = i + k + 1; // + 1 is to count directory too!!

                        currentLevel--;

                    }

                }
                else {
                    i++;
                }
                if (!FindNextFile(hFind, &FileData))
                {
                    if (GetLastError() == ERROR_NO_MORE_FILES)     {
                        fFinished = TRUE;
                    } else {
                        return FALSE;
                    }
                }
            }
            *count = i;
        }

        else {

            while (!fFinished)
            {

                wsprintf(szNewPath, TEXT("%s%s"), dir, FileData.cFileName);

                dwAttrs = GetFileAttributes(szNewPath);

                if (dwAttrs == FILE_ATTRIBUTE_DIRECTORY) {

                    if (isRecursiveInDirectory == TRUE && currentLevel < recursiveLevel) {

                        currentLevel++;

                        wsprintf(localTemp, TEXT("%s%s%s"), dir, FileData.cFileName, TEXT("\\") );

                        int j = 0;
                        ptrArray[i] = new wchar_t[256];
                        wsprintf(ptrArray[i], TEXT("%s"), localTemp);
                        i++;
                        getFileList(localTemp, &ptrArray[i], &j, FALSE, isRecursiveInDirectory);
                        i = i + j;

                        currentLevel--;

                    }


                }
                else {
                    ptrArray[i] = new wchar_t[256];
                    wsprintf(ptrArray[i], TEXT("%s"), szNewPath);
                    i++;
                }

                wcscpy(szNewPath, TEXT(""));


                if (!FindNextFile(hFind, &FileData))
                    {
                    if (GetLastError() == ERROR_NO_MORE_FILES)
                    {

                        fFinished = TRUE;
                    }
                    else
                    {
                        return FALSE;
                    }
                }
                *count = i;
            }
        }

    // Close the search handle.

    FindClose(hFind);
    }
    return ret;

}

/*
* The key can be a file name so it can contain some special char to be encoded.
* The method get an array of wchar and overwrite the current key with the encoded one.
*/
void encodeKey(wchar_t* key) {

    if (IS_KEY_BASE64 == FALSE)
        return;

    wchar_t* encoded = NULL;
    encoded = encodeBase64UTF8(key);

    wcscpy(key, encoded);

    if (encoded != NULL)
        delete [] encoded;

}

/*
* The key can be a file name so it can contain some special char to be encoded.
* The method get an array of wchar and overwrite the current key with the encoded one.
*/
void decodeKey(wchar_t* key) {

    if (IS_KEY_BASE64 == FALSE)
        return;

    wchar_t* decoded = NULL;
    decoded = decodeBase64WUTF8(key);

    wcscpy(key, decoded);

    if (decoded != NULL)
        delete [] decoded;

}


char* readTextFromBinaryFileChar(wchar_t* filename) {

    HINSTANCE   hLib;
    HWND   hwndInk = NULL;
    LPTSTR lpszClass = TEXT("richink") ;
    EDITSTREAM   es;
    COOKIE      cookie;
    HLOCAL sz;
    UINT cbSz;
    char* ret = NULL;

    // open file if only if exists
    HANDLE hfile = CreateFile(filename, GENERIC_READ, 0,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    hLib = LoadLibrary( TEXT("richink.dll") );
    if (!hLib)
      return NULL;

    hwndInk = CreateWindow(lpszClass, NULL, WS_BORDER, 0, 0, 0, 0, NULL, 0, NULL, NULL);

    if (!hwndInk)
      return NULL;

    // Stream in the Ink Data from a memory block.
    memset (&cookie, 0, sizeof(cookie));
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = 0;
    cookie.hFile        = hfile;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = ReadCallback;
    SendMessage (hwndInk, EM_STREAMIN, (WPARAM)SF_PWI, (LPARAM)&es);

    // Next call stream out to get the size of the Unicode text.
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = 0;
    cookie.bCount       = 0;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);

    // Allocate memory and stream out the Unicode text to it.
    sz = (LPWSTR)LocalAlloc(LPTR, (cbSz = cookie.pbCur - cookie.pbStart) + 2);

    // Check for alloc err.
    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)sz;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = cbSz;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);

    wchar_t* tt = toWideChar((char*)sz, "ascii");
    ret = toMultibyte(tt);

    CloseHandle(hfile);

    if (tt) { delete [] tt; tt = NULL; }

    return ret;
}
