/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "HwndFunctions.h"

#include "tchar.h"
#include "localizationUtils.h"
#include "pim/utils.h"

#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif

/**
 * Find the funambol alert.
 */
void findAlert(){
    HwndFunctions::wndMsgBox = FindWindow(NULL , getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT));
}

/**
 * Finds the funambol alert windows and closes it.
 */
void HwndFunctions::closePreviousMsgBox(){

    HANDLE handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)findAlert, NULL, 0, NULL);
    DWORD t = WaitForSingleObject(handle, 500);

    if(wndMsgBox){
        SendMessage(wndMsgBox, WM_CLOSE, NULL, NULL);
    }
    wndMsgBox = NULL;

}

/**
 * Find the funambol plugin window. The call is blocking.
 */
static void findFunambolWindowLocking()
{
    //-- for finding if the UI window is present
    HwndFunctions::wnd = FindWindow(NULL, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL));
}

/**
 * Find the funambol plugin window, with a 2 seconds timeout.
 */
void HwndFunctions::findFunambolWindow()
{
    HANDLE hand = CreateThread(NULL, 0,
            (LPTHREAD_START_ROUTINE)findFunambolWindowLocking, NULL, 0, NULL);
    DWORD t = WaitForSingleObject(hand, 2000); // 2 secs.
}

/**
 * Get the Funambol plugin window handle if present.
 */
HWND HwndFunctions::getWindowHandle() {

    if (wnd == NULL)
        findFunambolWindow();

    return wnd;
}

/*
void HwndFunctions::initHwnd() {
    if (wnd == NULL)
        findFunambolWindow();
}
*/

HWND HwndFunctions::wnd = NULL;
HWND HwndFunctions::wndMsgBox = NULL;


int sourceNameToId(const char* sourceName){

    int source = -1;

    if     (!strcmp(sourceName, CONTACTS_NAME_A))       source = SOURCE_CONTACTS;
    else if(!strcmp(sourceName, APPOINTMENTS_NAME_A))   source = SOURCE_CALENDAR;
    else if(!strcmp(sourceName, TASKS_NAME_A))          source = SOURCE_TASKS;
    else if(!strcmp(sourceName, FILES_NAME_A))          source = SOURCE_BRIEFCASE;
    else if(!strcmp(sourceName, NOTES_NAME_A))          source = SOURCE_NOTES;
    else if(!strcmp(sourceName, MAILS_NAME_A))          source = SOURCE_MAIL;
    else if(!strcmp(sourceName, CONFIG_NAME_A))         source = SOURCE_CONFIG;

    return source;
}

// TODO: implement it
const char* sourceIdToName( const char* sourceName ){

    return NULL;
}
