/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include "pim/TaskBuilder.h"
#include "pim/UtilsRecurrence.h"
#include "pim/MAPIUtils.h"
#include "base/util/utils.h"



/**
 * Enumeration of task fields used by WM plugin.
 * @note Don't change the order of these properties!
 *       This list MUST be synchronized with PIMPR_props_task array of CEPROPID fields.
 */
enum {
    ePIMPR_SUBJECT                            ,
    ePIMPR_SENSITIVITY                        ,
    ePIMPR_IS_RECURRING                       ,
    ePIMPR_REMINDER_SET                       ,
//  ePIMPR_REMINDER_SOUND_FILE                ,
    ePIMPR_REMINDER_OPTIONS                   ,
    ePIMPR_IMPORTANCE                         ,
    ePIMPR_TEAM_TASK                          ,
    ePIMPR_START_DATE                         ,
    ePIMPR_DUE_DATE                           ,
    ePIMPR_DATE_COMPLETED                     ,
    ePIMPR_COMPLETE                           ,
    ePIMPR_REMINDER_TIME                      ,
//  ePIMPR_RECURRING_REGENERATING             ,
    ePIMPR_CATEGORIES                         ,
    ePIMPR_BODY_TEXT                          ,
    
    // --- Recurring fields from here ---
    ePIMPR_RECURRING_TYPE                     ,
    ePIMPR_RECURRING_PATTERNSTARTDATE         ,
    ePIMPR_RECURRING_PATTERNENDDATE           ,
//  ePIMPR_RECURRING_STARTTIME                ,
//  ePIMPR_RECURRING_ENDTIME                  ,
    ePIMPR_RECURRING_NOEND                    ,
    ePIMPR_RECURRING_OCCURRENCES              ,
    ePIMPR_RECURRING_INTERVAL                 ,
    ePIMPR_RECURRING_DAYOFWEEKMASK            ,
    ePIMPR_RECURRING_DAYOFMONTH               ,
    ePIMPR_RECURRING_INSTANCE                 ,
//  ePIMPR_RECURRING_DURATION                 ,
    ePIMPR_RECURRING_MONTHOFYEAR              ,

    // This is the total number of appointment properties supported (including recurring ones)
    NUM_COLS_TASK
};


/**
 * Array of task fields used by WM plugin.
 * ***************************************
 *
 * Recurring properties are appended at the end.
 * @note Don't change the order of these properties!
 */
static CEPROPID PIMPR_props_task[] = { 
    PIMPR_SUBJECT                           ,
    PIMPR_SENSITIVITY                       ,
    PIMPR_IS_RECURRING                      ,
    PIMPR_REMINDER_SET                      ,
//  PIMPR_REMINDER_SOUND_FILE               ,
    PIMPR_REMINDER_OPTIONS                  ,
    PIMPR_IMPORTANCE                        ,
    PIMPR_TEAM_TASK                         ,
    PIMPR_START_DATE                        ,
    PIMPR_DUE_DATE                          ,
    PIMPR_DATE_COMPLETED                    ,
    PIMPR_COMPLETE                          ,
    PIMPR_REMINDER_TIME                     ,
//  PIMPR_RECURRING_REGENERATING            ,
    PIMPR_CATEGORIES                        ,
    PIMPR_BODY_TEXT                         ,
    
    // --- Recurring fields from here ---
    PIMPR_RECURRING_TYPE                    ,
    PIMPR_RECURRING_PATTERNSTARTDATE        ,
    PIMPR_RECURRING_PATTERNENDDATE          ,
//  PIMPR_RECURRING_STARTTIME               ,
//  PIMPR_RECURRING_ENDTIME                 ,
    PIMPR_RECURRING_NOEND                   ,
    PIMPR_RECURRING_OCCURRENCES             ,
    PIMPR_RECURRING_INTERVAL                ,
    PIMPR_RECURRING_DAYOFWEEKMASK           ,
    PIMPR_RECURRING_DAYOFMONTH              ,
    PIMPR_RECURRING_INSTANCE                ,
//  PIMPR_RECURRING_DURATION                ,
    PIMPR_RECURRING_MONTHOFYEAR
};


/**
 * Array of task fields names for SIF format.
 * Recurring fields are listed in recurrenceFields[] array.
 * @note Don't change the order of these properties!
 *       This list MUST be synchronized with PIMPR_props_task array of CEPROPID fields.
 */
const wchar_t* taskFields[] = {

    {L"Subject"                            },                // Returns or sets the subject for the Outlook item. This property corresponds to the MAPI property PR_SUBJECT. The Subject property is the default property for Outlook items.
    {L"Sensitivity"                        },                // Returns or sets the sensitivity for the Outlook item. Can be one of the following OlSensitivity constants: olConfidential(3), olNormal(0), olPersonal(1), or olPrivate(2). This property corresponds to the MAPI property PR_SENSITIVITY
    {L"IsRecurring"                        },                // True if the appointment or task is a recurring appointment or task. When the GetRecurrencePattern method is used with an AppointmentItem or TaskItem object, this property is set to True
    {L"ReminderSet"                        },                // True if a reminder has been set for this appointment, mail item or task. 
//  {L"ReminderSoundFile"                  },                // Returns or sets the path and filename of the sound file to play when the reminder occurs for the appointment or task. This property is only valid if the ReminderOverrideDefault and ReminderPlaySound properties are set to True
    {L"ReminderOptions"                    },                // What actions to take when the reminder occurs. (LED=1, Vibrate=2, Dialog=4, Sound=8, Repeat=16)
    {L"Importance"                         },                // Returns or sets the relative importance level for the Outlook item. Can be one of the following OlImportance constants: olImportanceHigh(2), olImportanceLow(0), or olImportanceNormal(1). This property corresponds to the MAPI property PR_IMPORTANCE. 
    {L"TeamTask"                           },                // True if the task is a team task
    {L"StartDate"                          },                // Returns or sets the starting date and time for the task
    {L"DueDate"                            },                // Returns or sets a Date indicating the due date for the task.
    {L"DateCompleted"                      },                // Returns or sets the completion date of the task
    {L"Complete"                           },                // True if the task is completed
    {L"ReminderTime"                       },                // Returns or sets the date and time at which the reminder should occur for this item. 
//  {L"RecurringRegenerating"              },                // A value indicating whether the task recurs after it has been marked as completed.
    {L"Categories"                         },                // Returns or sets the categories assigned to the Outlook item. 
    {L"Body"                               },                // Returns or sets the clear-text body of the Outlook item.
    {NULL}
};



/*
* Method to populate the task XML structure.
* It get every property name and value and build the XML structure to exchange with server.
*
* @param taskStringItem : the string that will contain the xml structure
* @param pTask          : the task object that contain every value
* @param isCRC          : it says if the sif is calculated for the crc or not. If we are looking at the crc
*                         we have to consider the recurrence fields only if the task is recurring
*/
void populateTaskStringItem (std::wstring &taskStringItem, ITask *pTask, BOOL isCRC) {

    BSTR element = NULL;
    long elementLong = 0;
    DATE date = NULL, startdate = NULL;
    std::wstring element_s;

    VARIANT_BOOL variantBool = NULL; // 0==false -1==true
    wchar_t tempDate [DIM_LOCAL_TEMP];

    wchar_t* charBlob = NULL;

    //
    // FOR RECURRENCE
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int x = 0, y = 0, z = 0;

    taskStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    taskStringItem += TEXT("<task>");

    // BSTR
    pTask->get_Subject(&element);
    element_s = std::wstring(element); encodeSpecialChar(element_s);
    taskStringItem += TEXT("<Subject>") + element_s + TEXT("</Subject>");
    SysFreeString(element);

    pTask->get_Categories(&element);
    element_s = std::wstring(element); encodeSpecialChar(element_s);
    taskStringItem += TEXT("<Categories>") + element_s + TEXT("</Categories>");
    SysFreeString(element);

    pTask->get_ReminderSoundFile(&element);
    element_s = (std::wstring)element; encodeSpecialChar(element_s);
    taskStringItem += TEXT("<ReminderSoundFile>") + element_s + TEXT("</ReminderSoundFile>");
    SysFreeString(element);

    pTask->get_Body(&element);
    removeEndCarriage(&element);
    element_s = wstring(element); encodeSpecialChar(element_s);
    taskStringItem += TEXT("<Body>") + element_s + TEXT("</Body>");
    SysFreeString(element);

    // VARIANT_BOOL
    pTask->get_ReminderSet(&variantBool);
    wsprintf(tempDate, TEXT("%i"), variantBool);
    normalizeBoolean(tempDate);
    taskStringItem += TEXT("<ReminderSet>") + wstring(tempDate) + TEXT("</ReminderSet>");

    pTask->get_Complete(&variantBool);
    wsprintf(tempDate, TEXT("%i"), variantBool);
    normalizeBoolean(tempDate);
    taskStringItem  += TEXT("<Complete>") + wstring(tempDate) + TEXT("</Complete>");

    pTask->get_TeamTask(&variantBool);
    wsprintf(tempDate, TEXT("%i"), variantBool);
    normalizeBoolean(tempDate);
    taskStringItem += TEXT("<TeamTask>") + wstring(tempDate) + TEXT("</TeamTask>");


    // DATE
    pTask->get_StartDate(&startdate);

    wsprintf(tempDate, TEXT(""));
    double toCompare = MAX_DATE_DOUBLE;
    double toCompare4000 = MAX_DATE_DOUBLE_4000;

    SYSTEMTIME st;
    if (startdate != toCompare) {
        doubleToSystemTimeBirthday(tempDate, startdate);
        //doubleToSystemTime(tempDate, date);
        VariantTimeToSystemTime(startdate, &st);
        x = st.wHour;
        localTimeToUTC(st);
        y = st.wHour;


    }
    taskStringItem += TEXT("<StartDate>") + wstring(tempDate) + TEXT("</StartDate>");


    pTask->get_DueDate(&date);

    wsprintf(tempDate, TEXT(""));
    if (date != toCompare) {
        doubleToSystemTimeBirthday(tempDate, date);
        //doubleToSystemTime(tempDate, date);
    }
    taskStringItem += TEXT("<DueDate>") + wstring(tempDate) + TEXT("</DueDate>");

    /*
    pTask->get_DateCompleted(&date);

    wsprintf(tempDate, TEXT(""));
    if (date != toCompare) {
            doubleToSystemTime(tempDate, date);
    }
    taskStringItem  = taskStringItem + TEXT("<DateCompleted>") + tempDate + TEXT("</DateCompleted>");
    */

    pTask->get_ReminderTime(&date);

    wsprintf(tempDate, TEXT(""));
    if (date != toCompare) {
        // We use local time "YYYYMMDDThhmmss" (since 6.5.6)
        doubleToSystemTimeHourZero(tempDate, date, FALSE, FALSE);
    }
    taskStringItem += TEXT("<ReminderTime>") + wstring(tempDate) + TEXT("</ReminderTime>");


    // long

    pTask->get_Sensitivity(&elementLong);
    wsprintf(tempDate, TEXT("%i"), elementLong);
    taskStringItem += TEXT("<Sensitivity>") + wstring(tempDate) + TEXT("</Sensitivity>");

    pTask->get_ReminderOptions(&elementLong);
    wsprintf(tempDate, TEXT("%i"), elementLong);
    taskStringItem += TEXT("<ReminderOptions>") + wstring(tempDate) + TEXT("</ReminderOptions>");

    pTask->get_Importance(&elementLong);
    wsprintf(tempDate, TEXT("%i"), elementLong);
    taskStringItem +=  TEXT("<Importance>") + wstring(tempDate) + TEXT("</Importance>");

    /******************************************************/
    //Recurrence
    VARIANT_BOOL isRecurring;
    pTask->get_IsRecurring(&isRecurring);
    wsprintf(tempDate, TEXT("%i"), isRecurring);
    normalizeBoolean(tempDate);
    taskStringItem += TEXT("<IsRecurring>") + wstring(tempDate) + TEXT("</IsRecurring>");

    IRecurrencePattern *pRecurrence;
    pTask->GetRecurrencePattern (&pRecurrence);

    wstring recurrence = L"";
    if (!isCRC || isRecurring == -1) { // variant bool to say TRUE
        recurrence = getRecurrenceTags(pRecurrence, isRecurring, 0, 0, olFolderTasks, startdate, TRUE);
    }
/*
    long            recurrenceType = 0,
        interval       = 0,
        monthOfYear    = 0,
        dayOfMonth     = 0,
        dayOfWeekMask  = 0,
        instance       = 0;

    pRecurrence->get_RecurrenceType(&recurrenceType);
    pRecurrence->get_Interval(&interval);
    pRecurrence->get_MonthOfYear(&monthOfYear);
    pRecurrence->get_DayOfMonth(&dayOfMonth);
    pRecurrence->get_Instance(&instance);
    pRecurrence->get_DayOfWeekMask(&dayOfWeekMask);

    //
    // Modification to get right information about changing day, month...
    //
    TIME_ZONE_INFORMATION tmz;
    int k = GetTimeZoneInformation(&tmz);

    z = (x-y) < 0 ? (x-y)*(-1) : (x-y) ;

    //
    // Change day?
    //
    changeDay = z > 12 ? TRUE : FALSE;
    if (changeDay) {
        if (tmz.Bias < 0)
            isBackward = TRUE;
        else
            isBackward = FALSE;
    }


    if (recurrenceType < 0) {
        recurrenceType = 0;
    }
    wsprintf(tempDate, TEXT("%i"), recurrenceType);
    taskStringItem  = taskStringItem + TEXT("<RecurrenceType>") + tempDate + TEXT("</RecurrenceType>");

    pRecurrence->get_Interval(&elementLong);
    wsprintf(tempDate, TEXT("%i"), elementLong);
    taskStringItem  = taskStringItem + TEXT("<Interval>") + tempDate + TEXT("</Interval>");

    //
    // Day of Month
    //
    if (dayOfMonth != 0) {
        if (changeDay) {
            if (isBackward) {
                dayOfMonth = dayOfMonth - 1;
                if (dayOfMonth == 0) {
                    if (monthOfYear == 5 || monthOfYear == 7 || monthOfYear == 10) {
                        // if May, July, October the month before has 30 days
                        dayOfMonth = 30;
                    } else {
                        dayOfMonth = 31;
                    }
                    changeYearBackward = TRUE;
                }
            } else { // is
                dayOfMonth = dayOfMonth + 1;
                if (dayOfMonth >  31  ||
                    (dayOfMonth == 31 &&
                    (monthOfYear == 4 || monthOfYear == 6 || monthOfYear == 9)
                    )
                    ) {
                        dayOfMonth = 1;
                        changeYearForward = TRUE;
                }
            }
        }
        wsprintf(tempDate, TEXT("%i"), dayOfMonth);
        taskStringItem  = taskStringItem + TEXT("<DayOfMonth>") + tempDate + TEXT("</DayOfMonth>");
    } else {
        taskStringItem  = taskStringItem + TEXT("<DayOfMonth>0</DayOfMonth>");
    }

    //
    // Month of Year
    //
    if (monthOfYear != 0) {
        if (changeYearBackward) {
            monthOfYear = monthOfYear - 1;
            if (monthOfYear == 0) {
                monthOfYear = 12;
            }
        } else if (changeYearForward) {
            monthOfYear = monthOfYear + 1;
            if (monthOfYear > 12) {
                monthOfYear = 1;
            }
        }
        wsprintf(tempDate, TEXT("%i"), monthOfYear);
        taskStringItem  = taskStringItem + TEXT("<MonthOfYear>") + tempDate + TEXT("</MonthOfYear>");
    } else {
        taskStringItem  = taskStringItem + TEXT("<MonthOfYear>0</MonthOfYear>");
    }
    //
    // DayOfWeekMask
    //
    if (dayOfWeekMask != 0) {
        if (changeDay) {
            if (isBackward) {
                dayOfWeekMask = convertDayOfWeekMaskBackward2(dayOfWeekMask);
            } else {
                dayOfWeekMask = convertDayOfWeekMaskForward2(dayOfWeekMask);
            }
        }
        wsprintf(tempDate, TEXT("%i"), dayOfWeekMask);
        taskStringItem  = taskStringItem + TEXT("<DayOfWeekMask>") + tempDate + TEXT("</DayOfWeekMask>");
    } else {
        taskStringItem  = taskStringItem + TEXT("<DayOfWeekMask>0</DayOfWeekMask>");
    }

    //
    // Instance
    //
    if (instance != 0) {
        wsprintf(tempDate, TEXT("%i"), instance);
        taskStringItem  = taskStringItem + TEXT("<Instance>") + tempDate + TEXT("</Instance>");
    } else {
        taskStringItem  = taskStringItem + TEXT("<Instance>0</Instance>");
    }

    //
    // Pattern Start Date
    //
    pRecurrence->get_PatternStartDate(&date);
    wsprintf(tempDate, TEXT(""));
    toCompare = MAX_DATE_DOUBLE;

    if (date != toCompare && date < toCompare4000 ) {
        doubleToSystemTime(tempDate, date);
    }
    taskStringItem  = taskStringItem + TEXT("<PatternStartDate>") + tempDate + TEXT("</PatternStartDate>");

    //
    // No End Date
    //
    pRecurrence->get_NoEndDate(&variantBool);
    wsprintf(tempDate, TEXT("%i"), variantBool);
    normalizeBoolean(tempDate);
    taskStringItem  = taskStringItem + TEXT("<NoEndDate>") + tempDate + TEXT("</NoEndDate>");

    //
    // Pattern End Date
    //
    pRecurrence->get_PatternEndDate(&date);
    wsprintf(tempDate, TEXT(""));
    toCompare = MAX_DATE_DOUBLE;

    if (date != toCompare && date < toCompare4000 ) {
        doubleToSystemTime(tempDate, date);
    }
    taskStringItem  = taskStringItem + TEXT("<PatternEndDate>") + tempDate + TEXT("</PatternEndDate>");

    //
    // Occurrences
    //

    pRecurrence->get_Occurrences(&elementLong);
    if (elementLong < 0) {
        elementLong = 0;
    }
    wsprintf(tempDate, TEXT("%i"), elementLong);
    taskStringItem  = taskStringItem + TEXT("<Occurrences>") + tempDate + TEXT("</Occurrences>");
*/
    taskStringItem += recurrence;
    taskStringItem += TEXT("</task>");

}



/*
* The method calculate the CRC hash value for the task item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pTask : the task object that contain every value
*/

long calculateTaskHash (ITask *pTask) {

    wstring p ;

    populateTaskStringItem(p, pTask, TRUE);
    long l = stringCrc32(p.data());

    return l;
}





WinItem* taskToWinTask(IItem *pTask, bool isSif) {

    HRESULT hr                = E_FAIL;
    CEPROPVAL *prgPropvalPoom = NULL;    
    ULONG cbBuffer            = 0;
    HANDLE hHeap              = GetProcessHeap(); 
    WinTask* winT             = NULL;
    WinRecurrence* winRec     = NULL;
    wchar_t tempValue[10];
    wstring isRecurring, value, start;


    if (isSif) { winT = new WinTaskSIF(); } 
    else       { winT = new WinTask()   ; }

    cbBuffer = 0;
    hr = pTask->GetProps(PIMPR_props_task, CEDB_ALLOWREALLOC, NUM_COLS_TASK, &prgPropvalPoom, &cbBuffer, hHeap);
    if(FAILED(hr) || NULL == prgPropvalPoom || 0 == cbBuffer) {
        goto Exit;
    }


    //
    // Fill all task properties
    //
    for (int i = 0; i < ePIMPR_RECURRING_TYPE; i++) {  

        switch (i) 
        {
        case ePIMPR_START_DATE:
        case ePIMPR_DUE_DATE:
        case ePIMPR_DATE_COMPLETED:
            {
                // Date values: "yyyyMMdd" - tasks are ALWAYS allday
                value = getDatePropertyFromMapi(&prgPropvalPoom[i], false, true);
                break;
            }
        case ePIMPR_REMINDER_TIME:
            {
                // "YYYYMMDDThhmmss" use local time!
                value = getDatePropertyFromMapi(&prgPropvalPoom[i], false, false);
                break;
            }
        case ePIMPR_SUBJECT:
        case ePIMPR_CATEGORIES:
        case ePIMPR_BODY_TEXT:
            {
                // String values.
                value = getStringPropertyFromMapi(&prgPropvalPoom[i]);   
                break;
            }
        default:
            {
                // Int values.
                int tmp = getIntPropertyFromMapi(&prgPropvalPoom[i]);
                wsprintf(tempValue, TEXT("%d"), tmp);
                value = tempValue;
                break;
            }
        }

        winT->setProperty(taskFields[i], value);
        //LOG.info("%2d) %ls = \"%ls\"", i+1, taskFields[i], value.c_str());
    }


    if (!winT->getProperty(TEXT("IsRecurring"), isRecurring)) {
        goto Exit;
    }
    if (isRecurring != TEXT("1")) {
        goto Exit;
    }


    //
    // Manage recurring properties: need to retrieve all values together
    // ---------------------------- to apply safe checks and conversions.
    //
    winRec = winT->getRecPattern();
    if (!winRec) { 
        goto Exit; 
    }
    
    // tasks are ALWAYS allday
    winT->getProperty(TEXT("StartDate"), start);
    setWinRecurrenceProps(*winRec, prgPropvalPoom, ePIMPR_RECURRING_TYPE, start, true, false);


Exit:
    // Free memory.    
    HeapFree(hHeap, 0, prgPropvalPoom);
    return winT;
}




HRESULT winTaskToTask(WinTask& winT, IItem *pTask, const wstring& path, long oidToUpdate) {

    HRESULT hr                      = E_FAIL;
    int incNumProp                  = 0;
    bool exists                     = false; 
    WinRecurrence* winRec           = NULL;
    ITask* pTaskOld                 = NULL;
    IRecurrencePattern* pRecurrence = NULL;
    CEPROPVAL *prgPropvalPoom       = new CEPROPVAL[NUM_COLS_TASK];      // Warning: allocation of CEPROPVAL array is critical...
    IP_OUTLOOK_APP* pOutlookApp     = NULL;
    FILETIME date;
    wstring start;


    if (oidToUpdate) {
        //
        // It's an UPDATED item: Clear the recurrence pattern & save task: 
        // if the task is recurring or not, we have to delete any previous information 
        // regarding recurrence.
        //
        pOutlookApp = getOutlookApp();
        if (!pOutlookApp) { goto finally; }

        // 'pAppointmentOld' is the IAppointment (old interface)
        hr = pOutlookApp->GetItemFromOid(oidToUpdate, (IDispatch**)&pTaskOld);
        if (!pTaskOld || FAILED(hr)) { goto finally; }

        hr = pTaskOld->ClearRecurrencePattern();
        if (FAILED(hr)) {
            LOG.error("Error clearing rec pattern");
        }

        hr = pTaskOld->Save();
        if (FAILED(hr)) {
            LOG.error("Error saving task \"%ls\"", winT.getName().c_str());
            goto finally;
        }
        pTaskOld->Release();
        pTaskOld = NULL;
    }



    // Check if it's recurring.
    bool isRecurring = false;
    int tmp = 0;
    if (winT.getIntProperty(TEXT("IsRecurring"), &tmp)) {
        if (tmp == 1) isRecurring = true;
    }


    //
    // Set all normal properties.
    //
    for (int i=0; i < ePIMPR_RECURRING_TYPE; i++) {
        exists = false; 
        wstring& value = winT.getPropertyRef(taskFields[i], &exists);
        
        if (exists) {
            switch (i) 
            {
            case ePIMPR_START_DATE:
            case ePIMPR_DUE_DATE:
            case ePIMPR_DATE_COMPLETED:
                {
                    // Date values: "yyyyMMdd" - tasks are ALWAYS allday
                    date = getDatePropertyFromString(value, true);
                    setDatePropertyToMapi(&prgPropvalPoom[incNumProp], PIMPR_props_task[i], date);
                    //LOG.debug("%2d. %ls = %ls (date) (%lu %lu)", incNumProp+1, taskFields[i], value.c_str(), date.dwHighDateTime, date.dwLowDateTime);
                    break;
                }
            case ePIMPR_REMINDER_TIME:
                {
                    // "YYYYMMDDThhmmss" expected local time.
                    date = getDatePropertyFromString(value, false);
                    setDatePropertyToMapi(&prgPropvalPoom[incNumProp], PIMPR_props_task[i], date);
                    break;
                }
            case ePIMPR_SUBJECT:
            case ePIMPR_CATEGORIES:
                {
                    // String values.
                    setStringPropertyToMapi(&prgPropvalPoom[incNumProp], PIMPR_props_task[i], value);
                    //LOG.debug("%2d. %ls = %ls (string)", incNumProp+1, taskFields[i], value.c_str());
                    break;
                }
            case ePIMPR_BODY_TEXT:
                {
                    convertNewlines(value);     // BUG #1562
                    setStringPropertyToMapi(&prgPropvalPoom[incNumProp], PIMPR_props_task[i], value);
                    //LOG.debug("%2d. %ls = %ls (body)", incNumProp+1, taskFields[i], value.c_str());
                    break;
                }
            default:
                {
                    // Int values: should add safe checks?                  ***** TODO ******
                    int val = getIntPropertyFromWinItem(value);
                    setIntPropertyToMapi(&prgPropvalPoom[incNumProp], PIMPR_props_task[i], val);
                    //LOG.debug("%2d. %ls = %d (int)", incNumProp+1, taskFields[i], val);
                    break;
                }
            }
            incNumProp++;
        }       
    }


    if (isRecurring) {
        //
        // Manage recurring properties: need to retrieve all values together
        // ---------------------------- to apply safe checks and conversions.
        //
        winRec = winT.getRecPattern();
        if (!winRec) { goto finally; }

        winT.getProperty(TEXT("StartDate"), start);
        incNumProp = setMAPIRecurrenceProps(prgPropvalPoom, incNumProp, *winRec, start, 
                                            start, true, NULL, false);  // tasks are ALWAYS allday
    }


    //
    // Save all props together (with rec props if recurring).
    //
    hr = saveItemProps(pTask, incNumProp, prgPropvalPoom);
    if (FAILED(hr)) {
        LOG.error("Error saving task \"%ls\"", winT.getName().c_str());
        goto finally;
    }

    hr = S_OK;


finally:

    // Release objects.
    if (pRecurrence) { pRecurrence->Release(); }
    if (pTaskOld)    { pTaskOld->Release();    }

    if (prgPropvalPoom) {
        delete [] prgPropvalPoom;
    }
    return hr;
}

ArrayList* getTaskProperties(){

    int i = 0;
    ArrayList* props = new ArrayList();

    while (taskFields[i]){
        Property prop;
        char* field = toMultibyte(taskFields[i]);
        prop.setPropName(field);
        props->add(prop);
        
        delete [] field;

        i++;
    }

    return props;
}

ArrayList* getVTodoProperties(){

    ArrayList* props = new ArrayList();
    Property* p = new Property();
    ArrayList valenums;
    StringBuffer val;

    p->setPropName("BEGIN");
    val = "VTODO";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);

    valenums.clear();

    p->setPropName("END");
    val = "VTODO";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);

    valenums.clear();
    
    p->setPropName("VERSION");
    val = "1.0";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);

    p->setPropName("X-FUNAMBOL-ALLDAY");
    props->add(*p);
    p->setPropName("DESCRIPTION");
    props->add(*p);
    p->setPropName("CATEGORIES");
    props->add(*p);
    p->setPropName("STATUS");
    props->add(*p);
    p->setPropName("COMPLETED");
    props->add(*p);
    p->setPropName("DUE");
    props->add(*p);
    p->setPropName("PRIORITY");
    props->add(*p);
    p->setPropName("X-FUNAMBOL-AALARMOPTIONS");
    props->add(*p);
    p->setPropName("AALARM");
    props->add(*p);
    p->setPropName("CLASS");
    props->add(*p);
    p->setPropName("DTSTART");
    props->add(*p);
    p->setPropName("SUMMARY");
    props->add(*p);
    p->setPropName("X-FUNAMBOL-TEAMTASK");
    props->add(*p);
    p->setPropName("RRULE");
    props->add(*p);

    delete p; p = NULL;

    return props;
}