/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <stdio.h>
#include <windows.h>
#include <Winbase.h>
#include "base/Log.h"
#include "base/util/utils.h"
#include "base/base64.h"
#include "base/util/XMLProcessor.h"
#include "base/stringUtils.h"

#include "pim/Utils.h"
#include "pim/UtilsRecurrence.h"
#include "pim/RecurrenceException.h"
#include "pim/AppointmentBuilder.h"
#include "pim/MAPIUtils.h"



/**
 * Array of recurring fields names for SIF format.
 * It's used by both appointment and task items.
 * @note Don't change the order of these properties!
 */
const wchar_t* recurrenceFields[] = {

    {L"RecurrenceType"                     },                // Returns or set a RecurrenceType. values are orRecursDaily...
    {L"PatternStartDate"                   },                // Returns or sets the start date of the recurrence.
    {L"PatternEndDate"                     },                // Returns or sets the end date of the recurrence.
    {L"StartTime"                          },
    {L"EndTime"                            },
    {L"NoEndDate"                          },                // True if there is no end date. True is 1.
    {L"Occurrences"                        },                // Return or sets the number of the occurrences of the recurrence.
    {L"Interval"                           },                // Is the interval of the recurrence. If RecurrenceType is olRecursDaily, event occurs every <Interval> day. If olRecursWeekly, event occurs every <inteval> week..
    {L"DayOfWeekMask"                      },                // The combination days of the week constants (i. e. event recurring on Monday and Wednesday. The DayOfWeekMask should be olMonday + olWednesday)
    {L"DayOfMonth"                         },                // Returns or sets the single day of the month from 1 to 31.
    {L"Instance"                           },                // Returns or sets the ordinal number of the day, week, month.
    //{L"RecurringDuration"                  },
    {L"MonthOfYear"                        },                // Returns or sets the month of year.
    
    {NULL}
};



// ************* remove... ******************
#define STRING_NO_RECURRENCE    TEXT("<RecurrenceType/><Interval/><MonthOfYear/><DayOfMonth/><DayOfWeekMask/><Instance/><PatternStartDate/><NoEndDate/><PatternEndDate/><Occurrences/><Exceptions/>")
#define EXCLUDE_DATE            TEXT("ExcludeDate")
#define INCLUDE_DATE            TEXT("IncludeDate")

int convertDayOfWeekMaskBackward(int dayOfWeekMaskSent) {
    int dayOfWeekMask = 0;
    if ((dayOfWeekMaskSent % 2) != 0) {
        dayOfWeekMask = dayOfWeekMaskSent >> 1;
        dayOfWeekMask += 64;
    }
    else {
        dayOfWeekMask = dayOfWeekMaskSent >> 1;
    }
    return dayOfWeekMask;
}
int convertDayOfWeekMaskForward(int dayOfWeekMaskSent) {
    int dayOfWeekMask = 0;
    if (dayOfWeekMaskSent >= 64) {
        dayOfWeekMask = dayOfWeekMaskSent << 1;
        dayOfWeekMask -= 127;
    }
    else {
        dayOfWeekMask = dayOfWeekMaskSent << 1;
    }
    return dayOfWeekMask;
}

wstring getRecurrenceTags(IRecurrencePattern* pRecurrence, VARIANT_BOOL isRecurring,
                          int x, int y, OlDefaultFolders olFolder, DATE startdate, BOOL isAllDay) {

    wstring appointmentStringItem = TEXT("");
    wchar_t tempDate [DIM_LOCAL_TEMP];
    VARIANT_BOOL variantBool = NULL; // 0==false -1==true

    if (isRecurring == FALSE) {
        return STRING_NO_RECURRENCE;
    }

    int z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;

    double toCompare = MAX_DATE_DOUBLE;
    double toCompare4000 = MAX_DATE_DOUBLE_4000;

    BSTR element        = NULL      ;
    long elementLong    = 0         ;

    DATE date           = NULL      ;
    wstring element_s;

    long            recurrenceType = 0,
                    interval       = 0,
                    monthOfYear    = 0,
                    dayOfMonth     = 0,
                    dayOfWeekMask  = 0,
                    instance       = 0;

    pRecurrence->get_RecurrenceType(&recurrenceType);
    pRecurrence->get_Interval(&interval);
    pRecurrence->get_MonthOfYear(&monthOfYear);
    pRecurrence->get_DayOfMonth(&dayOfMonth);
    pRecurrence->get_Instance(&instance);
    pRecurrence->get_DayOfWeekMask(&dayOfWeekMask);

    //
    // Modification to get right information about changing day, month...
    //
    TIME_ZONE_INFORMATION tmz;
    int k = GetTimeZoneInformation(&tmz);

    z = (x-y) < 0 ? (x-y)*(-1) : (x-y) ;

    //
    // Change day?
    //
    changeDay = z > 12 ? TRUE : FALSE;

    if (isAllDay)
        changeDay = FALSE;

    if (changeDay) {
        if (tmz.Bias < 0)
            isBackward = TRUE;
        else
            isBackward = FALSE;
    }

    //
    // RecurrenceType
    //
    if (recurrenceType < 0) {
        recurrenceType = 0;
    }
    wsprintf(tempDate, TEXT("%i"), recurrenceType);
    appointmentStringItem  = appointmentStringItem + TEXT("<RecurrenceType>") + tempDate + TEXT("</RecurrenceType>");

    //
    // Interval
    //
    wsprintf(tempDate, TEXT("%i"), interval);
    appointmentStringItem  = appointmentStringItem + TEXT("<Interval>") + tempDate + TEXT("</Interval>");

    //
    // Day of Month
    //
    if (dayOfMonth != 0) {

        if (changeDay) {
            if (isBackward) {
                dayOfMonth = dayOfMonth - 1;
                if (dayOfMonth == 0) {
                    if (monthOfYear == 5 || monthOfYear == 7 || monthOfYear == 10) {
                        // if May, July, October the month before has 30 days
                        dayOfMonth = 30;
                    } else {
                        dayOfMonth = 31;
                    }
                    changeYearBackward = TRUE;
                }
            } else { // is
                dayOfMonth = dayOfMonth + 1;
                if (dayOfMonth >  31  ||
                      (dayOfMonth == 31 &&
                        (monthOfYear == 4 || monthOfYear == 6 || monthOfYear == 9)
                       )
                    ) {
                    dayOfMonth = 1;
                    changeYearForward = TRUE;
                }
            }

        } //endif

        wsprintf(tempDate, TEXT("%i"), dayOfMonth);
        appointmentStringItem  = appointmentStringItem + TEXT("<DayOfMonth>") + tempDate + TEXT("</DayOfMonth>");
    } else {
        appointmentStringItem  = appointmentStringItem + TEXT("<DayOfMonth>0</DayOfMonth>");
    }

    //
    // Month of Year
    //
    if (monthOfYear != 0) {
        if (changeYearBackward) {
            monthOfYear = monthOfYear - 1;
            if (monthOfYear == 0) {
                monthOfYear = 12;
            }
        } else if (changeYearForward) {
            monthOfYear = monthOfYear + 1;
            if (monthOfYear > 12) {
                monthOfYear = 1;
            }
        }
        wsprintf(tempDate, TEXT("%i"), monthOfYear);
        appointmentStringItem  = appointmentStringItem + TEXT("<MonthOfYear>") + tempDate + TEXT("</MonthOfYear>");
    } else {
        appointmentStringItem  = appointmentStringItem + TEXT("<MonthOfYear>0</MonthOfYear>");
    }
    //
    // DayOfWeekMask
    //
    if (dayOfWeekMask != 0) {
        if (changeDay) {
            // do not change week mask
            if (isBackward) {
                dayOfWeekMask = convertDayOfWeekMaskBackward(dayOfWeekMask);
            } else {
                dayOfWeekMask = convertDayOfWeekMaskForward(dayOfWeekMask);
            }
        }
        wsprintf(tempDate, TEXT("%i"), dayOfWeekMask);
        appointmentStringItem  = appointmentStringItem + TEXT("<DayOfWeekMask>") + tempDate + TEXT("</DayOfWeekMask>");
    } else {
        appointmentStringItem  = appointmentStringItem + TEXT("<DayOfWeekMask>0</DayOfWeekMask>");
    }

    //
    // Instance
    //
    if (instance != 0) {
        wsprintf(tempDate, TEXT("%i"), instance);
        appointmentStringItem  = appointmentStringItem + TEXT("<Instance>") + tempDate + TEXT("</Instance>");
    } else {
        appointmentStringItem  = appointmentStringItem + TEXT("<Instance>0</Instance>");
    }

    //
    // Pattern Start Date
    //
    pRecurrence->get_PatternStartDate(&date);
    wsprintf(tempDate, TEXT(""));
    toCompare = MAX_DATE_DOUBLE;

    //
    // We use the start date to avoid every problem
    //
    if (date != toCompare && date < toCompare4000 ) {
        if (isAllDay) {
            doubleToSystemTimeBirthday(tempDate, startdate);
        } else {
            doubleToSystemTime(tempDate, startdate);
        }
        //doubleToSystemTime(tempDate, date);

    }
    appointmentStringItem  = appointmentStringItem + TEXT("<PatternStartDate>") + tempDate + TEXT("</PatternStartDate>");

    //
    // No End Date
    //
    pRecurrence->get_NoEndDate(&variantBool);
    wsprintf(tempDate, TEXT("%i"), variantBool);
    normalizeBoolean(tempDate);
    appointmentStringItem  = appointmentStringItem + TEXT("<NoEndDate>") + tempDate + TEXT("</NoEndDate>");

    //
    // Pattern End Date
    //
    pRecurrence->get_PatternEndDate(&date);
    wsprintf(tempDate, TEXT(""));
    toCompare = MAX_DATE_DOUBLE;
    if (date != toCompare && date < toCompare4000 ) {
        if (startdate) {
            // Fix for PatternEndDate (since v.6.5.6):
            // We need the exact date+time when the last occurrence starts, so we get the
            // time (hours) from startdate and add it to the patternEndDate midnight.
            double hours = startdate - (int)startdate;
            date = (int)date + hours;
        }
        doubleToSystemTime(tempDate, date);
    }
    appointmentStringItem  = appointmentStringItem + TEXT("<PatternEndDate>") + tempDate + TEXT("</PatternEndDate>");

    //
    // Occurrences
    //
    pRecurrence->get_Occurrences(&elementLong);
    if (elementLong < 0) {
        elementLong = 0;
    }
    wsprintf(tempDate, TEXT("%i"), elementLong);
    appointmentStringItem  = appointmentStringItem + TEXT("<Occurrences>") + tempDate + TEXT("</Occurrences>");

    //
    // ---------------- EXCEPTIONS for calendar only ---------
    //

    if (olFolder == olFolderCalendar) {
        if (isRecurring == -1) {
            wstring exceptionItems = createExceptions(pRecurrence, startdate, isAllDay);
            appointmentStringItem += exceptionItems;
        } else {
            appointmentStringItem += TEXT("<Exceptions/>");
        }
    } else if (olFolder == olFolderTasks) {
        //
        // Exceptions in POOM tasks are not permitted!!
        //
        appointmentStringItem += TEXT("<Exceptions/>");
    }

    //
    // ---------------- END EXCEPTIONS ------------------------
    //

    return appointmentStringItem;

}

/*
* Modificated exception creation from client to server.
* The normalization is done in the begin sync process. At this point the items that are
* here should not have any exceptions appointments excepts deletions
*/
wstring createExceptions(IRecurrencePattern* pRecurrence, DATE startdate, BOOL isAllDay) {

    IExceptions* pExceptions;
    int numExceptions;

    pRecurrence->get_Exceptions(&pExceptions);
    pExceptions->get_Count(&numExceptions);

    wstring exceptions = TEXT("<Exceptions>");

    for (int i = 0; i < numExceptions; i++) {
        IException* pException;
        pExceptions->Item(i + 1, &pException);
        wstring exception = createException(pException, startdate, isAllDay);
        exceptions += TEXT("<ExcludeDate>");
        exceptions += exception;
        exceptions += TEXT("</ExcludeDate>");
        pException->Release();
    }

    pExceptions->Release();

    // the exceptions if exists are only in exclude date and so deleted and never created
    exceptions +=  TEXT("<IncludeDate/>");

    exceptions +=  TEXT("</Exceptions>");

    return exceptions;
}

wstring createException(IException* pException, DATE start_date, BOOL isAllDay) {

    wstring exceptionItem = TEXT("");
    DATE originalDate = 0;
    VARIANT_BOOL varBool; // -1 = TRUE, 0 = FALSE
    wchar_t tempDate [DIM_LOCAL_TEMP];

    pException->get_OriginalDate(&originalDate);

    DATE offset = (start_date) - ((long)start_date) ;
    originalDate += offset;
    if (isAllDay) {
        doubleToSystemTimeBirthday(tempDate, originalDate);    // YYYY-MM-DD
    }
    else {
        doubleToSystemTime(tempDate, originalDate);
    }

    // if the recurrence is deleted get the delation and go exit
    pException->get_Deleted(&varBool);
    if (varBool == -1) {
        exceptionItem = wstring(tempDate);
    } else {
        LOG.info("Warning: exception in date %s is not deleted.", tempDate);
    }
    return exceptionItem;
}

/**
 * Parse the recurrence tags in the item sent from server to client.
 *
 * @param pRecurrence       the POOM object to fill
 * @param ptrData           the SIF item to parse
 * @param recurrenceStart   the start date of the event or task
 * @param x                 used to check the day shift
 * @param y                 used to check the day shift
 */
void parseRecurrenceTags(IRecurrencePattern *pRecurrence,
                         const wchar_t* ptrData,
                         DATE recurrenceStart,
                         int x, int y) {

    wchar_t* dummy                 = NULL;
    wchar_t* dummyEvent            = NULL;
    wstring  dummyEvent_s;
    VARIANT_BOOL isNotAllDayEvent  = TRUE;
    long            recurrenceType = 0,
                    interval       = 0,
                    monthOfYear    = 0,
                    dayOfMonth     = 0,
                    dayOfWeekMask  = 0,
                    instance       = 0;

    //
    // FOR RECURRENCE
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;
    //
    // Modification to get right information about changing day, month...
    //
    TIME_ZONE_INFORMATION tmz;
    GetTimeZoneInformation(&tmz);

    //----- server -> client  => do not change date
    z = (x-y) < 0 ? (x-y)*(-1) : (x-y) ;

    //
    // Change day? Backward and Forward are inverted because it is from utc to localtime
    //
    changeDay = z > 12 ? TRUE : FALSE;
    if (changeDay) {
        if (tmz.Bias < 0)
            isBackward = FALSE;
        else
            isBackward = TRUE;
    }

    dummyEvent = getElementContent(ptrData, TEXT("RecurrenceType"), NULL);
    if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
        pRecurrence->put_RecurrenceType(wcstol(dummyEvent, &dummy, 0));
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Interval"), NULL);
    if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
        pRecurrence->put_Interval(wcstol(dummyEvent, &dummy, 0));
        delete [] dummyEvent;dummyEvent=NULL;
    }

    //
    // Day of Month
    //
    dummyEvent = getElementContent(ptrData, TEXT ("DayOfMonth"), NULL);
    if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
        dayOfMonth = wcstol(dummyEvent, &dummy, 0);
        if (changeDay) {
            if (isBackward) {
                dayOfMonth = dayOfMonth - 1;
                if (dayOfMonth == 0) {
                    if (monthOfYear == 5 || monthOfYear == 7 || monthOfYear == 10) {
                        // if May, July, October the month before has 30 days
                        dayOfMonth = 30;
                    } else {
                        dayOfMonth = 31;
                    }
                    changeYearBackward = TRUE;
                }
            } else { // is
                dayOfMonth = dayOfMonth + 1;
                if (dayOfMonth >  31  ||
                      (dayOfMonth == 31 &&
                        (monthOfYear == 4 || monthOfYear == 6 || monthOfYear == 9)
                       )
                    ) {
                    dayOfMonth = 1;
                    changeYearForward = TRUE;
                }
            }
        }; //end if(changeDay)

        pRecurrence->put_DayOfMonth(dayOfMonth);
        delete [] dummyEvent;dummyEvent=NULL;
    }

    //
    // Month of Year
    //
    dummyEvent = getElementContent(ptrData, TEXT ("MonthOfYear"), NULL);
    if (dummyEvent != NULL && wcstol(dummyEvent, &dummy , 0) > 0) {
        monthOfYear = wcstol(dummyEvent, &dummy , 0);
        if (changeYearBackward) {
            monthOfYear = monthOfYear - 1;
            if (monthOfYear == 0) {
                monthOfYear = 12;
            }
        } else if (changeYearForward) {
            monthOfYear = monthOfYear + 1;
            if (monthOfYear > 12) {
                monthOfYear = 1;
            }
        }
        pRecurrence->put_MonthOfYear(monthOfYear);
        delete [] dummyEvent;dummyEvent=NULL;
    }

    //
    // DayOfWeekMask
    //
    dummyEvent = getElementContent(ptrData, TEXT ("DayOfWeekMask"), NULL);
    if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
        dayOfWeekMask = wcstol(dummyEvent, &dummy, 0);
        if (changeDay) {
            if (isBackward) {
                dayOfWeekMask = convertDayOfWeekMaskBackward(dayOfWeekMask);
            } else {
                dayOfWeekMask = convertDayOfWeekMaskForward(dayOfWeekMask);
            }
        }

        pRecurrence->put_DayOfWeekMask(dayOfWeekMask);
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Instance"), NULL);
    if (dummyEvent) {
        if (wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_Instance(wcstol(dummyEvent, &dummy, 0));
        }
        delete [] dummyEvent; dummyEvent=NULL;
    }

    // Pattern Start Date
    // -------------------
    // We use recurrenceStart instead of PatternStartDate
    // to handle timezone changes.
    // PatternStartDate is used only as a backup if the others are not available
    if (!recurrenceStart) {
        dummyEvent = getElementContent(ptrData, TEXT("PatternStartDate"), NULL);
        if (dummyEvent) {
            // If is not empty, get it
            if (*dummyEvent) {
                recurrenceStart = getDATEFromString(dummyEvent);
            }
            delete [] dummyEvent; dummyEvent=NULL;
        }
    }
    // Make sure the date is without the time part
    recurrenceStart = (int)recurrenceStart;

    pRecurrence->put_PatternStartDate(recurrenceStart);

    if (dummyEvent) { delete [] dummyEvent; dummyEvent = NULL; }

    //VARIANT_BOOL -- NoEndDate
    dummyEvent = getElementContent(ptrData, TEXT ("NoEndDate"), NULL);

    if (dummyEvent != NULL &&
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 ||
         wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pRecurrence->put_NoEndDate(VARIANT_TRUE);

    } else {
        pRecurrence->put_NoEndDate(VARIANT_FALSE);

        //Pattern End Date
        dummyEvent = getElementContent(ptrData, TEXT ("PatternEndDate"), NULL);
        // If not null and not empty, parse it
        if (dummyEvent && *dummyEvent) {
            DATE en = getDATEFromString(dummyEvent);
            pRecurrence->put_PatternEndDate(en);
            delete [] dummyEvent; dummyEvent = NULL;
        }

    }
    // Occurrences
    //
    dummyEvent = getElementContent(ptrData, TEXT ("Occurrences"), NULL);
    if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
        pRecurrence->put_Occurrences(wcstol(dummyEvent, &dummy , 0));
        delete [] dummyEvent; dummyEvent = NULL;
    }

}


/*
* Return the oid of the new appointment just inserted
*/
long createAppointmentFromException(IAppointment* pAppointment, RecurrenceException rec, IP_OUTLOOK_APP* polApp) {

    IAppointment* pAppointmentCopy, *pAppointmentNew;
    pAppointment->Copy(&pAppointmentCopy);
    pAppointmentCopy->Save();
    long oid = 0;
    pAppointmentCopy->get_Oid(&oid);
    pAppointmentCopy->Release();

    polApp->GetItemFromOid (oid, (IDispatch**)&pAppointmentNew);

    if (pAppointmentNew) {
        DATE originaldate = rec.getOriginalDate();

        DATE start = 0, end = 0;
        pAppointment->get_Start(&start);
        pAppointment->get_End(&end);

        DATE offset = end - start;
        end = (originaldate + offset);

        pAppointmentNew->ClearRecurrencePattern();
        pAppointmentNew->put_Start(originaldate);
        pAppointmentNew->put_End(end);
        pAppointmentNew->Save();
        pAppointmentNew->Release();
    }

    return oid;

}

/*
* The file contains the oid of the appointment whose crc must be set to 0 during the
* creation of the cache file. It means to send to the server the right apppointmnet
* at the next sync
*/
void writeOidToReset(vector<long> oids, const wchar_t* wpath) {

    wstring filename(L"appOidToReset.dat");
    wstring path(wpath);
    path += L"\\";
    path += filename;

    FILE* f;
    f = _wfopen(path.c_str(), L"w+");

    if (!f)
        return;

    int oidLength = oids.size();

    for (int i = 0; i < oidLength; i++) {
        long currentLongOid = oids[i];
        fwprintf(f, TEXT("<oid>%i</oid>\n"), currentLongOid);
    }
    fflush(f);
    fclose(f);
}



void setWinRecurrenceProps(WinRecurrence& winRec, const CEPROPVAL* prgPropvalPoom, int firstProperty,
                           const wstring& start, const bool isAllDay, bool isAppointment) {

    wstring pStart, pEnd, startTime, endTime;
    
    // If using timezone
    bool toUTC = true;
    if (isAllDay || winRec.hasTimezone()) {
        toUTC = false;
    }

    //
    // Get all properties from MAPI.
    //
    int i = firstProperty;
    int recType       = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    pStart            = getDatePropertyFromMapi(&prgPropvalPoom[i++], toUTC, isAllDay);
    if (isAppointment) { // startTime and endTime are not valid in recurrent tasks
        pEnd              = getDatePropertyFromMapi(&prgPropvalPoom[i++], toUTC, false);
        startTime         = getDatePropertyFromMapi(&prgPropvalPoom[i++], toUTC, false);
        endTime           = getDatePropertyFromMapi(&prgPropvalPoom[i++], toUTC, false);
    } else {
        pEnd              = getDatePropertyFromMapi(&prgPropvalPoom[i++], toUTC, true);   
    }
    int noEndDate     = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    int occurrences   = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    int interval      = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    int dayOfWeekMask = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    int dayOfMonth    = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    int instance      = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    //int duration    = getIntPropertyFromMapi(&prgPropvalPoom[i++]);
    int monthOfYear   = getIntPropertyFromMapi(&prgPropvalPoom[i++]);


    //
    // Recurrence Conversions.
    //
    // Yearly interval = 1 (for compatibility)
    if (recType > 4) {
        interval = 1;
    }
    // For compatibility: weekly with interval = 0 is not correct...
    if ((recType == 1) && (interval == 0)) {
        interval = 1;
    }
    
    // PatternStartDate
    if (!isAllDay) {
        if (toUTC) {
            // OLD STYLE: get patternStart from start (use exactly the same).           **** TODO: remove? ****
            if (start != TEXT("")) { 
                pStart = start; 
            }
        } else {
            // "PatternStartDate" is composed by PatternStartDate + StartTime
            if (pStart.size() >= 8 && startTime.size() >= 15) {
                pStart  = pStart.substr(0, 8);      // the start date
                pStart += TEXT("T");
                pStart += startTime.substr(9, 6);   // the start time
            }
        }
    }


    // For compatibility
    if (noEndDate) {
        occurrences = 0;
        pEnd = TEXT("");
    }
    else {
        DATE pEndDate = 0;
        systemTimeToDouble(pEnd.c_str(), &pEndDate, NULL);
        if (pEndDate && pEndDate < LIMIT_MAX_DATE) {
            if (toUTC) {
                // OLD STYLE: Correct with the exact hours from "Start" (bug #3688)      **** TODO: remove? ****
                DATE startDate = NULL;
                systemTimeToDouble(start.c_str(), &startDate, NULL);
                if (startDate) {
                    double hours = startDate - (int)startDate;
                    pEndDate = (int)pEndDate + hours;
                    wchar_t tmp[30];
                    doubleToSystemTime(tmp, pEndDate);
                    pEnd = tmp;
                }
            }
            else {
                // "PatternEndDate" is composed by PatternEndDate + StartTime 
                // (note: StartTime, not EndTime!)
                if (pEnd.size() >= 8 && startTime.size() >= 15) {
                    pEnd = pEnd.substr(0, 8);        // the end date
                    pEnd += TEXT("T");
                    pEnd += startTime.substr(9, 6);  // the start time
                }
            }
        }
        else {
            // To avoid PatternEndDate = 4501-01-01
            pEnd = TEXT("");
        }
    }

    // CHANGE-DAY to UTC: modify monthOfYear / dayOfMonth / dayOfWeekMask.
    // All-day events don't need conversions.
    if (toUTC) {
        changeDay(L"UTC", start, &monthOfYear, &dayOfMonth, &dayOfWeekMask);
    }


    //
    // Fill WinRec with all properties.
    //
    int j=0;
    winRec.setIntProperty(recurrenceFields[j++], recType      );
    winRec.setProperty(recurrenceFields[j++],    pStart       );
    winRec.setProperty(recurrenceFields[j++],    pEnd         );
    j++;       // "startTime" is not used, but keep the sequence
    j++;       // "endTime"   is not used, but keep the sequence    
    winRec.setIntProperty(recurrenceFields[j++], noEndDate    );
    winRec.setIntProperty(recurrenceFields[j++], occurrences  );
    winRec.setIntProperty(recurrenceFields[j++], interval     );
    winRec.setIntProperty(recurrenceFields[j++], dayOfWeekMask);
    winRec.setIntProperty(recurrenceFields[j++], dayOfMonth   );
    winRec.setIntProperty(recurrenceFields[j++], instance     );
    // winRec.setIntProperty(recurrenceFields[j++],   duration   );
    winRec.setIntProperty(recurrenceFields[j++], monthOfYear  );
}






int setMAPIRecurrenceProps(CEPROPVAL* prgPropvalPoom, const int propsNumber,
                           WinRecurrence& winRec, const wstring& start, const wstring& end,
                           const bool isAllDay, const TIME_ZONE_INFORMATION* tzInfo,
                           bool isAppointment) {

    // The index of properties, is updated for every prop inserted and thus returned.
    int incNumProp = propsNumber;

    // "Start" is mandatory
    if (start == TEXT("")) {
        LOG.error("Cannot save recurrence, \"Start\" property is missing.");
        return incNumProp;
    }

    // If using timezone, data is already in local time.
    bool toLocal = true;
    if (isAllDay || winRec.hasTimezone()) {
        toLocal = false;
    }

    int recType       = -1;
    int noEndDate     = -1;
    int occurrences   = -1;
    int interval      = -1;
    int dayOfWeekMask = -1;
    int dayOfMonth    = -1;
    int instance      = -1;
    //int duration    = -1;
    int monthOfYear   = -1;
    wstring pStart, pEnd, startTime, endTime;
    int durationInMinutes = 0; // the duration of the recurrence
    //
    // Get all props from WinRecurrence.
    //
    int i=0;
    winRec.getIntProperty(recurrenceFields[i++],       &recType);
    winRec.getProperty   (recurrenceFields[i++],         pStart);
    winRec.getProperty   (recurrenceFields[i++],           pEnd);
    i++;       // "startTime" is not used, but keep the sequence
    i++;       // "endTime"   is not used, but keep the sequence
    winRec.getIntProperty(recurrenceFields[i++],     &noEndDate);
    winRec.getIntProperty(recurrenceFields[i++],   &occurrences);
    winRec.getIntProperty(recurrenceFields[i++],      &interval);
    winRec.getIntProperty(recurrenceFields[i++], &dayOfWeekMask);
    winRec.getIntProperty(recurrenceFields[i++],    &dayOfMonth);
    winRec.getIntProperty(recurrenceFields[i++],      &instance);
    //winRec.getIntProperty(recurrenceFields[i++],      &duration);
    winRec.getIntProperty(recurrenceFields[i++],   &monthOfYear);

    // The boolean need to take care about the appointment synced from a 7.0.x client
    // against a server 6.5.x that doesn't support the timezone.
    // the data were store with local time
    if (pStart.find(TEXT("Z")) == wstring::npos) {
        toLocal = false;
    }   
    //
    // Recurrence Conversions.
    //
    if (recType<0 || recType>6) {
        // RecurrenceType is mandatory.
        LOG.error("Error setting recurrenceType: %d not supported", recType);
        return incNumProp;
    }
    if (interval < 1) {
        interval = 1;
    }

    // ---- PatternStartDate ----
    if (toLocal) {
        // OLD STYLE (v6.5): get value from Start if not empty. 
        // Used by allday, tasks, and also to keep compatibility on incoming items.
        pStart = start;
    }
    else {
        // Extract pStart (only date) & startTime (only time)
        if (pStart == TEXT("")) {
            // pStart can be missing, for example in vCal 1.0
            pStart = getPatternStartDateFromStart(start, tzInfo);
        }
        replaceAll(L"-", L"", pStart);          // Safe: to accept SIF format "yyyy-mm-dd" too.
        if (pStart.size() >= 15) {
            startTime  = TEXT("18991230T");     // stands for the date = 0
            startTime += pStart.substr(9, 6);
        }
        if (pStart.size() >= 8) {
            pStart     = pStart.substr(0, 8);
            pStart    += TEXT("T000000");
        }
    }

    // ---- PatternEndDate ----
    if (!toLocal) {
        // Extract pEnd (only date)
        if (pEnd.size() >= 8) {
            replaceAll(L"-", L"", pEnd);        // Safe: to accept SIF format "yyyy-mm-dd" too.
            pEnd  = pEnd.substr(0, 8);
            pEnd += TEXT("T000000");
        }
    }
        
    // ---- EndTime ----
    // It's now mandatory, if we set the startTime. We have to calculate it.
    if (startTime.size() > 0) {
        // "Duration" is constant, can be retrieved from (Start - End)
        DATE startDate, endDate, startTimeDate;
        stringTimeToDouble(start,     &startDate);                      // not converting to local time
        stringTimeToDouble(end,       &endDate);
        stringTimeToDouble(startTime, &startTimeDate);

        DATE endTimeDate = startTimeDate + (endDate - startDate);
                                
        // the end time date must be calculated only for the time. The date must be
        // the default one 18991230, so the first part is deleted
        endTimeDate = endTimeDate - (int)endTimeDate; 
                
        WCHAR tmp[20];
        doubleToSystemTimeHourZero(tmp, endTimeDate, FALSE, FALSE);     // not converting to UTC
        endTime = tmp;  

        // calculate the duration of the recurrence: this could be 1,0416666667 that is 25 hours
        // so get the 1, that means 1 day that are 1440 minutes
        DATE val = endDate - startDate;        
        durationInMinutes = (int)val * 1440;  
        // then the 0,41666667 * 1440 that means how many minutes used in a day
        durationInMinutes += round((val - (int)val) * 1440);

    }


    // CHANGE-DAY to Local: modify monthOfYear / dayOfMonth / dayOfWeekMask.
    // All-day event don't need conversions.
    if (toLocal) {
        changeDay(TEXT("Local"), start, &monthOfYear, &dayOfMonth, &dayOfWeekMask);
    }


    //
    // Set all props to MAPI.
    // If using timezone, values should be in local time - however all formats are accepted.
    // NOTE: the order is important!!!
    //
    setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_TYPE, recType);

    FILETIME date = getDatePropertyFromString(pStart, isAllDay);
    setDatePropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_PATTERNSTARTDATE, date);

    if ((noEndDate != 1) && (occurrences <= 0)) {    
        // Set PatternEndDate only if Occurrences not found or wrong.
        if (pEnd.size() > 0) {
            date = getDatePropertyFromString(pEnd);      // Expected UTC value...
            setDatePropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_PATTERNENDDATE, date);
        }
    }

    if (startTime.size() > 0) {
        date = getDatePropertyFromString(startTime, isAllDay);
        setDatePropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_STARTTIME, date);
    }
    /* the end time is not used because the durationInMinutes substitutes this value
    if (endTime.size() > 0) {
        date = getDatePropertyFromString(endTime, isAllDay);
        setDatePropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_ENDTIME, date);
    }
    */
    if (durationInMinutes > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_DURATION, durationInMinutes);        
    }
    if (noEndDate > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_NOEND, noEndDate);
    }
    if ((noEndDate != 1) && (occurrences > 0)) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_OCCURRENCES, occurrences);
    }
    if (interval > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_INTERVAL, interval);
    }
    if (dayOfWeekMask > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_DAYOFWEEKMASK, dayOfWeekMask);
    }
    if (dayOfMonth > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_DAYOFMONTH, dayOfMonth);
    }
    if (instance > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_INSTANCE, instance);
    }    
    if (monthOfYear > 0) {
        setIntPropertyToMapi(&prgPropvalPoom[incNumProp++], PIMPR_RECURRING_MONTHOFYEAR, monthOfYear);
    }

    // Return the updated number of properties.
    return incNumProp;
}



wstring getPatternStartDateFromStart(const wstring& start, const TIME_ZONE_INFORMATION* tzInfo) {

    if (start.find(L"Z", 0) == wstring::npos) {
        // "Start" is in local time, so PatternStart is the same.
        return start;
    }
    if (start.find(L"-", 0) != wstring::npos) {
        // Start is in old SIF style: "yyyy-mm-dd".
        wstring ret = start;
        replaceAll(L"-", L"", ret);
        ret += L"T000000";
        return ret;
    }
    if (start.size() == 8) {
        // Start is in format "yyyymmdd" (allday).
        return start;
    }

    // Backup the current tzInfo of the System
    bool tzBackuped = false;
    TIME_ZONE_INFORMATION backupTzInfo;
    if (tzInfo) {
        GetTimeZoneInformation(&backupTzInfo);
        // Optimization: do it ONLY if tz is different.
        if (!isSameTimezone(tzInfo, &backupTzInfo)) {
            SetTimeZoneInformation(tzInfo);
            tzBackuped = true;
        }
    }

    SYSTEMTIME t;
    DATE startDate = 0;
    WCHAR tmp[20];
    systemTimeToDouble(start.c_str(), &startDate, NULL);      // converts to local time

    VariantTimeToSystemTime(startDate, &t);
    wsprintf(tmp, TEXT("%d%02d%02dT%02d%02d%02d"), t.wYear, t.wMonth, t.wDay,
                                                   t.wHour, t.wMinute, t.wSecond);
    // Restore correct tzInfo.
    if (tzInfo && tzBackuped) {
        SetTimeZoneInformation(&backupTzInfo);
    }

    wstring patternStart = tmp;
    return patternStart;
}





bool changeDay(const wstring& dest, const wstring& start, 
               int* monthOfYear, int* dayOfMonth, int* dayOfWeekMask) 
{

    DATE date = 0;
    SYSTEMTIME time, localTime, utcTime;

    // This call gets 'date' ALWAYS in local time...        *** verify if true on WM! ****  TODO!
    systemTimeToDouble     (start.c_str(), &date, NULL);
    VariantTimeToSystemTime(date,  &time);

    localTime = time;
    localTimeToUTC2(time);
    utcTime   = time;


    // Day to be changed!
    if (localTime.wDay != utcTime.wDay) {
        if (dest == L"Local") {
            time = localTime;
        }

        // MonthOfYear: get from Start
        if (*monthOfYear != 0 && *monthOfYear != -1) {
            *monthOfYear = time.wMonth;
        }

        // DayOfMonth: get from Start
        if (*dayOfMonth != 0 && *dayOfMonth != -1) {
            *dayOfMonth = time.wDay;
        }

        // DayOfWeekMask: shift the mask
        if (*dayOfWeekMask != 0 && *dayOfWeekMask != -1) {
            
            // Calculate if forward or backward 1 day.
            TIME_ZONE_INFORMATION tmz;
            GetTimeZoneInformation(&tmz);
            bool forward;
            if (dest == L"Local") {
                if (tmz.Bias < 0) forward = true;
                else              forward = false;
            }
            else {
                if (tmz.Bias < 0) forward = false;
                else              forward = true;
            }

            // Forward = left shift.
            if (forward) {
                *dayOfWeekMask = *dayOfWeekMask << 1;
                if (*dayOfWeekMask > 127) {
                    *dayOfWeekMask -= 127;
                }
            }
            // Backward = right shift.
            else {
                if (*dayOfWeekMask & 1) {
                    *dayOfWeekMask += 127;
                }
                *dayOfWeekMask = *dayOfWeekMask >> 1;
            }
        }
        return true;
    }
    return false;
}

ArrayList* getRecurrenceProperties(){

    int i = 0;
    ArrayList* props = new ArrayList();

    while (recurrenceFields[i]){
        if (wcscmp (recurrenceFields[i] , L"StartTime") == 0 ||
            wcscmp (recurrenceFields[i] , L"EndTime") == 0 ) {
            i++;    
        }else{
            Property prop;
            char* field = toMultibyte(recurrenceFields[i]);
            prop.setPropName(field);
            props->add(prop);
            delete [] field;
            i++;
        }
    }

    return props;
}


// ************************** TODO: remove? **************************
/*
* delete the occurrences of the recurrence if there are any between them
*/
//void deleteIntraOccurrences(IRecurrencePattern* pRecurrence, DATE originalDate, DATE startDate) {
//
//    IAppointment* pExAppointment;
//
//    if (originalDate > startDate) {
//        for (originalDate-=1; originalDate > startDate; originalDate-=1) {
//
//            pRecurrence->GetOccurrence(originalDate, &pExAppointment);
//            if (pExAppointment) {
//                pExAppointment->Delete();
//                pExAppointment->Release();
//            }
//        }
//    } else {
//        for (originalDate+=1; originalDate < startDate; originalDate+=1) {
//
//            pRecurrence->GetOccurrence(originalDate, &pExAppointment);
//            if (pExAppointment) {
//                pExAppointment->Delete();
//                pExAppointment->Release();
//            }
//        }
//    }
//}

//vector<RecurrenceException> getExceptionList(IRecurrencePattern* pRecurrence,
//                                             const wchar_t* ptrData,
//                                             const wchar_t* action) {
//
//    wchar_t* dummy                  = NULL;
//    wchar_t* dummyEvent             = NULL;
//    wstring  dummyEvent_s;
//    VARIANT_BOOL isNotAllDayEvent   = TRUE;
//    unsigned int pos = 0;
//
//    vector<RecurrenceException> listExceptions;
//
//    unsigned int previous = pos;
//    while ((dummyEvent = getElementContent(&ptrData[pos], (wchar_t*)action, &pos)) != NULL) {
//
//        if (wcscmp(dummyEvent, TEXT("")) == 0) {
//            pos += previous;
//            previous = pos;
//            continue;
//
//        }
//
//        BOOL isAllDayFormat = FALSE;
//        DATE originalDate;
//
//        RecurrenceException rec;
//
//        isAllDayFormat = isDateForAllDayEventFormat(dummyEvent);
//        if (isAllDayFormat) {
//            systemTimeToDoubleBirthFormat(dummyEvent, &originalDate, TEXT("START")); // put 00:00 for the date
//        } else {
//            systemTimeToDouble(dummyEvent, &originalDate, NULL);
//        }
//
//        rec.setOriginalDate(originalDate);
//
//        if (wcscmp(action, EXCLUDE_DATE) == 0) {
//            rec.setIsDeleted(TRUE);
//        } else {
//            rec.setIsDeleted(FALSE);
//            rec.setAllDayEvent(isAllDayFormat);
//        }
//
//        pos += previous;
//        previous = pos;
//
//        listExceptions.push_back(rec);
//    }
//
//    return listExceptions;
//}



/*
* Try to check if exists already an occurrence in that date...
* If exists it delete the existing ont to insert the other appointment exception
*/
//BOOL checkAlreadyAnException(IRecurrencePattern* pRecurrence, DATE originalDate, DATE startDate) {
//
//    IAppointment* pApp;
//    pRecurrence->GetOccurrence(startDate, &pApp);
//
//    if (pApp) {
//
//        if ((int)originalDate - (int)startDate) {
//            LOG.info("An exception of the calendar already exists. Delete it to insert the new one");
//            pApp->Delete();
//            pApp->Release();
//            return TRUE;
//        }
//    }
//
//    return FALSE;
//
//}

