/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "pim/mail/MailServer2Client.h"
#include "base/util/utils.h"
#include "base/util/BasicTime.h"
#include "pim/difftime.h"
#include "pim/FILEServer2Client.h"
#include "base/Log.h"
#include "string.h"
#include <algorithm>
#include <vector>
#include "customization.h"
#include "pim/account.h"
#include "localizationUtils.h"
#include "OutOfMemoryException.h"

#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif
using namespace std;

HRESULT SetReplyTo(LPMAPISESSION lpSession, LPMESSAGE lpMessage, LPSTR pszReplyTo);
char* getMessageID(IMessage* pmsg, MailClientData* m);
//------------------------------------------------------------- Typedef

enum {
    eeePR_ENTRYID,
    eeePR_MESSAGE_FLAGS,
    eeePR_LAST_MODIFICATION_TIME,
    eeeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeeNUM_COLS, endSync) =
{
    eeeNUM_COLS,
    PR_ENTRYID,                       // entry id
    PR_MESSAGE_FLAGS,
    PR_LAST_MODIFICATION_TIME        //

};

 enum {
    ePR_ENTRYID,
    ePR_MESSAGE_FLAGS,
    //MM
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,
    ePR_SENDER_NAME,
    ePR_SENDER_EMAIL_ADDRESS,
    ePR_HASATTACH,
    //MM
    NUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) =
{
    NUM_COLS,
    PR_ENTRYID,                         // entry id
    PR_MESSAGE_FLAGS,
    //MM
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>
    PR_HASATTACH

};

enum {
    eePR_ENTRYID,
    eePR_LAST_MODIFICATION_TIME,
    eePR_DISPLAY_NAME,
    eePR_MESSAGE_FLAGS,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) =
{
    eeNUM_COLS,
    PR_ENTRYID,
    PR_LAST_MODIFICATION_TIME,
    PR_DISPLAY_NAME,
    PR_MESSAGE_FLAGS
};

//----------------------------------------------------- Local functions
static ULONG CountRecipsInString(LPCWSTR psz)
{
  ULONG cRecips = 0;

  if (psz != NULL)
  {
    if (*psz != '\0')
      ++cRecips;

    while (*psz != L'\0')
    {
      if (*psz == L';' || *psz == L',')
        ++cRecips;
      ++psz;
    }
  }

  return cRecips;
}


//------------------------------------------------------ Constructors

MailServer2Client::MailServer2Client()
{
    server_folderToSync = NULL;
}

MailServer2Client::~MailServer2Client()
{
    if (server_folderToSync) {
        delete [] server_folderToSync; server_folderToSync = NULL;
    }
}

HRESULT MailServer2Client::deletePreviousAttachment(IMessage* pmsg) {

    LOG.debug("Enter into deletePreviousAttachment function");
    LPMAPITABLE pAttachTable = NULL;
    // for each row in the attach table
    LPSRowSet pRows = NULL;

    HRESULT hr = pmsg->GetAttachmentTable(0, &pAttachTable);
    EXIT_ON_FAILED(hr);

    // iterate all the attachment
    while(TRUE) {

        hr = pAttachTable->QueryRows(1, 0, &pRows);
        if (FAILED(hr) || pRows->cRows != 1)
            break;

        // only one row is returned anyway
        int nProps = pRows->aRow[0].cValues;

        long num = -1;
        BOOL found = FALSE;

        for (int i=0; i<nProps; i++) {
            ULONG ulPT = pRows->aRow[0].lpProps[i].ulPropTag;

            // TCHAR* szPropName = ReverseLookup(ulPT);
            switch (ulPT) {
                case PR_ATTACH_NUM:
                    num = pRows->aRow[0].lpProps[i].Value.ul;
                    found = TRUE;
                    break;
                case PR_ATTACH_SIZE:
                    break;
                case PR_ATTACH_FILENAME:
                    break;
                case PR_MSG_STATUS:
                    break;
                case 0x30000003:
                    // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
                    break;
                default:
                    break;
            }    // switch
            if (found)
                break;

        }
        FreeProws(pRows);

        pmsg->DeleteAttach(num, NULL, 0, NULL);

    }
FuncExit:
    // Free each element in the pRows buffer
    FreeProws(pRows);
    RELEASE_OBJ(pAttachTable);
    LOG.debug("Exiting from deletePreviousAttachment function");
    return hr;

}

//----------------------------------------------------------- Methods

HRESULT MailServer2Client::AddAttachment(LPMESSAGE pmsg, MailClientData* m)
{
    LOG.debug("Enter into AddAttachment function");
    HRESULT hr = S_OK;
    BodyPart* bodyPart = NULL;
    bodyPart = m->getEmailData()->getEmailItem().getFirstAttachment();

    if (bodyPart == NULL) {
        LOG.debug("Exiting from AddAttachment function: no body part found");
        return hr;
    }

    /*
    * if the sync is inclusive, delete all the previous attachment because they comes
    * new all from the current inclusive filter
    */
    if (m->getIsSyncInclusive()) {
        deletePreviousAttachment(pmsg);
    }
    do {
        const char* tmpFileName = bodyPart->getContent();
        if(!tmpFileName){
            LOG.error("NULL temp file name from bodypart.");
            hr = -1;
            goto FuncExit;
        }

        wchar_t* filename = toWideChar(bodyPart->getFilename());
        if (!filename) {
            filename = createFileName(TEXT("noname"));
        }

        wchar_t* name = 0;
        if (bodyPart->getName()) {
            name = toWideChar(bodyPart->getName());
        }
        else {
            name = wstrdup(filename);
        }

        /*
        * Added extension to the name if there isn't one
        */
        string mimeType(bodyPart->getMimeType());
        std::transform(mimeType.begin(), mimeType.end(), mimeType.begin(), tolower);

        if (mimeType.find("message/rfc822") != string::npos && wcsstr(name, TEXT(".eml")) == 0) {
            wstring s(name);
            s.append(TEXT(".eml"));
            delete [] name;
            name = wstrdup(s.c_str());
        }

        SPropValue  prop[3];
        IAttach*    pAttachment = NULL;
        ULONG       ulAttachmentNum = 0;
        FILE*       fileAttach = NULL;;

        memset(prop, 0, sizeof(prop));
        fileAttach = fopen(tmpFileName, "rb");

        if (fileAttach == NULL) {
            LOG.error("AddAttachment: error opening input file");
            hr = -1;
            delete [] name;
            delete [] filename;

            continue;
        }

        hr = pmsg->CreateAttach(NULL, 0, &ulAttachmentNum, &pAttachment);
        EXIT_ON_FAILED(hr);

        // Set the initial properties on the attachment
        // no data actually there yet
        prop[0].ulPropTag = PR_ATTACH_METHOD;
        prop[0].Value.ul = ATTACH_BY_VALUE;

        prop[1].ulPropTag = PR_ATTACH_DATA_BIN;

        prop[2].ulPropTag = PR_ATTACH_FILENAME;
        prop[2].Value.LPSZ = name;

        hr = pAttachment->SetProps(3, prop, NULL);
        EXIT_ON_FAILED(hr);

        // Now get the data property and fill it in
        //
        IStream* pStream;

        hr = pAttachment->OpenProperty (PR_ATTACH_DATA_BIN,
                                     NULL,
                                     0,
                                     MAPI_MODIFY | MAPI_CREATE,
                                     (LPUNKNOWN *)&pStream);
        EXIT_ON_FAILED(hr);

        DWORD cbFile = fgetsize(fileAttach);
        if (cbFile == 0xFFFFFFFF) {// error
            LOG.error("AddAttachment: error loading temp file");
            delete [] name;
            delete [] filename;

            continue;
        }

        ULONG        cbWritten;
        ULONG        cbReadSize;
        const ULONG  ChunkSize = 4096;
        BYTE         Buffer[ChunkSize];

        do {
            cbReadSize = fread(Buffer, sizeof(BYTE), ChunkSize, fileAttach );

            if (cbReadSize > 0)    {
                hr = pStream->Write(Buffer, cbReadSize, &cbWritten);
            }
        } while (!FAILED(hr) && (cbReadSize == ChunkSize));

        fflush(fileAttach);
        fclose(fileAttach);
        pStream->Commit(STGC_DEFAULT);
        pStream->Release();

        // Set the size property
        //
        prop[0].ulPropTag = PR_ATTACH_SIZE;
        prop[0].Value.ul = cbFile + sizeof(prop[2]);
        hr = pAttachment->SetProps(1, prop, NULL);

        if (tmpFileName != NULL) {
            wchar_t* t = toWideChar(tmpFileName);
            DeleteFile(t);
            delete [] t;
        }

        delete [] name;
        delete [] filename;

    } while ((bodyPart = m->getEmailData()->getEmailItem().getNextAttachment()) != NULL);

FuncExit:
    LOG.debug("Exiting from AddAttachment function");
    return hr;
}


static TCHAR szSMTP[] = L"SMTP";

void AddRecipients(LPWSTR pszSrc, ULONG ulType, ADRLIST* plst, LPWSTR& pszLocalRecips, LPSPropValue& pval)
{
    LOG.debug("Enter into AddRecipients function");
    if (pszSrc == NULL) {
        LOG.debug("Exiting from AddRecipients function: no recipients");
        return;
    }
    LPWSTR psz = pszSrc;
    while (*psz != L'\0') {
        while (*psz == L' ' && *psz != '\0')
            ++psz;

        if (*psz == L'\0')
            break;

        LPWSTR pszEnd = psz;
        // while (*pszEnd != L' ' && (*pszEnd != ';' || *pszEnd != ',') && *pszEnd != '\0')
        while (*pszEnd != ',' && *pszEnd != '\0')
            ++pszEnd;

        int cch = pszEnd - psz;

        wcsncpy(pszLocalRecips, psz, cch);
        *(pszLocalRecips + cch) = L'\0';

        plst->aEntries[plst->cEntries].cValues = 3;
        plst->aEntries[plst->cEntries].rgPropVals = pval;

        // Set the type (To, Cc, Bcc)...
        pval->ulPropTag = PR_RECIPIENT_TYPE;
        pval->Value.ul = ulType;
        ++pval;

        // Set the address type (we only do SMTP)...
        pval->ulPropTag = PR_ADDRTYPE;
        pval->Value.lpszW = szSMTP;
        ++pval;

        // Set the address...
        pval->ulPropTag = PR_EMAIL_ADDRESS;
        pval->Value.lpszW = pszLocalRecips;
        ++pval;

        /* Modify the plst->aEntries[plst->cEntries].cValues = 4;
        // Set the name...
        pval->ulPropTag = PR_DISPLAY_NAME;
        pval->Value.lpszW = TEXT("Fabio M");
        ++pval;
        */


        ++plst->cEntries;

        pszLocalRecips += wcslen(pszLocalRecips) + 1;

        if (*pszEnd != L'\0')
            ++pszEnd;

        psz = pszEnd;
    }
    LOG.debug("Exiting from AddRecipients function");
}

HRESULT MailServer2Client::SetMessageRecipients (LPMESSAGE pmsg, MailClientData* m) {

    LOG.debug("Enter into SetMessageRecipients function");
    ULONG cRecipients = 0;
    ULONG cchTotalRecips = 0;

    ADRLIST* plst = NULL;
    LPSPropValue pval = NULL;
    BYTE* pb = NULL;
    LPWSTR pszLocalRecips;
    HRESULT hr;
    wchar_t *to = NULL, *cc = NULL, *bcc = NULL;
    if (m->getEmailData()->getEmailItem().getTo()) {
        to = toWideChar(m->getEmailData()->getEmailItem().getTo());
        cRecipients += CountRecipsInString(to);
        cchTotalRecips += wcslen(to) + 3;
    }

    if (m->getEmailData()->getEmailItem().getCc()) {
        cc = toWideChar(m->getEmailData()->getEmailItem().getCc());
        cRecipients += CountRecipsInString(cc);
        cchTotalRecips += wcslen(cc) + 3;
    }

    if (m->getEmailData()->getEmailItem().getBcc()) {
        bcc = toWideChar(m->getEmailData()->getEmailItem().getBcc());
        cRecipients += CountRecipsInString(bcc);
        cchTotalRecips += wcslen(bcc) + 3;
    }

    //
    // Allocate one big block of memory to hold all the strings.
    // The block is arranged as follows:
    //
    // ADRLIST        SPropValue's     Copy of addresses
    // |---------------------|--------------------|-------------------|
    //

    DWORD cb = sizeof(ADRLIST) + ((sizeof(ADRENTRY) + sizeof(SPropValue) * 3) * cRecipients);

    pb = new BYTE[cb + (cchTotalRecips * sizeof(WCHAR))];
    if (pb == NULL)
    return E_OUTOFMEMORY;

    ZeroMemory(pb, cb + (cchTotalRecips * sizeof(WCHAR)));

    plst = (ADRLIST*) pb;

    pszLocalRecips = (LPWSTR) (pb + cb);

    pb += sizeof(ADRLIST) + (sizeof(ADRENTRY) * cRecipients);

    pval = (SPropValue*) pb;

    AddRecipients(to, MAPI_TO, plst, pszLocalRecips, pval);
    AddRecipients(cc, MAPI_CC, plst, pszLocalRecips, pval);
    AddRecipients(bcc, MAPI_BCC, plst, pszLocalRecips, pval);

    hr = pmsg->ModifyRecipients(MODRECIP_ADD, plst);

    delete[] (BYTE*) plst;
    if (to) {delete [] to; to = NULL;}
    if (cc) {delete [] cc; cc = NULL;}
    if (bcc) {delete [] bcc; bcc = NULL;}

    LOG.debug("Exiting from SetMessageRecipients function");
    return hr;
}

void MailServer2Client::bt2ft(const BasicTime& b, FILETIME& f)
{
    SYSTEMTIME local, s1, s2;

    local.wYear         = b.getYear();
    local.wMonth        = b.getMonth();
    local.wDay          = b.getDay();
    local.wDayOfWeek    = DayOfWeek(local.wYear, local.wMonth, local.wDay);    // b.getWeekday(); // 0 sunday, 1monday
    local.wHour         = b.getHour();
    local.wMinute       = b.getMin();
    local.wSecond       = b.getSec();
    local.wMilliseconds = 0;

    s1 = DT_AddDiff(nano100SecInHour, -(b.getTzHour()), &local);
    if (b.getTzMin() != 0 && b.getTzHour() != 0)
        s2 = DT_AddDiff(nano100SecInMin, -(abs(b.getTzHour())/b.getTzHour())*(b.getTzMin()), &s1);
    else s2 = DT_AddDiff(nano100SecInMin,b.getTzMin(),&s1);
    SystemTimeToFileTime(&s2, &f);

}

/*
* OnlyFlags says the client has to set only the PR_MESSAGE_FLAGS of the message the PR_MSG_STATUS.
* This is for the update mail. In our case the draft modification are sent from server
* as a delete of the old one and a new insert of the new one.
*/
HRESULT MailServer2Client::SetMessageProps(IMAPISession* pSession, LPMESSAGE pmsg, MailClientData* m, BOOL onlyFlags) {

    LOG.debug("Enter into SetMessageProps function");
    SPropValue rgprops[12] = {0};
    ULONG    cProps   = 0;
    HRESULT   hr;
    BodyPart bp;
    wchar_t *subj = NULL,   *from = NULL,
            *messId = NULL, *headers = NULL;

    // it is used only when the email is partial. To store the remaining email
    // to be downloaded
    long emailSizeToDownload = 0; //
    LPSTREAM  pstm    = NULL;
    ULONG messageFlag = (ULONG) 0x00000000;
    ULONG messageStatus = (ULONG) 0x00000000;


    //
    // Set this flag only for Inbox and Draft. This shows the From in the
    // visualization of the mail into the pocket pc

    if (wcscmp(m->getParent(), TEXT("I")) == 0 ||
        wcscmp(m->getParent(), TEXT("T")) == 0) {

        messageFlag = messageFlag | MSGFLAG_UNMODIFIED;

    } else if (wcscmp(m->getParent(), TEXT("D")) == 0) {

        messageFlag = messageFlag | MSGFLAG_FROMME | MSGFLAG_UNSENT;

    } else if (wcscmp(m->getParent(), TEXT("S")) == 0) {

        messageFlag = messageFlag | MSGFLAG_FROMME ;

    } 
    // the only flag needed to insert an email. User for send log only
    else if (wcscmp(m->getParent(), TEXT("O")) == 0) {
        
        messageFlag = 40 ;
    }

    if (m->getEmailData()->getRead())
        messageFlag = messageFlag | MSGFLAG_READ;


    // Set the flags and a subject if they exist.
    rgprops[cProps].ulPropTag = PR_MESSAGE_FLAGS;
    rgprops[cProps].Value.ul = messageFlag;
    ++cProps;

    // rgprops[cProps].Value.ul = MSGSTATUS_RECTYPE_SMTP; //MSGSTATUS_HEADERONLY; //  MSGSTATUS_PARTIAL | MSGSTATUS_RECTYPE_SMTP |  //  // | MSGSTATUS_PARTIAL | MSGSTATUS_HAS_PR_CE_MIME_TEXT; //
    // rgprops[cProps].Value.ul = MSGSTATUS_HEADERONLY | MSGSTATUS_PENDING_ATTACHMENTS; //  MSGSTATUS_PARTIAL | MSGSTATUS_RECTYPE_SMTP |  //  // | MSGSTATUS_PARTIAL | MSGSTATUS_HAS_PR_CE_MIME_TEXT; //  chn

    /*
    * how understand if the file is to download or it has some other to be downloaded
    */
    BOOL foundToBeDownloaded = FALSE;
    /*
    * internal isInclusive drives the choices of the action to perform.
    * it can be true even if the getIsInclusive is false because some choice can
    * converge
    */
    bool isInclusive = m->getIsSyncInclusive();
    rgprops[cProps].ulPropTag = PR_MSG_STATUS;
    if (m->getEmailData()->getIsMailPartial()) {
        /*                
        if (isInclusive == false) {
            if (m->getCurrentMessageSizeFilter() >= m->getMaxMailMessageSize()) {
                if (m->getEmailData()->getRemainingAttachNumber() <= 0) {
                   isInclusive = true;
                } else {
                    MailMessage &msg2 = m->getEmailData()->getEmailItem();
                    if (!msg2.empty()) {
                        BodyPart& bpart = msg2.getBody();
                        if (bpart.getContent() != NULL) {
                            unsigned long len = strlen(bpart.getContent());
                            if (len >= 400000 && m->getEmailData()->getRemainingBodySize() > 0) {
                                isInclusive = true;
                            }
                        }
                    }
                }
            }
        }
        
        if (isInclusive == false) {
            if (m->getCurrentMessageSizeFilter() >= m->getMaxMailMessageSize() && 
                    (m->getEmailData()->getRemainingAttachNumber() <= 0 ||
                     m->getEmailData()->getRemainingBodySize() > 0))
                        isInclusive = true;
                
            }
        }
        */

        if (isInclusive == false) {
            if (m->getCurrentMessageSizeFilter() >= m->getMaxMailMessageSize() && 
                (m->getIncludeAttachment() == true                  ||
                 m->getEmailData()->getRemainingBodySize() > 0      ||
                 m->getEmailData()->getRemainingAttachNumber() <= 0 )) {    
                isInclusive = true;
            }
        }
        if (isInclusive == false) {
            if(m->getEmailData()->getRemainingBodySize() > 0) {
                foundToBeDownloaded = TRUE;
            } else {
                for (unsigned int i = 0; i < m->getEmailData()->getRemainingAttachNumber(); i++) {
                    if (m->getEmailData()->getAttachmentSize(i) > 0 &&
                        (m->getEmailData()->getAttachmentSize(i) < m->getMaxMailMessageSize())) {
                        foundToBeDownloaded = TRUE;
                    }
                }
            }
        }

        if (foundToBeDownloaded) {
            messageStatus = messageStatus | MSGSTATUS_HEADERONLY | MSGSTATUS_PARTIAL;
        }

        // The following code must be used when the partial attachments are handled properly
        // if (m->getEmailData()->getRemainingAttachNumber() > 0) { // there is any attach to be downloaded
        //     messageStatus = messageStatus | MSGSTATUS_PENDING_ATTACHMENTS;
        // }
        // if (m->getEmailData()->getRemainingBodySize() > 0) { // there is any body to be downloaded
        //    messageStatus = messageStatus | MSGSTATUS_HEADERONLY;
        //    if (m->getEmailData()->getRemainingAttachNumber() == 0) {
        //        messageStatus = messageStatus | MSGSTATUS_PARTIAL;
        //    }
        // }
    }
    if (messageStatus == (ULONG) 0x00000000) {
        messageStatus = MSGSTATUS_RECTYPE_SMTP;
    }
    rgprops[cProps].Value.ul = messageStatus;
    ++cProps;

    //if (!onlyFlags) { // no more used when the inclusive filter was inserted. If the msg MailMessage is
    // emty no other actions are done

    MailMessage &msg = m->getEmailData()->getEmailItem();

    /*
    * empty checks the date of the message instead of the message id
    */
    if (!msg.empty()) {

        if (msg.attachmentCount() > 0) {
            rgprops[cProps].ulPropTag = PR_HASATTACH;
            rgprops[cProps].Value.ul = msg.attachmentCount();
            ++cProps;
        }

        // Set the recipients up.
        hr = SetMessageRecipients(pmsg, m);
        EXIT_ON_FAILED(hr);

        //hr = SetReplyTo(pSession, pmsg, "Sync4j@yahoogroups.com");
        //EXIT_ON_FAILED(hr);


        if (msg.getSubject() != NULL) {
            rgprops[cProps].ulPropTag = PR_SUBJECT;
            subj = toWideChar(msg.getSubject());
            rgprops[cProps].Value.lpszW = subj;
            ++cProps;
        }
        if (msg.getFrom() != NULL) {
            rgprops[cProps].ulPropTag = PR_SENDER_EMAIL_ADDRESS;
            from = toWideChar(msg.getFrom());
            rgprops[cProps].Value.lpszW = from;
            ++cProps;
        }
        
        // insert importance // the priority
        if (msg.getImportance() != NULL) {
            rgprops[cProps].ulPropTag = PR_IMPORTANCE;
            long priority = IMPORTANCE_NORMAL;
            if (strcmp(msg.getImportance(), "1") == 0) {
                priority = IMPORTANCE_HIGH;
            } else if (strcmp(msg.getImportance(), "5") == 0) {
                priority = IMPORTANCE_LOW;
            }
            rgprops[cProps].Value.l = priority;
            ++cProps;
        }
        
        if (wcscmp(m->getParent(), TEXT("O")) != 0) {
            
            FILETIME ft;
        
            // Received time (Received: header) to view externally. currently it isn't of use
            bt2ft(msg.getReceived(), ft);
            rgprops[cProps].ulPropTag = PR_MESSAGE_DELIVERY_TIME;
            rgprops[cProps].Value.ft = ft;
            ++cProps;
        
            // Sent time (Date: header). to view internally. this is right
            bt2ft(msg.getDate(), ft);
            rgprops[cProps].ulPropTag = PR_CLIENT_SUBMIT_TIME; // PR_DELIVER_TIME; // // PR_LAST_MODIFICATION_TIME;
            rgprops[cProps].Value.ft = ft;
            ++cProps;
        }
        
        rgprops[cProps].ulPropTag = PR_CUSTOM_MESSAGE_ID;
        messId = toWideChar(msg.getMessageId());
        if (messId) {
            rgprops[cProps].Value.lpszW  = messId;
            ++cProps;
        }

        rgprops[cProps].ulPropTag = PR_CUSTOM_MESSAGE_HEADER;

        headers = toWideChar(msg.getHeaders());
        if (headers) {
            rgprops[cProps].Value.lpszW  = headers;
            ++cProps;
        }

        // Body
        bp = msg.getBody();

        // add to the body the name of the attachments and their sizes
        if (m->getEmailData()->getIsMailPartial()) {
            int attachNum = 0, attachSize = 0;
            unsigned long maxMsg = 0, bodySize = 0, totalBodySize = 0, filter;
            string appendToBody;
            string maxMsgString;
            string filterString;
            string totalBodySizeString;
            char tmp[64];

            // are in bytes
            bodySize = m->getEmailData()->getRemainingBodySize() > 0 ?
                       m->getEmailData()->getRemainingBodySize() : 0;
            filter = m->getCurrentMessageSizeFilter();
            totalBodySize = bodySize + filter;
            attachNum = m->getEmailData()->getRemainingAttachNumber() > 0 ?
                        m->getEmailData()->getRemainingAttachNumber() : 0;
            maxMsg = m->getMaxMailMessageSize();

            //
            // round to the nearest ten value. This because
            // the 500 kb supported are not really 500 but a multiple 1024
            // so a round is needed to not write 496 kb
            //
            unsigned long toShow = (maxMsg/1024 % 5);
            toShow = (5 - toShow) + maxMsg/1024;
            sprintf(tmp, "%luKb", toShow);
            maxMsgString += tmp;

            if ((toShow = filter/1024) > 450) {
                toShow = (filter/1024 % 5);
                toShow = (5 - toShow) + filter/1024;
            }
            sprintf(tmp, "%luKb", toShow);
            filterString += tmp;

            sprintf(tmp, "%luKb", totalBodySize/1024);
            totalBodySizeString += tmp;

            // inside here filter < maxMsg
            if (isInclusive == false) {

                if (bodySize > 0) {

                    if (totalBodySize > maxMsg) {
                        if (attachNum > 0) {
                            //case Pre-action 10
                            appendToBody += "Click here to download the rest of the message";
                            appendToBody += " up to the maximum allowed size of ";
                            appendToBody += maxMsgString;

                        } else {
                            // case Pre-action 3
                            appendToBody += "Only first ";
                            appendToBody += filterString;
                            appendToBody += " (of " + totalBodySizeString + ").";
                            appendToBody += " Click below to get the rest";
                            appendToBody += " up to the maximum allowed size of ";
                            appendToBody += maxMsgString;

                        }
                    } else {
                        if (attachNum > 0) {
                            bool existsMinorOfMaxMsg = false;
                            bool existsMajorOfMaxMsg = false;
                            for (int i = 0; i < attachNum; i++) {
                                if (m->getEmailData()->getAttachmentSize(i) > 0 &&
                                    (m->getEmailData()->getAttachmentSize(i) < maxMsg)) {
                                    existsMinorOfMaxMsg = true;
                                } else {
                                    existsMajorOfMaxMsg = true;
                                }
                            }

                            // all attachments are under max message
                            if (!existsMajorOfMaxMsg && existsMinorOfMaxMsg) {
                                // case Pre-action 7
                                appendToBody += "Only first ";
                                appendToBody += filterString;
                                appendToBody += " downloaded. Click below to get the rest";
                                appendToBody += " up to the maximum allowed size of ";
                                appendToBody += maxMsgString;
                            }
                            // at least an attachments is under max message
                            else if (existsMinorOfMaxMsg) {
                                // case Pre-action 9
                                appendToBody += "Only first ";
                                appendToBody += filterString;
                                appendToBody += " downloaded. Click below to get the rest";
                                appendToBody += " up to the maximum allowed size of ";
                                appendToBody += maxMsgString;
                            }
                            // no attachment under max message
                            // all attachments are above max message
                            // only the remaining part of the body to be downloaded
                            //if (existsMajorOfMaxMsg && !existsMinorOfMaxMsg) {
                                // case Pre-action 8
                            //}
                            else {
                                // case Pre-action 8
                                appendToBody += "Click below to get the rest"; 
                                appendToBody += " of the message up to the maximum allowed size of ";
                                appendToBody += maxMsgString;
                            }
                        } else {
                               // case Pre-action 2
                                appendToBody += "Only first ";
                                appendToBody += filterString;
                                appendToBody += " downloaded. Click below to get the rest.";
                        }
                    }

                }
                else {
                    if (attachNum > 0) {
                        bool existsMinorOfMaxMsg = false;
                        bool existsMajorOfMaxMsg = false;
                        for (int i = 0; i < attachNum; i++) {
                            if (m->getEmailData()->getAttachmentSize(i) > 0 &&
                                (m->getEmailData()->getAttachmentSize(i) < maxMsg)) {
                                existsMinorOfMaxMsg = true;
                            } else {
                                existsMajorOfMaxMsg = true;
                            }
                        }

                        // all attachments are under max message
                        if (!existsMajorOfMaxMsg && existsMinorOfMaxMsg) {
                            // case Pre-action 4
                            char toappend[256];
                            appendToBody += "Click below to download the following attachment(s):\r\n";
                            for (int i = 0; i < attachNum; i++) {
                                long fileSizeKB = m->getEmailData()->getAttachmentSize(i)/1024;
                                sprintf(toappend, "%s (%i Kb)\r\n",
                                                    m->getEmailData()->getAttachmentName(i),
                                                    fileSizeKB == 0 ? 1 : fileSizeKB);
                                appendToBody += toappend;
                            }
                        }
                        // at least an attachments is under max message
                        else if (existsMinorOfMaxMsg) {
                            // case Pre-action 6
                            char toappend[256];
                            appendToBody += "Click below to download the attachment(s) listed below";
                            appendToBody += " up to the maximum allowed size of ";
                            appendToBody += maxMsgString + ":\r\n";
                            for (int i = 0; i < attachNum; i++) {
                                long fileSizeKB = m->getEmailData()->getAttachmentSize(i)/1024;
                                sprintf(toappend, "%s (%i Kb)\r\n",
                                                    m->getEmailData()->getAttachmentName(i),
                                                    fileSizeKB == 0 ? 1 : fileSizeKB);
                                appendToBody += toappend;
                            }

                        }
                        // no attachment under max message
                        // all attachments are above max message
                        //if (existsMajorOfMaxMsg && !existsMinorOfMaxMsg) {
                            // case Pre-action 5
                        // }
                        else {
                            // case Pre-action 5
                            char toappend[256];
                            appendToBody += "None of the attachment(s) listed below";
                            appendToBody += " can be downloaded because they exceed";
                            appendToBody += " the maximum allowed size of ";
                            appendToBody += maxMsgString + ":\r\n";
                            for (int i = 0; i < attachNum; i++) {
                                long fileSizeKB = m->getEmailData()->getAttachmentSize(i)/1024;
                                sprintf(toappend, "%s (%i Kb)\r\n",
                                                    m->getEmailData()->getAttachmentName(i),
                                                    fileSizeKB == 0 ? 1 : fileSizeKB);
                                appendToBody += toappend;
                            }
                        }

                    } else {
                        // case Pre-action 1
                        // no action to perform: all body and attachment are downloaded
                    }
                }

            } else { // is inclusive == true
                if (bodySize > 0) {
                    if (attachNum > 0 ) {
                        // case Post-action 10
                        appendToBody += "Only first ";
                        appendToBody += filterString;
                        appendToBody += " of the message downloaded. The following attachment(s)";
                        appendToBody += " could not be downloaded because the maximum allowed size of ";
                        appendToBody += maxMsgString;
                        appendToBody += " was exceeded by the message:\r\n";
                        char toappend[256];
                        for (int i = 0; i < attachNum; i++) {
                            long fileSizeKB = m->getEmailData()->getAttachmentSize(i)/1024;
                            sprintf(toappend, "%s (%i Kb)\r\n",
                                                m->getEmailData()->getAttachmentName(i),
                                                fileSizeKB == 0 ? 1 : fileSizeKB);
                            appendToBody += toappend;
                        }

                    } else {
                        // case Post-action 3
                        appendToBody += "Only first ";
                        appendToBody += filterString;
                        appendToBody += " downloaded";
                        appendToBody += " (of " + totalBodySizeString + ")";
                        appendToBody += " because the the maximum allowed size of ";
                        appendToBody += maxMsgString;
                        appendToBody += " was exceeded by the message.\r\n";
                    }
                } else {
                    if (attachNum > 0 ) {
                        // case Post-action 6, 8, 9
                        appendToBody += "The following attachment(s) could not be downloaded ";
                        appendToBody += " because the maximum allowed size of ";
                        appendToBody += maxMsgString;
                        appendToBody += " was exceeded by the message:\r\n";
                        char toappend[256];
                        for (int i = 0; i < attachNum; i++) {
                            long fileSizeKB = m->getEmailData()->getAttachmentSize(i)/1024;
                            sprintf(toappend, "%s (%i Kb)\r\n",
                                                m->getEmailData()->getAttachmentName(i),
                                                fileSizeKB == 0 ? 1 : fileSizeKB);
                            appendToBody += toappend;
                        }

                    } else {
                        // case Post-action 1, 2
                        // nothing to do
                    }

                }

            }

/********************************************************/
            if (appendToBody.length() > 0) {
                appendToBody.insert(0, "\r\n=====\r\n");
                bp.appendContent(appendToBody.c_str());
            }
        } else {
            bp.appendContent(" ");
        }

        if (bp.getContent()) {
            // Stream the body in...
            hr = pmsg->OpenProperty(PR_BODY, NULL, 0, MAPI_MODIFY | MAPI_CREATE,
              (LPUNKNOWN*)&pstm);
            EXIT_ON_FAILED(hr);
            wchar_t *content = toWideChar(bp.getContent(), bp.getCharset());
            pstm->Write(content, (wcslen(content)+1)*sizeof(wchar_t), NULL);
            pstm->Release();
            delete [] content;
        }
        if (m->getEmailData()->getIsMailPartial() && foundToBeDownloaded) {

            rgprops[cProps].ulPropTag = PR_MESSAGE_SIZE;
            rgprops[cProps].Value.ul = strlen(bp.getContent());
            ++cProps;

            rgprops[cProps].ulPropTag = PR_MESSAGE_SIZE_EX;
            rgprops[cProps].Value.ul = m->getEmailData()->getTotalEmailSize();
            //
            // To store the real mail to be downloaded
            //
            //rgprops[cProps].Value.ul = emailSizeToDownload;
            ++cProps;
        }
        //
        // Add attachments only if the attachnumber is == 0. If
        //
        AddAttachment(pmsg, m);

    }
    hr = pmsg->SetProps(cProps, rgprops, NULL);
    EXIT_ON_FAILED(hr);

FuncExit:
    if(subj)
        delete [] subj;
    if(from)
        delete [] from;
    if (messId) {
         delete [] messId; messId = NULL;
    }
    if (headers) {
         delete [] headers; headers = NULL;
    }

    LOG.debug("Exiting from SetMessageProps function");
    return hr;
}

void MailServer2Client::setFolderToSync(const wchar_t* t){
    if (server_folderToSync) {
        delete [] server_folderToSync; server_folderToSync = NULL;
    }
    server_folderToSync = wstrdup(t);
}

const wchar_t* MailServer2Client::getFolderToSync() {
    return server_folderToSync;
}


/**
* It deletes a just created email that cannot be completed
* due to low memory on the device or 
*/
HRESULT deleteGhostEmail(LPMAPIFOLDER pfldrInbox, SPropValue* rgprops2) {
    
    HRESULT hr = S_OK;

    if (pfldrInbox == NULL || rgprops2 == NULL) {
        return S_FALSE;
    }

    int len = rgprops2[eePR_ENTRYID].Value.bin.cb;

    ENTRYLIST entryList;
    entryList.cValues = 0;
    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
    entryList.lpbin[0].cb = len;
    MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[0].lpb) );
    memcpy(entryList.lpbin[0].lpb, rgprops2[eePR_ENTRYID].Value.bin.lpb, len);                        
    entryList.cValues++;

    hr = pfldrInbox->DeleteMessages (&entryList, 0, NULL, 0);
    return hr;
}


/*
* Return the ID of the inserted message
*/

wchar_t* MailServer2Client::addMessage(MailClientData* m) {

    LOG.debug("Enter into addMessage function");
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    LPADRBOOK lpAddrBook = NULL;
    ULONG*   rgTags;
    folderNumber = 0;
    BOOL accountFound             = FALSE;

    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1;
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID;
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++)
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID;
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;
    }

    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;

    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;
    LPMAPIFOLDER      pfldrOutbox =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;

    LPSRowSet    pRows = NULL;
    SPropValue*       rgprops2    =  NULL;
    ULONG             cValues2    =  0;
    wchar_t* ret                  = NULL;
    bool msgset = false;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);

    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);


    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);

        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
          break;
        }
        //
        // the display name is the name of the account.
        //
        wchar_t* displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;

        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;
            // Make sure we got the props we need
            if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
            {
              LOG.error("Store missing PR_ENTRYID!");
              hr = E_FAIL;
              break;
            }

            // Open this message store
            hr = pSession->OpenMsgStore (NULL,
                           psrs->aRow[0].lpProps[0].Value.bin.cb,
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value.
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);

            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);

            if(!rgprops){
                LOG.error("Error getting folder from message store.");
                goto FuncExit;
            }

            //
            // watch at the folder that we want to sync
            //
            for (i = 0; i < folderNumber; i++) {
                wchar_t* tmp = c2wc(server_folderToSync[i]);

                if (wcscmp(tmp, m->getParent()) != 0) {
                    if (tmp) delete [] tmp; tmp = NULL;
                    continue;
                }

                // open the message store
                hr = pStore->OpenEntry (rgprops[i].Value.bin.cb,
                            (LPENTRYID)rgprops[i].Value.bin.lpb,
                            NULL,
                            MAPI_MODIFY,
                            NULL,
                            (IUnknown **) &pfldrInbox);
                EXIT_ON_FAILED (hr);

                if (!pfldrInbox){
                    LOG.error("MailServer2Client::addMessage: pfldrInbox NULL!");
                    goto FuncExit;
                }

                ret = doesMessageExist(m->getEmailData()->getEmailItem().getMessageId(),
                                     pfldrInbox, server_folderToSync[i]);

                if (ret != NULL ) {
                    //ret = new wchar_t[64];
                    //wcscpy(ret, ITEM_ARLEADY_EXISTS);

                }
                else {
                    hr = pfldrInbox->CreateMessage (NULL, 0, &pmsg);
                    EXIT_ON_FAILED (hr);

                    if (!pmsg){
                        LOG.error("MailServer2Client::addMessage: pmsg NULL!");
                        goto FuncExit;
                    }

                    // populate the array with the binary ID
                    hr = pmsg->GetProps ((SPropTagArray *) rgMsgTags, MAPI_UNICODE, &cValues2, &rgprops2);
                    EXIT_ON_FAILED(hr);
                                                            
                    try {                       
                    // Now set the props on the message....
                    hr = SetMessageProps (pSession, pmsg, m);

                    } catch (OutOfMemoryException&) {
                        deleteGhostEmail(pfldrInbox, rgprops2);
                        throw OutOfMemoryException(0, -50); 
                    }
                    if (hr != S_OK) {
                        deleteGhostEmail(pfldrInbox, rgprops2);
                        LOG.error("Error in insert new mail message.");
                        goto FuncExit;
                    }

                    LOG.debug("Message props set.");

                    //
                    // Get the ID of the new message
                    //
                    if (rgprops2[eePR_ENTRYID].ulPropTag == PR_ENTRYID) {
                        ret = convertBinaryToWChar(rgprops2[eePR_ENTRYID].Value.bin, server_folderToSync[i]);
                        LOG.debug("Message enry id: %ls", ret);
                    }
                    else {
                        LOG.error("Error getting msg entry id.");
                    }

                }
                //
                // If there is a mail in Outbox with the same messageId of a mail just inserted in Sent,
                // then delete the Outbox one.
                //
                /*
                if (server_folderToSync[i] == 'S') {
                    if (messageExistInOubox_deleteIt(pStore, m->getEmailData()->getEmailItem().getMessageId()) ){
                        LOG.debug("Message in outbox succesfully deleted. Message id: %s", m->getEmailData()->getEmailItem().getMessageId());
                    } else {
                        LOG.debug("No message in outbox to delete. Message id: %s", m->getEmailData()->getEmailItem().getMessageId());
                    }

                }
                */

                msgset = true;
                break;  // found.
            }
        }

        // Clean up
        MAPIFreeBuffer (rgprops);
        MAPIFreeBuffer (rgprops2);
        MAPIFreeBuffer (rgpropsMsg);
        FreeProws   (psrs);

        rgprops   =  NULL;
        rgpropsMsg =  NULL;
        psrs    =  NULL;

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrInbox);
        RELEASE_OBJ (pStore);
    }
    if (accountFound == FALSE) {
        LOG.error("No " PROVIDER " account found");
    }
FuncExit:

    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;
    LOG.debug("Exiting from addMessage function");
    return ret;
}


/*
* Return the ID of the inserted message
*/
HRESULT MailServer2Client::updateMessage(MailClientData* m) {
    LOG.debug("Enter into updateMessage function");
    if (!m) {
        LOG.error("Update message: mail client data is NULL.");
        return -1;
    }

    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    ULONG             returned    =  0;
    IMessage*         pmsg        =  NULL;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

   // Decode the id
    char* eid = NULL;
    size_t len = decodeEntryId(m->getEntryId(), &eid);
    if(!eid){
        LOG.error("UpdateMessage: it is not possible to update the mail with id %S. Continue sync...", m->getEntryId());        
        return -1;
    }

    ENTRYLIST entryList;

    entryList.cValues = 0;
    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
    entryList.lpbin[entryList.cValues].cb = len;
    MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
    memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
    entryList.cValues++;

    hr = pSession->OpenEntry(entryList.lpbin[0].cb,
        (ENTRYID*)entryList.lpbin[0].lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &pmsg);

    EXIT_ON_FAILED (hr);
    if(!pmsg){
        LOG.error("Pmsg is null.");
        goto FuncExit;
    }

    delete [] eid;

    // Now set the props on the message....
    //hr = SetMessageProps (pSession, pmsg, m, true); // update only the flags
    hr = SetMessageProps (pSession, pmsg, m); // the decision in done by the MailMessage
    EXIT_ON_FAILED(hr);

    MAPIFreeBuffer (entryList.lpbin);

FuncExit:

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pSession);

    LOG.debug("Exiting from updateMessage function");
    return hr;
}

/*
* Return the ID of the inserted message
*/

HRESULT MailServer2Client::deleteMessage(MailClientData* m) {
    LOG.debug("Enter into deleteMessage function");
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
    BOOL accountFound             = FALSE;

    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1;
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID;
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++)
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID;
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;
    }

    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;

    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;

    LPSRowSet          pRows        = NULL;
    SPropValue*       rgprops2     =  NULL;
    ULONG             cValues2     =  0;
    wchar_t*          ret         = NULL;
    wchar_t*          displayName  = NULL;
    bool msgset = false;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);

    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);

        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
            break;
        }
        //
        // the display name is the name of the account.
        //
        displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;

        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;
            // Make sure we got the props we need
            if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
            {
              LOG.error("Store missing PR_ENTRYID!");
              hr = E_FAIL;
              break;
            }

            // Open this message store
            hr = pSession->OpenMsgStore (NULL,
                           psrs->aRow[0].lpProps[0].Value.bin.cb,
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value.
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);

            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);

            ASSERT (rgprops);

            //
            // watch at the folder that we want to sync
            //
            for (i = 0; i < folderNumber; i++) {
                wchar_t* t = c2wc(server_folderToSync[i]);
                // we don't want to delete the mail that may be in Outbox.
                // Otherwise the O/ can be a mail that has been moved in Sent folder.
                // it can happen that a mail sent and moved in Sent could be removed.
                if (wcscmp(t, m->getParent()) != 0 || wcscmp(t, TEXT("O")) == 0) {
                    if (t) {
                        delete [] t; t = NULL;
                    }
                    continue;
                }

                // open the message store
                hr = pStore->OpenEntry (rgprops[i].Value.bin.cb,
                            (LPENTRYID)rgprops[i].Value.bin.lpb,
                            NULL,
                            MAPI_MODIFY,
                            NULL,
                            (IUnknown **) &pfldrInbox);
                EXIT_ON_FAILED (hr);

                ASSERT (pfldrInbox);

                // Decode the id
                char* eid = NULL;
                size_t len = decodeEntryId(m->getEntryId(), &eid);
                if(!eid){
                    LOG.error("DeleteMessage: it is not possible to delete the mail with id %S. Continue sync...", m->getEntryId());
                    return -1;
                }

                ENTRYLIST entryList;

                entryList.cValues = 0;
                MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
                entryList.lpbin[entryList.cValues].cb = len;
                MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
                memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
                entryList.cValues++;

                delete [] eid;

                hr = pfldrInbox->DeleteMessages (&entryList, 0, NULL, 0);
                EXIT_ON_FAILED (hr);

                MAPIFreeBuffer (entryList.lpbin);

                msgset = true;
                break;
            }
        }
        // Clean up
        MAPIFreeBuffer (rgprops);
        MAPIFreeBuffer (rgprops2);
        MAPIFreeBuffer (rgpropsMsg);
        FreeProws   (psrs);

        rgprops   =  NULL;
        rgpropsMsg =  NULL;
        psrs    =  NULL;

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrInbox);
        RELEASE_OBJ (pStore);
    }
    if (accountFound == FALSE) {
        LOG.error("No " PROVIDER " account found");
        goto FuncExit;
    }
FuncExit:

    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;
    LOG.debug("Exiting from deleteMessage function");
    return hr;
}

bool MailServer2Client::messageExistInOubox_deleteIt(IMsgStore* pStore, IN const char* msgid) {

    if (msgid == NULL)
        return false;

    wchar_t* msgID = toWideChar(msgid);
    ULONG           cValues     =   0;
    SPropValue  *   rgprops     =   NULL;
    HRESULT hr                  = S_OK;
    bool bFound                 = false;

    SBinary    sbEntry = {0};
    LPMAPITABLE pContentsTable = NULL;
    LPMAPIFOLDER pfldrOutbox =   NULL;
    LPSRowSet pRows = NULL;
    SRow aRow = {0};

    ULONG rgTags[] = { 1, PR_IPM_OUTBOX_ENTRYID };

    enum {
        ePR_ENTRYID,            // 0
        ePR_CUSTOM_MESSAGE_ID,            // 2
        NUM_COLS                // number of columns (put it to the end)
    };

    // These tags represent the message information we would like to pick up
    SizedSPropTagArray(NUM_COLS, Columns) =
    {
        NUM_COLS,
        PR_ENTRYID,
        PR_CUSTOM_MESSAGE_ID
    };

    hr = pStore->GetProps((SPropTagArray *)rgTags, MAPI_UNICODE, &cValues, &rgprops);
    EXIT_ON_FAILED(hr);

    ASSERT(rgprops);
    ASSERT(rgprops[0].ulPropTag == PR_IPM_OUTBOX_ENTRYID);

    hr = pStore->OpenEntry (rgprops[0].Value.bin.cb,
                            (LPENTRYID)rgprops[0].Value.bin.lpb,
                            NULL,
                            MAPI_MODIFY,
                            NULL,
                            (IUnknown **) &pfldrOutbox);

    EXIT_ON_FAILED(hr);

    hr = pfldrOutbox->GetContentsTable(0, &pContentsTable);
    EXIT_ON_FAILED(hr);

    hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    // iterate message one by one (each time only read 1)
    while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows))) {
        // this is the end of the content table
        if(pRows->cRows == 0) break;

        // check this Row
        aRow = pRows->aRow[0];

        // Correctly returned?
        if ((&aRow.lpProps[ePR_ENTRYID])->ulPropTag == PR_ENTRYID
            && (&aRow.lpProps[ePR_CUSTOM_MESSAGE_ID])->ulPropTag == PR_CUSTOM_MESSAGE_ID) {

            // Is CUSTOM_MESSAGE_ID found?
            wchar_t* mm = (&aRow.lpProps[ePR_CUSTOM_MESSAGE_ID])->Value.lpszW;

            // the message-id there is in the client has the O/AABBCC....
            // the message-id sent by server could have only AABBCC...

            if (wcsstr(mm, msgID) != 0) {

                wchar_t* entryId = convertBinaryToWChar(aRow.lpProps[ePR_ENTRYID].Value.bin, 'O');
                char* eid = NULL;
                size_t len = decodeEntryId(entryId, &eid);
                if(!eid){
                    LOG.error("messageExistInOubox_deleteIt: error decoding entryId %S", entryId);
                    return false;
                }

                ENTRYLIST entryList;

                entryList.cValues = 0;
                MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
                entryList.lpbin[entryList.cValues].cb = len;
                MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
                memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
                entryList.cValues++;

                delete [] eid;
                delete [] entryId;

                hr = pfldrOutbox->DeleteMessages (&entryList, 0, NULL, 0);
                EXIT_ON_FAILED (hr);
                bFound = true;

                break;
            }
        } // if

        // Free pRows for each row
        FreeProws(pRows);
        pRows = NULL;
    } // while

FuncExit:
    if(pRows) {
        // Free each element in the pRows buffer
        FreeProws(pRows);
        pRows = NULL;
    }
    RELEASE_OBJ(pContentsTable);

    if (msgID)  {
        delete [] msgID; msgID = NULL;
    }

    return bFound;
}


wchar_t* MailServer2Client::doesMessageExist (IN const char* msgid, LPMAPIFOLDER pfldrInbox, wchar_t folder) {

    if (msgid == NULL)
        return false;

    wchar_t* msgID = toWideChar(msgid);
    HRESULT hr = S_OK;
    wchar_t* ret = NULL;
    SBinary    sbEntry = {0};
    LPMAPITABLE pContentsTable = NULL;
    LPSRowSet pRows = NULL;
    SRow aRow = {0};

    enum {
        ePR_ENTRYID,            // 0
        ePR_CUSTOM_MESSAGE_ID,            // 2
        NUM_COLS                // number of columns (put it to the end)
    };

    // These tags represent the message information we would like to pick up
    SizedSPropTagArray(NUM_COLS, Columns) =
    {
        NUM_COLS,
        PR_ENTRYID,
        PR_CUSTOM_MESSAGE_ID
    };

    hr = pfldrInbox->GetContentsTable(0, &pContentsTable);
    EXIT_ON_FAILED(hr);

    hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    // iterate message one by one (each time only read 1)
    while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows))) {
        // this is the end of the content table
        if(pRows->cRows == 0) break;

        // check this Row
        aRow = pRows->aRow[0];

        // Correctly returned?
        if ((&aRow.lpProps[ePR_ENTRYID])->ulPropTag == PR_ENTRYID
            && (&aRow.lpProps[ePR_CUSTOM_MESSAGE_ID])->ulPropTag == PR_CUSTOM_MESSAGE_ID) {

            // Is CUSTOM_MESSAGE_ID found?
            wchar_t* mm = aRow.lpProps[ePR_CUSTOM_MESSAGE_ID].Value.lpszW;
            if (wcscmp(mm, msgID) == 0) {
                ret = convertBinaryToWChar(aRow.lpProps[ePR_ENTRYID].Value.bin, folder);
                break;
            }
        } // if

        // Free pRows for each row
        FreeProws(pRows);
        pRows = NULL;
    } // while

FuncExit:
    if(pRows) {
        // Free each element in the pRows buffer
        FreeProws(pRows);
        pRows = NULL;
    }
    RELEASE_OBJ(pContentsTable);

    if (msgID)  {
        delete [] msgID; msgID = NULL;
    }

    return ret;
}



HRESULT SetReplyTo(LPMAPISESSION lpSession, IMessage* lpMessage, LPSTR pszReplyTo)
{

    HRESULT hRes = S_OK;
    LPADRLIST lpAddrList = NULL;
    LPADRBOOK lpAddrBook = NULL;
    SPropValue pspvReply[2];
    LPFLATENTRYLIST lpReplyEntryList = NULL;
    LPFLATENTRY lpReplyEntry = NULL;
    int cValues = 2;
    ULONG i = 0;

    // Allocate an address list structure.
    int cb = CbNewADRLIST(1);

    hRes = MAPIAllocateBuffer(cb, (LPVOID *)&lpAddrList);
    if (FAILED(hRes)) goto Cleanup;

    ZeroMemory(lpAddrList, cb);

    hRes = MAPIAllocateBuffer(
        cValues * sizeof(SPropValue),
        (LPVOID FAR *)&lpAddrList->aEntries[0].rgPropVals);
    if (FAILED(hRes)) goto Cleanup;

    // Open the address book to resolve Reply To recipient.
    hRes = lpSession->OpenAddressBook(0, NULL, 0, &lpAddrBook);
    if (FAILED(hRes)) goto Cleanup;

    // Set up address list structure.
    lpAddrList->cEntries = 1;
    lpAddrList->aEntries[0].rgPropVals[0].ulPropTag = PR_DISPLAY_NAME;
    lpAddrList->aEntries[0].rgPropVals[0].Value.lpszA = pszReplyTo;

    lpAddrList->aEntries[0].rgPropVals[1].ulPropTag = PR_RECIPIENT_TYPE;
    lpAddrList->aEntries[0].rgPropVals[1].Value.l = MAPI_TO;
    lpAddrList->aEntries[0].cValues = cValues;

    // Resolve name.
    hRes = lpAddrBook->ResolveName(0, 0, NULL, lpAddrList);
    if (FAILED(hRes)) goto Cleanup;

    // Get the Entry ID.
    for (i = 0; i < lpAddrList->aEntries[0].cValues; i++)
    {
        if (lpAddrList->aEntries[0].rgPropVals[i].ulPropTag == PR_ENTRYID)
        {
            // Allocate a new FLATENTRY structure for
            // the PR_REPLY_RECIPIENT_ENTRIES property.
            cb = CbNewFLATENTRY(
                   lpAddrList->aEntries[0].rgPropVals[i].Value.bin.cb);
            hRes = MAPIAllocateBuffer(cb, (LPVOID *)&lpReplyEntry);
            if (FAILED(hRes)) goto Cleanup;
            ZeroMemory((VOID *)lpReplyEntry, cb);

            // Copy the bits of the Entry ID into the FLATENTRY structure.
            CopyMemory(lpReplyEntry->abEntry,
                       lpAddrList->aEntries[0].rgPropVals[i].Value.bin.lpb,
                       lpAddrList->aEntries[0].rgPropVals[i].Value.bin.cb);
            lpReplyEntry->cb = lpAddrList->
                                 aEntries[0].rgPropVals[i].Value.bin.cb;

            // Allocate a new FLATENTRYLIST structure.
            // This is what is stored in PR_REPLY_RECIPIENT_ENTRIES.
            cb = CbNewFLATENTRYLIST(cb);
            hRes = MAPIAllocateBuffer(cb, (LPVOID *)&lpReplyEntryList);
            if (FAILED(hRes)) goto Cleanup;
            ZeroMemory((VOID *)lpReplyEntryList, cb);

            // Copy the bits of the FLATENTRY into the FLATENTRYLIST.
            lpReplyEntryList->cEntries  = 1;
            lpReplyEntryList->cbEntries = CbFLATENTRY(lpReplyEntry);
            CopyMemory(lpReplyEntryList->abEntries,
                       lpReplyEntry, CbFLATENTRY(lpReplyEntry));

            // Force exit from the loop.
            i = lpAddrList->aEntries[0].cValues;
        }
    }

    // Set PR_REPLY_RECIPIENT_NAMES.
    pspvReply[0].ulPropTag = PR_REPLY_RECIPIENT_NAMES;
    pspvReply[0].Value.lpszA = pszReplyTo;

    // Set PR_REPLY_RECIPIENT_ENTRIES.
    pspvReply[1].ulPropTag = PR_REPLY_RECIPIENT_ENTRIES;
    // Allocate memory in the lpb to hold the FLATENTRYLIST
    hRes = MAPIAllocateBuffer(CbFLATENTRYLIST(lpReplyEntryList),
                              (LPVOID *)&pspvReply[1].Value.bin.lpb);
    // Copy the memory into the SPropValue.
    CopyMemory(pspvReply[1].Value.bin.lpb,
               lpReplyEntryList,
               CbFLATENTRYLIST(lpReplyEntryList));
    pspvReply[1].Value.bin.cb = CbFLATENTRYLIST(lpReplyEntryList);

    // Set the properties.
    hRes = lpMessage->SetProps(2,
                               pspvReply,
                               NULL);

Cleanup:
    if (lpReplyEntry) MAPIFreeBuffer(lpReplyEntry);
    if (lpReplyEntryList) MAPIFreeBuffer(lpReplyEntryList);
    //if (lpAddrList) FreePadrlist(lpAddrList);
    if (lpAddrBook) lpAddrBook->Release();
    return hRes;
}

char* getMessageID(IMessage* pmsg, MailClientData* m) {

    SPropValue* msgprops = NULL;
    ULONG    cmsgValue =  0;
    char* id = NULL;
    HRESULT hr = pmsg->GetProps ((LPSPropTagArray)&Columns, MAPI_UNICODE, &cmsgValue,
       &msgprops);
    EXIT_ON_FAILED (hr);

    if ( !msgprops ) {
        LOG.error("getMessageID: can't get message props.");
        goto FuncExit;
    }

    if (msgprops[ePR_ENTRYID].ulPropTag != PR_ENTRYID) {
        LOG.error("getMessageID: entry id not found.");
        goto FuncExit;
    }
    const wchar_t* folder = m->getParent();
    wchar_t f = folder ? folder[0] : 'O'; // set default O if folder is null;
    id = convertBinaryToChar(msgprops[ePR_ENTRYID].Value.bin, f);

FuncExit:
    return id;
}

/**
* The function needs to prepare the mail in the Outbox folder
* with the latest log attached.
*/
int sendLog() {

    MailClientData* mcd = new MailClientData(TEXT("undef"));
    mcd->setParent(TEXT("O"));

    mcd->getEmailData()->setRead(false);
    mcd->getEmailData()->setForwarded(false);
    mcd->getEmailData()->setReplied(false);
    mcd->getEmailData()->setReceived(false);
    mcd->getEmailData()->setCreated(false);
    mcd->getEmailData()->setDeleted(false);
    mcd->getEmailData()->setFlagged(false);

    MailMessage& msg = mcd->getEmailData()->getEmailItem();
        
    // This date doesn't matter at all for the Outbox.
    // it needs only the go over the addMessage check
    BasicTime bDate;        
    SYSTEMTIME systemTime;
    FILETIME ft;
    GetSystemTime(&systemTime);
    SystemTimeToFileTime(&systemTime, &ft);    
    FileTimeToBasicDate(ft, &bDate);
    msg.setDate(bDate);
    
    msg.setSubject("[Report] Synclog");    
    
    char tt[64];
    sprintf(tt, "%s%s%s%s", "fwm-logs", "@", "funambol.", "com");
    
    msg.setTo(tt);

    BodyPart bodyPart;    
    bodyPart.setContent("This is the log of the user");
    msg.setBody(bodyPart);

    BodyPart attach;

    WCHAR tmpPath[64];
    WCHAR tmpFileName[MAX_PATH];  // System constants for the path
    GetTempPath(64, tmpPath);
    wsprintf(tmpFileName, TEXT("%s\\synclog.txt"), tmpPath);
    
    int ret = 0;    
    char buffer[16384];
    DWORD MAX_LOG_SIZE = 500000;

    HANDLE hfile = CreateFile(TEXT("\\synclog.txt"), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    HANDLE hfilew = CreateFile(tmpFileName, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    
    DWORD dwSize = 0, lpFileSizeHigh = 0;
    dwSize = GetFileSize(hfile, &lpFileSizeHigh);
    if (dwSize == 0xFFFFFFFF) {
        dwSize = 0;
    }

    if (dwSize > MAX_LOG_SIZE) {
        DWORD ss = SetFilePointer(hfile, dwSize - MAX_LOG_SIZE, 0, FILE_BEGIN);
    }
        
    DWORD nBytesRead = 0;
    DWORD nBytesWrote = 0;
    while(hfile && hfilew) {
        BOOL bResult = ReadFile(hfile, &buffer, 16384, &nBytesRead, NULL);
        // Check for the end of file. 
        
        if (bResult &&  nBytesRead == 0) { 
            CloseHandle(hfile);
            CloseHandle(hfilew);
            ret = 1;
            break;

        } else {
            WriteFile(hfilew, buffer, nBytesRead, &nBytesWrote, NULL);
        }
    }

    //int ret = CopyFile(TEXT("\\synclog.txt"), tmpFileName, false);    
    // ret = 0 means failure

    if (ret) {

        attach.setFilename("synclog.txt");
        attach.setContent(_wcc(tmpFileName));
        attach.setEncoding("base64"); // currently the ppc default

        mcd->getEmailData()->getEmailItem().addAttachment(attach);

        MailServer2Client msc;
        msc.setFolderToSync(TEXT("O"));
        msc.addMessage(mcd);

        delete mcd;

        ret = 0;
    }
    
    return ret;
}



/*
* This add partial attachments to the mail if any.
* Currently this function is not used. It must be used in addition to the flag messageStatus
*
* if (m->getEmailData()->getIsMailPartial()) {
*    if (m->getEmailData()->getRemainingAttachNumber() > 0) { // there is any attach to be downloaded
*        messageStatus = messageStatus | MSGSTATUS_PENDING_ATTACHMENTS;
*    }
*    if (m->getEmailData()->getRemainingBodySize() > 0) { // there is any body to be downloaded
*        messageStatus = messageStatus | MSGSTATUS_HEADERONLY;
*        if (m->getEmailData()->getRemainingAttachNumber() == 0) {
*            messageStatus = messageStatus | MSGSTATUS_PARTIAL;
*        }
*    }
* }
*/

HRESULT MailServer2Client::AddPartialAttachments(LPMESSAGE pmsg, MailClientData* m)
{
    LOG.debug("Enter into AddAttachment function");
	HRESULT hr = S_OK;
    BodyPart* bodyPart = NULL;


    wchar_t* filename    = toWideChar("download all message");

    SPropValue  prop[3];
	IAttach*    pAttachment = NULL;
	ULONG       ulAttachmentNum = 0;
	FILE*       fileAttach = NULL;;

    memset(prop, 0, sizeof(prop));

    hr = pmsg->CreateAttach(NULL, 0, &ulAttachmentNum, &pAttachment);
	EXIT_ON_FAILED(hr);

	// Set the initial properties on the attachment
    // no data actually there yet
    /*
	prop[0].ulPropTag = PR_ATTACH_METHOD;
	prop[0].Value.ul = ATTACH_BY_VALUE;

	prop[1].ulPropTag = PR_ATTACH_DATA_BIN;
    */
    prop[0].ulPropTag = PR_ATTACH_FILENAME;
	prop[0].Value.LPSZ = filename;

    prop[1].ulPropTag    =   PR_ATTACH_SIZE;
    prop[1].Value.ul     =   5000;

    prop[2].ulPropTag    =   PR_MSG_STATUS;
    prop[2].Value.ul     =   MSGSTATUS_HEADERONLY;


    hr = pAttachment->SetProps(3, prop, NULL);
	EXIT_ON_FAILED(hr);



FuncExit:
    LOG.debug("Exiting from AddAttachment function");
    return hr;
}

/*
 HRESULT AddTnefData(IAddrBook* pAddrBook, IMessage* pSourceMsg, IMessage* pTargetMsg, unsigned long ulTnefFlags, SPropTagArray* pExcludeProps)
 {
   // we need at least IAddrBook and the source IMessage


   int i;
   DWORD num;
   WORD wKeyVal=rand();
   HRESULT hr;
   ULONG ulPropTag, ulCount;
   BYTE MIME_OID[]={0x2A,0x86,0x48,0x86,0xF7,0x14,0x03,0x0A,0x04};
   enum {ePR_ENTRYID, NUM_COLS};
   static SizedSPropTagArray(NUM_COLS, PropMailEID) = {NUM_COLS, {PR_ENTRYID}};

   // TNEF stuff (GUID for message header properties)
   static const GUID ID_X_MS_TNEF_CORRELATOR = {0x00020386, 0x0000, 0x0000,{0xc0,0x00,0x0,0x00,0x00,0x00,0x00,0x46 } };
   MAPINAMEID nameid={(GUID*)&ID_X_MS_TNEF_CORRELATOR, MNID_STRING, (LONG)L"X-MS-TNEF-Correlator"};
   MAPINAMEID *rgpnameid[1]={&nameid};
   ITnef* pTnef=NULL;

   char *pszWinmail="winmail.dat", *pszTnefCorrelationKey=NULL;
   IAttach* pAttach=NULL;
   IStream* pStream=NULL;
   SPropTagArray PropTagArrNul={0, 0}, *lpSPropTags=NULL;
   SPropValue props[20], *pMailEID=NULL;

   // convert PR_ENTRYID of source message into hex string to be used as TNEF correlation key
   if(FAILED(hr=pSourceMsg->GetProps((LPSPropTagArray)&PropMailEID, 0, &ulCount, &pMailEID))) goto err;
   if(!(pszTnefCorrelationKey=new char[2*pMailEID->Value.bin.cb+1]))
   {
     hr=E_OUTOFMEMORY;                                                           // new failed: out of memory
     goto err;
   }
   for(i=0; i<pMailEID->Value.bin.cb; sprintf(pszTnefCorrelationKey+2*i, "%02X", pMailEID->Value.bin.lpb[i]), ++i);

   if(!pTargetMsg)     pTargetMsg=pSourceMsg;                                    // create the TNEF stream in the source message if required
   if(!pExcludeProps)  pExcludeProps=&PropTagArrNul;                             // default is: don't exclude *ANY* properties

   // compute X-MS-TNEF-Correlator named property to trick OL/EX into adding a custom header line
   // called "X-MS-TNEF-Correlator:" with value pszTnefCorrelationKey
   if(FAILED(hr=pTargetMsg->GetIDsFromNames(1, rgpnameid, MAPI_CREATE, &lpSPropTags))) goto err;
   ulPropTag=lpSPropTags->aulPropTag[0];
   ulPropTag=PROP_TAG(PT_STRING8, PROP_ID(ulPropTag));
   props[i=0].ulPropTag    =ulPropTag;
   props[i++].Value.lpszA  =pszTnefCorrelationKey;
   if(FAILED(hr=pTargetMsg->SetProps(i, props, NULL))) goto err;
   if(FAILED(hr=pTargetMsg->SaveChanges(KEEP_OPEN_READWRITE))) goto err;

   // set some attachment properties: filename, method, MIME-type and PR_TNEF_CORRELATION_KEY which should
   // have an identical binary value as the "X-MS-TNEF-Correlator:" header because OL is verifying the property
   // against the header information.
   if(FAILED(hr=pTargetMsg->CreateAttach(NULL, 0, &num, &pAttach))) goto err;    // create a raw attachment
   props[i=0].ulPropTag    =PR_ATTACH_LONG_FILENAME;                             // the attachment's long pszFileName
   props[i++].Value.lpszA  =pszWinmail;
   props[i].ulPropTag      =PR_ATTACH_FILENAME;                                  // the attachment's (short) pszFileName (which is the same as the long pszFileName)
   props[i++].Value.lpszA  =pszWinmail;
   props[i].ulPropTag      =PR_ATTACH_METHOD;                                    // the attachment method
   props[i++].Value.l      =ATTACH_BY_VALUE;
   props[i].ulPropTag      =PR_ATTACH_MIME_TAG;                                  // the attachment's MIME type
   props[i++].Value.lpszA  ="application/ms-tnef";
   props[i].ulPropTag      =PROP_TAG(PT_BOOLEAN,0x7FFE);
   props[i++].Value.b      =0;
   props[i].ulPropTag      =PROP_TAG(PT_LONG,0x7FFD);
   props[i++].Value.l      =0;
   props[i].ulPropTag      =PR_ATTACH_TAG;                                       // signal, that attachment has MIME info
   props[i].Value.bin.cb   =sizeof(MIME_OID);
   props[i++].Value.bin.lpb=(BYTE*)MIME_OID;
   props[i].ulPropTag      =PR_TNEF_CORRELATION_KEY;                             // write the correlation key
   props[i].Value.bin.lpb  =(unsigned char*)pszTnefCorrelationKey;
   props[i++].Value.bin.cb =strlen(pszTnefCorrelationKey);
   if(FAILED(hr=pAttach->SetProps(i, props, NULL))) goto err;                    // write all properties at once

   // next, we open an IID_IStream stream interface to connect the TNEF stream to,
   // add all properties of the source message and finish the TNEF stream
   if(FAILED(hr=pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, STGM_WRITE, MAPI_CREATE|MAPI_MODIFY, (IUnknown**)&pStream))) goto err;
   if(FAILED(hr=OpenTnefStreamEx(NULL, pStream, "STREAM_TNEF", TNEF_ENCODE, pSourceMsg, wKeyVal, pAddrBook, &pTnef))) goto err;
   if(FAILED(hr=pTnef->AddProps(ulTnefFlags, 0, NULL, pExcludeProps))) goto err;
   if(FAILED(hr=pTnef->Finish(0, &wKeyVal, NULL))) goto err;

   if(FAILED(hr=pStream->Commit(STGC_DEFAULT))) goto err;                        // commit data
   if(FAILED(hr=pAttach->SaveChanges(FORCE_SAVE))) goto err;                     // save changes, forcing the attachment to close
   if(FAILED(hr=pTargetMsg->SaveChanges(KEEP_OPEN_READWRITE))) goto err;         // save changes to target message but keep it open for later mods

 err:
   // release the interfaces we have openened and free MAPI buffers
   if(lpSPropTags) MAPIFreeBuffer(lpSPropTags);
   if(pTnef)       pTnef->Release();
   if(pStream)     pStream->Release();
   if(pAttach)     pAttach->Release();
   if(pMailEID)    MAPIFreeBuffer(pMailEID);
   if(pszTnefCorrelationKey) delete[] pszTnefCorrelationKey;

   return hr;
 }
 */