/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "notify/AddressChangeSyncSource.h"
#include "base/util/utils.h"
#include "base/LOG.h"
#include "pim/Utils.h"
#include "notify/checknet.h"
#include "notify/s4n_service.h"

static int all   = -1;
static int allkeys = -1;
static ArrayList items;

static char msisdn[40] = "" ;

// Set a new MS-ISDN number
void setMSIsdn(const char *val) {
    sprintf(msisdn, "%s", val);
}

// Get the current MS-ISDN number
const char * getMSIsdn() { return msisdn; }

static char *getTcpAddress()
{
    char *address = new char [ strlen(getLocalAddress()) + 10 ];

    sprintf(address, "%s:%d", getLocalAddress(), getListenerPort() );
    return address;
}

//------------------------------------------------------------------ Constructor
AddressChangeSyncSource::AddressChangeSyncSource(const wchar_t* name, AbstractSyncSourceConfig* config )
    : BaseSyncSource(name, config) {}


//--------------------------------------------------------------- Public Methods
SyncItem* AddressChangeSyncSource::getFirstItem() {
    SyncItem item;

    if ( !items.isEmpty() ) {
        items.clear();
    }
    item.setKey(TEXT("ip"));

    char *addr = getTcpAddress();
    item.setData( addr, strlen(addr) );
    items.add(item);
    delete [] addr;


    item.setKey(TEXT("msisdn"));
    item.setData( msisdn, strlen(msisdn) );
    items.add(item);

    all = 0;
    return (SyncItem*)items[all];
}

SyncItem* AddressChangeSyncSource::getNextItem() {
    if( all == -1)
        return NULL;
    all++;
    if (all == items.size()) {
        return NULL;
    }
    return (SyncItem*)items[all];
}

SyncItem* AddressChangeSyncSource::getFirstNewItem() {
    return getFirstItem();
}

SyncItem* AddressChangeSyncSource::getNextNewItem() {
    return getNextItem();
}

SyncItem* AddressChangeSyncSource::getFirstUpdatedItem() {
    return getFirstItem();
}

SyncItem* AddressChangeSyncSource::getNextUpdatedItem() {
    return getNextItem();
}

SyncItem* AddressChangeSyncSource::getFirstDeletedItem() {
    return NULL;
}

SyncItem* AddressChangeSyncSource::getNextDeletedItem() {
    return NULL;
}

void AddressChangeSyncSource::setItemStatus(const wchar_t* key, int status) {
}

int AddressChangeSyncSource::addItem(SyncItem& item) {
    return 200;
}

int AddressChangeSyncSource::updateItem(SyncItem& item) {
    return 200;
}

int AddressChangeSyncSource::deleteItem(SyncItem& item) {
    return 200;
}

int AddressChangeSyncSource::beginSync() {
    return 0;
}
int AddressChangeSyncSource::endSync() {
    return 0;
}
int AddressChangeSyncSource::removeAllItems() {
    return 0;
}

ArrayElement* AddressChangeSyncSource::clone() {
    return new AddressChangeSyncSource(getName(), &(getConfig()));
}

SyncItem *AddressChangeSyncSource::getFirstItemKey(void) {
    SyncItem item;

    if ( !items.isEmpty() ) {
        items.clear();
    }
    wchar_t ipaddress[64];

    getIPAddress(ipaddress);

    item.setKey(TEXT("ip"));
    items.add(item);

    LOG.debug("Local IP: %S", ipaddress );

    item.setKey(TEXT("msisdn"));
    items.add(item);

    allkeys = 0;
    return (SyncItem*)items[allkeys];
}

SyncItem *AddressChangeSyncSource::getNextItemKey(void) {
    if( allkeys == -1)
        return NULL;
    allkeys++;
    if (all == items.size()) {
        return NULL;
    }
    return (SyncItem*)items[allkeys];
}
