/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef INCL_CONFIG_PARAMS
#define INCL_CONFIG_PARAMS

#include "base/fscapi.h"
#include "base/util/ArrayList.h"
#include "base/util/ArrayElement.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include <string>

using namespace std;


#define CP_DIRTY_VISIBLE_NAME           0x00001
#define CP_DIRTY_EMAIL_ADDRESS          0x00002
#define CP_DIRTY_PUSH_STATUS            0x00004
#define CP_DIRTY_EMAIL_FILTER_STATUS    0x00008
//#define CP_DIRTY_EMAIL_SIGNATURE      0x00010
//#define CP_DIRTY_IP_ADDRESS           0x00020

#define CP_DIRTY_EMAIL_ACCOUNT_ALL      (CP_DIRTY_VISIBLE_NAME | CP_DIRTY_EMAIL_ADDRESS)
#define CP_DIRTY_ALL                    0xfffff


class ConfigParams {

private:

   string visibleName;
   string emailAddress;
 //string emailSignature;       // Not yet implemented.
 //string ipAddress;            // Not used.
   bool pushStatus;
   bool emailFilterStatus;

   /// Flag used to know which parameters have been set.
   /// We save those values in endSync().
   int dirty;

   void setDirtyFlag(int flag)  { dirty |= flag; }

   bool readEmailAccountParams();


public:

    /**
     * Constructor: create a Container
     */
    ConfigParams();
    ~ConfigParams() ;

    const string getVisibleName()       { return visibleName;       } 
    const string getEmailAddress()      { return emailAddress;      }
  //const string getEmailSignature()    { return emailSignature;    }
  //const string getIpAddress()         { return ipAddress;         }
    const bool   getPushStatus()        { return pushStatus;        }
    const bool   getEmailFilterStatus() { return emailFilterStatus; }

    void setVisibleName      (const string v);
    void setEmailAddress     (const string v);
  //void setEmailSignature   (const string v);
    void setPushStatus       (const bool   v);
    void setEmailFilterStatus(const bool   v);

    int  getDirty() { return dirty; }


    /**
    * Reads the current configuration from the registry and from the 
    * email account.
    * @return  true if all it is ok, false otherwise
    */
    bool read();

    /**
    * Saves the current configuration in the registry.
    * It also saves settings related to the email account. 
    * @return  true if all it is ok, false otherwise
    * @note "IPAddress" is ignored, nonsense from Server to Client.
    */
    bool save();


    /**
     * Returns the proper parameter, given its name (the itemKey).
     */
    string getParam(const wstring paramName);

};

#endif
