/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
#ifndef INCL_CONFIG_SYNC_SOURCE
#define INCL_CONFIG_SYNC_SOURCE

#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "pim/BaseSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "base/util/StringBuffer.h"

#include "spdm/errors.h"
#include "pim/Utils.h"
#include "ConfigParams.h"
#include "HwndFunctions.h"

#include <string>
#include <list>


#define PUSH_STATUS_FIELD         TEXT("./Push/Status")
#define EMAIL_FILTER_STATUS_FIELD TEXT("./Email/FilterStatus")
#define EMAIL_ADDRESS_FIELD       TEXT("./Email/Address")
#define VISIBLE_NAME_FIELD        TEXT("./Email/DisplayName")
//#define EMAIL_SIGNATURE_FIELD   TEXT("./Email/Signature")
//#define IP_ADDRESS_FIELD        TEXT("./IPAddress")


typedef std::list<std::wstring>         itemKeyList;
typedef itemKeyList::iterator           itemKeyIterator;


class ConfigSyncSource : public BaseSyncSource { //public SyncSource, public ArrayElement {

private:

    /// the mime type of the item
    string mimeType;    
    
    /**
    * Class that contains all the configuration properties that should/could be 
    * synced with the server
    */
    ConfigParams c;

    /// List of item keys: the names of properties to be  synced.
    itemKeyList  itemKeys;

    /// Iterator for the 'itemKeys' list.
    itemKeyIterator  iter;


protected:

    ConfigSyncSource(SyncSource& s);

public:
       
    /**
    * set/get the mimetype
    */
    void setMimeType(string t);
    string getMimeType();
    
    /**
     * Constructor: create a SyncSource with the specified name
     *
     * @param name - the name of the SyncSource
     */
    ConfigSyncSource(const wchar_t* name, AbstractSyncSourceConfig* sc);

    virtual ~ConfigSyncSource();

    /**
     * Return the first SyncItem of all.
     * It is used in case of slow or refresh sync
     */
    SyncItem* getFirstItem();

    /**
     * Return the next SyncItem of all.
     * It is used in case of slow or refresh sync
     */
    SyncItem* getNextItem();


    /**
     * Return the first SyncItem of updated one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getFirstUpdatedItem() ;

    /**
     * Return the next SyncItem of updated one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getNextUpdatedItem();

    /// Not used: properties are only updated.
    SyncItem* getFirstNewItem();
    /// Not used: properties are only updated.
    SyncItem* getNextNewItem();
    /// Not used: properties are only updated.
    SyncItem* getFirstDeletedItem();
    /// Not used: properties are only updated.
    SyncItem* getNextDeletedItem();

    void setItemStatus(const wchar_t* key, int status);

    /// Updates a property value. It's used like 'updateItem()'.
    int addItem(SyncItem& item);

    /// Updates a property value.
    int updateItem(SyncItem& item);

    /// Not used.
    int deleteItem(SyncItem& item);


    int beginSync();

    int endSync();

    void assign(ConfigSyncSource& s);

    /// Not used.
    SyncItem* getFirstItemKey();
    /// Not used.
    SyncItem* getNextItemKey();

    ArrayElement* clone();
    
    int removeAllItems() { return 0;};

    /*************************************/
    /*********** custom methods **********/
    /*************************************/
    
    /**
    * it updates the parameters inside the ConfigParams class
    */
    int updateConfigParams(SyncItem& item);
   
};

#endif
