/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef INCL_TASK_BUILDER
#define INCL_TASK_BUILDER

#include "pimstore.h"
#include "pim/Utils.h"
#include "pim/UtilsRecurrence.h"
#include "pim/crc32.h"

#include "vocl/VObject.h"
#include "vocl/WinTask.h"
#include "vocl/WinTaskSIF.h"

#include "syncml/core/CTCap.h"
#include "syncml/core/Property.h"

#include <string>
using namespace std;


// Enumeration of appointment fields used by WM plugin.
// It's defined in AppointmentBuilder.cpp.
extern const wchar_t* taskFields[];


/**
 * Parses a task IItem using low level properties. 
 * It populates a WinTask object.
 * @param pTask   the IItem task to parse 
 * @param isSif   true if we want to create a WinTaskSIF object
 * @return        the WinTask allocated and filled with properties from pTask
 */
WinItem* taskToWinTask(IItem *pTask, bool isSif);

/**
 * Reads all properties from the given WinTask, and populates the pTask IItem.
 * 1. if it's an UPDATED item, clear RecPattern -> delete recurrence if any
 *    (use old-style ITask interface to use the utility ClearRecurrencePattern)
 * 2. set all normal props to IItem
 * 3. if recurring, set all rec props to IItem (append after normal props)
 * 4. save the IItem
 *
 * @param winT         input WinTask (or WinTaskSIF) to read props from
 * @param pTask        IItem to set props and save in POutlook
 * @param path         path to write oidToReset file (for RDATEs exceptions)
 * @param oidToUpdate  the oid of appointment to update, MUST be 0 if it's a NEW item
 * @return             S_OK = 0 if operation succesfull, < 0 if errors
 */
HRESULT winTaskToTask(WinTask& winT, IItem *pTask, const wstring& path, long oidToUpdate);


// -- These functions should be removed once the update from old CRC is no more necessary --
void populateTaskStringItem (std::wstring &contactStringItem, ITask *pTask, BOOL isCRC = FALSE);
long calculateTaskHash      (ITask *pTask);
ArrayList* getTaskProperties();
ArrayList* getVTodoProperties();
// -----------------------------------------------------------------------------------------


#endif