/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#pragma once

#define HrFromWSAError(x) MAKE_HRESULT(SEVERITY_ERROR, FACILITY_INTERNET, (x))

#include <cemapi.h>
#include "AdviseSink.h"

void SetStatusTextOut(UINT nId);

class CTransportSyncHandler : public IMailSyncHandler
{

public:
    CTransportSyncHandler();
    virtual ~CTransportSyncHandler();

    virtual HRESULT STDMETHODCALLTYPE   QueryInterface  (REFIID rif, LPVOID * ppobj);
    virtual ULONG   STDMETHODCALLTYPE   AddRef();
    virtual ULONG   STDMETHODCALLTYPE   Release();


    MAPIMETHOD (Initialize)
    (
        IMailSyncCallBack   *   pCallback,
        LPCWSTR                 pszProfileName,
        IMsgStore           *   pMsgStore
    );

    MAPIMETHOD (Synchronize)
    (
        MAILSYNCREQUEST     *   pRqst
    );

    MAPIMETHOD (ShutDown)
    (
        DWORD                   dwReserved
    );

    MAPIMETHOD (Install)
    (
        LPCWSTR                 pszProfileName,
        LPCWSTR                 pszIncomingServer,
        LPCWSTR                 pszOutgoingServer,
        GUID                *   pidNetwork
    );

    MAPIMETHOD (UnInstall)
    (
    );


    MAPIMETHOD (Connect)
    (
        DWORD                   dwReserved,
        SYNCCREDENTIALS     *   pCredentials
    );

    MAPIMETHOD (Disconnect)
    (
        DWORD                   dwReserved
    );

    MAPIMETHOD (DoProperties)
    (
        HWND                    hwndParent
    );

    MAPIMETHOD (GetCapability)
    (
        LPCWSTR                 pszName,
        LPSPropValue            pval
    );

    MAPIMETHOD (SetCapability)
    (
        LPCWSTR pszName,
        LPSPropValue pval
    );

    MAPIMETHOD (DecodeEvent)
    (
        TRANSPORTEVENT      *   pevt,
        LPWSTR              *   ppszEvent
    );

    MAPIMETHOD (GetFolderHierarchy)
    (
        FOLDERNODE          **  ppRootNode
    );


    MAPIMETHOD (SetFolderOptions)
    (
        LPMAPIFOLDER            pfldr,
        FOLDERNODE          *   pnode,
        FOLDEROPTIONS           opts,
        LPSPropValue            pval
    );


protected:
    IMsgStore           *   m_pMsgStore;
    IMailSyncCallBack   *   m_pCallback;
    ULONG                   m_cRef;
    BOOL                    m_fShutDown;
    SYNCPROGRESSITEM        m_SyncProgressItemPending;
    ULONG                   m_nDeleteAction;
    LPWSTR                  m_pszProfile;
    HANDLE                  m_hEventShutdown;
    CAdviseSink         *   m_advise;


    void    SetStatusText                   (UINT nId);
    void    SendProgressMessage             (DWORD dwMask);
    void    HandlePendingProgressItem       (void);
    wchar_t*    getValueFromReg                 (wchar_t* regval);
    HRESULT LogErrorEvent                   (HRESULT hr, ULONG cbData);
    HRESULT OnFolderOptions_Download        (LPMAPIFOLDER pfldr, LPSPropValue pval);
    HRESULT OnFolderOptions_QueryDownload   (LPMAPIFOLDER pfldr, LPSPropValue pval);
    HRESULT OnFolderOptions_Create          (LPMAPIFOLDER * pfldr);
    HRESULT OnFolderOptions_QueryCreate     (LPMAPIFOLDER pfldr, LPSPropValue pval);
    HRESULT OnFolderOptions_SetFetchOption  (FOLDEROPTIONS opts, LPMAPIFOLDER pfldr, LPSPropValue pval);
    HRESULT OnFolderOptions_GetFetchOption  (FOLDEROPTIONS opts, LPMAPIFOLDER pfldr, LPSPropValue pval);
};


