/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef INCL_S4JPROXY
#define INCL_S4JPROXY
/** @cond WMPLUGIN_NOTLISTENER */
/** \addtogroup NotificationListenerDll */
/** @{ */
#include "syncml/core/SyncNotification.h"

wchar_t *getSSourceName(const char *root, const char *uri);

#define RESP_OK      "200 - OK\n"
#define RESP_BADREQ  "400 - Bad Request\n"
#define RESP_INVSYNC "400 - Invalid Sync type\n"
#define RESP_BUSY    "500 - Connector busy\n"
#define RESP_GENERIC "500 - Internal error\n"
#define RESP_NULLP   "500 - Null pointer\n"

/**
* The class preposed to contain the parsed server notification message. It leads the response to the
* server and it is responsible to start the sync with the server.
*/

class S4JProxy{

    public:
        // parseMessage return codes: Error, Parse OK, Message Incomplete.
        enum RetCode { Error=-1, Ok=0, Continue=1 };

        static S4JProxy* getInstance();
        static void dispose();

        // Parse the PKG0 message and return a S4JRetCode
        RetCode parsePkg0(const char *msg, int msglen);

        // Get the return string for current status code
        const char *getResponse();

        // Start a synchronization for the syncs requested by the server
        int sync(bool debug=false);

        const wchar_t *getSyncMessage();
        bool hasErrors();

    private:

        SyncNotification np;
        SN_Errors errcode;
        wchar_t *syncMessage;

        static S4JProxy *instance;

        S4JProxy();
        ~S4JProxy();
    };
/** @} */
/** @endcond */
#endif

