/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


// startsync.cpp - main module for the sync application
//

// Windows Header Files:
#include <windows.h>
#include <initguid.h>
#include <Notify.h>
#include <Pmpolicy.h>
#include "pim/mail/MailServer2Client.h"
#include "base/startcmd.h"
#include "base/util/utils.h"
#include "http/WinTransportAgent.h"

#include "pim/SettingFunctions.h"
#include "pim/maincpp.h"
#include "notify/addresschange.h"
#include "pim/difftime.h"
#include "usernotify.h"
#include "base/Log.h"
#include "pushmessage.h"
#include "notify/util.h"
#include "notify/timed_msgbox.h"
#include "spds/constants.h"
#include "OTAConfigMessage.h"
#include "OTAConfigMessageHandler.h"
#include <cemapi.h>
#include "pim/utils.h"
#include <string>
#include <list>
#include "customization.h"
#include "processUtils.h"
using namespace std;

#include "pim/maincpp.h"
//
// USE the pim/ClientSettings.h. THis must be shared with the same ClientSettings of the
// FunambolClient
//
#include "pim/ClientSettings.h"
#include "OutOfMemoryException.h"

#include "HwndFunctions.h"
#include "localizationUtils.h"

#define DIMCONF 128
#define DIM_PATH 256
#define NOT_RETURN_TO_MAIN_FORM -3
#define NO_SOURCES_FROM_TRANSPORT -12

#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif


#define PARAM_AVAILABLE         TEXT("available")
#define PARAM_REMOVENOTIF       TEXT("removeNotif")
#define PARAM_SCHEDULE          TEXT("schedule")
#define PARAM_CRADLE            TEXT("cradle")
#define PARAM_REGISTER          TEXT("register")
#define PARAM_DEREGISTER        TEXT("deregister")
#define PARAM_NOTIFY            TEXT("/notify")
#define PARAM_TRANSPORT         TEXT("transport")
#define PARAM_INCLUSIVE         TEXT("inclusive")
#define PARAM_MANUAL            TEXT("manual")
#define PARAM_TCPIP             TEXT("tcpip")
#define PARAM_CLEANUP           TEXT("cleanup")
#define PARAM_REFRESHSERVER     TEXT("refresh-from-server")
#define PARAM_REFRESHCLIENT     TEXT("refresh-from-client")
#define PARAM_T_REFRESHSERVER   TEXT("transport-refresh-from-server")
#define PARAM_T_REFRESHCLIENT   TEXT("transport-refresh-from-client")
#define PARAM_SEND_LOG          TEXT("sendlog")
#define PARAM_REMOVE_SCHEDULE   TEXT("remove-schedule")
#define PARAM_REFRESH_CONFIG    TEXT("refresh-config")
#define PARAM_TWO_WAY_CONFIG    TEXT("two-way-config")



#define QUEUED_SYNC_DELAY       1       // Dalay time (seconds) before starting a queued sync.


//--------------------------------------------------------------------- Prototypes

static inline void Alert(const wchar_t *msg, const wchar_t *title=TEXT("Warning"));

static int getPollingTime();
static bool isServerNotified();
static void setServerNotified(bool val);

//static int polling(const wchar_t **sources);
static int schedule();
static int sync(const wchar_t **sources, const wchar_t **ids);
static int doSync(const char* mode, const wchar_t **sources, const wchar_t **ids);
int checkRefreshFromServer(const wchar_t** sources);

static bool g_debug = false;
static int retValue = 0;

void setValuesInRegistry(const wchar_t* name, const wchar_t* value);
wchar_t** getEnabledSourcesFromConfig();
void setSourceOnReg(wchar_t** sources);

void setProperty(const char* fullContext, const char* propertyName, const char* propertyValue);

//--------------------------------------------------------------- Static Functions

static inline void Alert(const wchar_t *msg, const wchar_t *title)
{
#ifdef DEBUG
  	MessageBox (NULL, msg, title, MB_SETFOREGROUND |MB_OK);
#endif
}

static void trim(wstring& str)
{
  wstring::size_type pos = str.find_last_not_of(' ');
  if(pos != string::npos) {
    str.erase(pos + 1);
    pos = str.find_first_not_of(' ');
    if(pos != string::npos) str.erase(0, pos);
  }
  else str.erase(str.begin(), str.end());
}

/*
* This function save the process id passed by setProcessId or resetProcessId on the registry
* NOT use the function directly but use (re)setProcessID instead
*/
static void writeProcessIdOnRegistry(unsigned long processId) {
    wchar_t pid[128];
    wsprintf(pid, TEXT("%lu"), processId);
    setClientConfiguration(NULL, PROPERTY_SPDM_PID, pid, NULL);
}

static void resetProcessID() {
    writeProcessIdOnRegistry(0);
}

static void setProcessID() {
    DWORD processId = GetCurrentProcessId();
    writeProcessIdOnRegistry(processId);
}


static int getPollingTime()
{

    wchar_t buf[32];

    memset(buf, 0, 32*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_POLLING_NOTIFICATION,
                                 buf, FALSE);
    return wcstol(buf, NULL, 10);

}

static bool isServerNotified()
{
    wchar_t buf[10];

    memset(buf, 0, 10*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_SERVER_NOTIFIED,
                                     buf, FALSE);
    return ( wcscmp(buf, TEXT("1")) == 0);

}

static void setServerNotified(bool val)
{

    setClientConfiguration(
            TEXT(""),
            PROPERTY_SPDM_SERVER_NOTIFIED,
            ( (val) ? TEXT("1") : TEXT("0") ),
            NULL);

}


static void getAppNameForSMS(wchar_t* appName) {

    wchar_t path[256];
    getClientConfigurationInternal (NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);
    wcscpy(appName, path);
    wcscat(appName, TEXT("\\"));
    wcscat(appName, SYNCAPP);    

}

static void getAppName(wchar_t* appName) {

    wchar_t path[256];
    getAppNameForSMS(path);
    wcscpy(appName, TEXT("\""));    
    wcscat(appName, path);
    wcscat(appName, TEXT("\""));


}
/*
* NOT_RETURN_TO_MAIN_FORM: is the value to avoid to send a message for the result page.
* It is globally set.
* the return value is the flag to signal if it is a scheduled sync for client push or 
* a usual scheduled sync.
* @return 0: the usual scheduled sync
* @return 1: the client push signal
*/

static int schedule()
{
    long t = getPollingTime();
    long push = 0;
    long interval = 0;       
    int ret = 0;
    
    ClientSettings* cs = getRegConfig();
    string pushS = cs->getPush();    
    push = atoi(pushS.c_str());

    WCHAR appName[MAX_PATH];
    getAppName(appName);
    SYSTEMTIME lpTime;
    SYSTEMTIME pollTime;

    LOG.debug("Schedule start");
    if (t > 0 || push > 0) {

        LOG.debug("Scheduling '%ls'", appName);

                        
            //
            // Schedule the polling before running the sync.
            // it needs to avoid disable of polling if the user stops the sync.
            //
            // NOTE: for the client push notification, the schedule is set using the
            // parameter clientPushInterval.
            if (push > 0) {
                string cpinterval = cs->getClientPushInterval();
                if (cpinterval == "" || cpinterval.length() > 0) {
                    interval = (cpinterval == "" ? 0 : atol(cpinterval.c_str()));
                    if (interval > 0) {
                        LOG.info("CP: Push schedule");
                        ret = 1;
                    } else {                
                        LOG.info("CP: Push doesn't set to schedule");
                        CeRunAppAtTime(appName, 0);
                        ret = 1; 
                        retValue = NOT_RETURN_TO_MAIN_FORM;
                        return ret;
                    }
                }                
            } else {
                LOG.info("Schedule");
                interval = t;
                ret = 0;
            }
            GetLocalTime(&lpTime);
            pollTime = DT_AddDiff(nano100SecInMin, interval, &lpTime);
            BOOL isSchedul = CeRunAppAtTime(appName,&pollTime);                       

        LOG.info("Next scheduled sync of %S will start at '%02d.%02d.%02d'", 
                 appName, pollTime.wHour, pollTime.wMinute, pollTime.wSecond);

    }
    else if (t == 0 && push == 0) {
        LOG.info("Scheduled removed");                
        CeRunAppAtTime(appName,0);
        resetPIMModifications(NULL);
    }

    LOG.debug("Schedule end");
    retValue = NOT_RETURN_TO_MAIN_FORM;
    return ret;
}

static int notify(const wchar_t **sources=NULL)
{
    int responseCode = notifyAddressChange(TEXT(APPLICATION_URI));

    if (responseCode == AN_AddressAccepted) {
        setServerNotified(true);
    }
    else {
        setServerNotified(false);
    }
    return responseCode;
}

static int sync(const wchar_t **sources = NULL, const wchar_t **ids = NULL)
{
    return doSync(NULL, sources, ids);
}

static int recover(const char* mode, const wchar_t **sources)
{
    return doSync(mode, sources, NULL);
}

//static int inclusive(const wchar_t **sources)
//{
//    doSync(mode, sources, NULL);
//}

/**
 * @param sources - syncronize the specified sources. Default is NULL: it means the sources
 *                to sync are the one set in the registry
 * @param ids - the ids of the mail to be downloaded with a inclusive filter
 *              usually it is NULL. Not null only with the "inclusive" command line
 */
static int doSync(const char* mode, const wchar_t **sources, const wchar_t **ids)
{
    HANDLE hMutex;  //Handle to mutex...

    // check for the storage memory...
    checkStorageMemory(0);

    // Create mutex here
    hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );

    switch(GetLastError()) {
        case ERROR_SUCCESS:
            LOG.debug("Mutex created.");
            break;
        case ERROR_ALREADY_EXISTS:
            // Save the sources to sync into a queue: sync will start at the end
            // of current sync.
            LOG.info("Another sync in progress, current syncis queued");
            updateQueueList(sources);
            retValue = ERR_ANOTHER_SYNC;
            return retValue;
        default:
            LOG.error("Failed to create mutex");
            return -1;
    }


    wchar_t path[DIM_PATH];
    getClientConfigurationInternal(NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);

    HANDLE handleBubble = CreateThread(NULL, 0, createBubble, NULL, 0, NULL);
    if (handleBubble == 0) {
        LOG.error("CreateThread failed: <0x%08x>\n", GetLastError());
    }
    WaitForSingleObject(handleBubble, 0);

    setProcessID();
    // ----------------------------------------------
    retValue = synchronize(path, sources, ids, mode);
    // ----------------------------------------------


    if (retValue == ERR_HTTP_INFLATE) {
        // Manage the ZLIB inflate error: retry the sync with compression disabled. 
        // Note: the registry key 'enableCompression' is set to "0", so the 
        //       zlib compression will be disabled from now on.
        string context = APPLICATION_URI;
        context += CONTEXT_SPDS_SYNCML;
        context += CONTEXT_CONN;
        setProperty(context.c_str(), PROPERTY_ENABLE_COMPRESSION, "0");

        ClientSettings* cs = getRegConfig();
        cs->getAccessConfig().setCompression(FALSE);

        LOG.info("ZLIB compression disabled, retry the sync...");
        retValue = synchronize(path, sources, ids, mode);
    }


    if (retValue == 0 || retValue < ERR_TRANSPORT_BASE) {
        // If sync ended succesfully or if it's NOT a network error, 
        // we reset the PIMList ONLY for the sources just synced.
    	resetPIMModifications(sources);
    }
    else {
        // In case of network errors, we leave the PIMList like it is now. 
        // So if it was a ClientPush, we will schedule a new sync.
    }


    // Start automatically the queued sync.
    // The 'queueList' value is cleared in registry, but a new sync notification
    // could be received during the sync.
    wchar_t** sList = NULL;
    while (sList = readQueueList()) {

        // If we have to do a queued sync, it's better to include also
        // sources that should be synced due to ClientPush. Merge the lists.
        wchar_t** allList = NULL;
        wchar_t** cpList  = readPIMModifications();
        allList = getCompleteList(sList, cpList);

        LOG.info("Starting a QUEUED sync in %d seconds...", QUEUED_SYNC_DELAY);
        SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_STARTSYNC_ENDED, 0, 0);
        Sleep(QUEUED_SYNC_DELAY * 1000);
        retValue = synchronize(path, (const wchar_t**)allList, ids, mode);

        // If sync ended succesfully or if it's NOT a network error, 
        // we reset the PIMList ONLY for the sources just synced.
        if (retValue == 0 || retValue < ERR_TRANSPORT_BASE) {
            resetPIMModifications((const wchar_t**)allList);
        }

        if (sList) {
            if (sList == allList) { allList = NULL; }
            for (int i=0; sList[i]  ; i++) { delete [] sList[i];   sList[i] = NULL;  }
            delete [] sList; sList = NULL;
        }
        if (cpList) {
            if (cpList == allList) { allList = NULL; }
            for (int i=0; cpList[i] ; i++) { delete [] cpList[i] ; cpList[i] = NULL; }
            delete [] cpList; cpList = NULL;
        }
        if (allList) {
            for (int i=0; allList[i]; i++) { delete [] allList[i]; allList[i] = NULL; }
            delete [] allList; allList = NULL;
        }
    }
    if (retValue != NOT_RETURN_TO_MAIN_FORM && retValue >= 0 && retValue < ERR_TRANSPORT_BASE) {
        int check = checkUpdate();
        getRegConfig()->getUpdateParams().setIsAnUpdateAvailable(check == 0 ? "0" : "1");
        getRegConfig()->storeUpdateParams();        
    }

    resetProcessID();

    CloseHandle( hMutex );
    LOG.debug("Mutex released.");

    return retValue;
}


/**
 * Parse the command line
 *
 * @param cmdline - the command line (syntax: command [source,source,...])
 *
 * @param command (out) - the command name
 * @param sources (out) - a NULL terminated array of source names
 */
static int parseCmdLine(const LPTSTR cmdline,
                        wchar_t **command,
                        wchar_t ***sources)
{

    *command = NULL;
    *sources = NULL;

    if (cmdline == NULL || wcscmp(cmdline, TEXT("")) == 0) {
        return 1;
    }

    wstring token(TEXT(" ,"));
    list<wstring> elements;
    list<wstring>::iterator it;
    wstring all = cmdline;
    size_t previous = 0;

    size_t found = all.find_first_of(token);    

    while (found != wstring::npos) {        
        wstring data = all.substr(previous, found - previous);
        if (data != TEXT("")) {
            elements.push_back(data);
        }
        previous = found + 1;
        found = all.find_first_of(token, previous);        
    }    
    if (previous < all.size()) {
        wstring data = all.substr(previous, all.size());
        trim(data);
        elements.push_back(data);
    }

    wchar_t **srclist = NULL;

    if (elements.size() == 0) {
        // nothing to do

    } else if (elements.size() == 1) {
        it = elements.begin();
        *command = wstrdup((*it).c_str()); 

    } else { // size > 1
        int size = elements.size();
        srclist = new wchar_t *[size];
        unsigned int i = 0;
        for (it = elements.begin(); it != elements.end(); it++) {
            wstring line = *it;       
            if (i == 0) { 
                *command = wstrdup(line.c_str()); 
            } else { 
                srclist[i-1] = wstrdup(line.c_str()); 
            }
            i++;
        }
        srclist[size - 1] = 0;
        *sources=srclist;
    } 

    return 0;
}

static void exitHandler() {
    if (retValue != ERR_ANOTHER_SYNC) {
        removeNotif();
        HwndFunctions::closePreviousMsgBox();
        HWND hwnd = HwndFunctions::getWindowHandle();

        wchar_t title[64];
        wcscpy(title, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT)); 
        if (hwnd && retValue != NOT_RETURN_TO_MAIN_FORM) {
            SendMessage(hwnd, ID_MYMSG_STARTSYNC_ENDED, NULL, (LPARAM)retValue ); // send msg startsync ended
        }          
        else if (!hwnd && retValue != NOT_RETURN_TO_MAIN_FORM && retValue >= 0 && retValue < ERR_TRANSPORT_BASE) {
            // if there is no window of the UI, starts the update procedure
            // no windows could be due by a schedule or push where the UI was closed with Close command
            updateProcedure(NULL);
        }        
        else if (retValue == -40) {
            MessageBox (NULL, getLocalizationUtils()->getLocalizationString(IDS_ERR_OUT_OF_MEMORY), 
                title, MB_SETFOREGROUND | MB_ICONHAND |MB_OK);
            
        } else if (retValue == -50) {
            MessageBox (NULL, getLocalizationUtils()->getLocalizationString(IDS_ERR_LOW_MEM), 
                title, MB_SETFOREGROUND | MB_ICONHAND |MB_OK);
            
        }
        
    }
    if (retValue == -6) { // sync stopped by the user
        // reset the pid property
        resetProcessID();
    }
}

//------------------------------------------------------------ Public functions
void quit(int value) {
    retValue = value;
    exit(retValue);
}




//------------------------------------------------------------------------ MAIN
int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPTSTR    lpCmdLine,
                   int       nCmdShow) {       
    
    _set_new_mode(1);
    _set_new_handler(operatorNewHandler);
    
    try {
        
        //MessageBox (NULL, lpCmdLine, TEXT("title"), MB_SETFOREGROUND |MB_OK);
        wchar_t *command, **sources;

        HRESULT         hr              = NOERROR;
        HANDLE          hevtMsgAvailable= NULL;
        HPUSHROUTER     hPushRouter     = NULL;
        PUSHMSG         PushMsg         = {0};

        PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
        //SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_BELOW_NORMAL);
        
        if(g_debug){
            Alert(lpCmdLine);
        }

        if (parseCmdLine(lpCmdLine, &command, &sources)) {
            Alert(TEXT("Syntax error in command line"), TEXT("StartSync"));
            return 1;
        }
        
        if (command == NULL) {
            Alert(TEXT("No command line. Exit"), TEXT("StartSync"));
            return 1;
        }

        if (wcscmp(command, TEXT("addresschange")) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            return notify();            
        }

        // Set the exit handler to remove the icon
        atexit(exitHandler);
        
        if(!(wcscmp(command,PARAM_AVAILABLE) == 0 ||
            wcscmp(command, PARAM_REMOVENOTIF) == 0 ||
            wcscmp(command, PARAM_SCHEDULE) == 0 ||
            wcscmp(command, PARAM_CRADLE) == 0 ||
            wcscmp(command, PARAM_REGISTER) == 0 ||
            wcscmp(command, PARAM_DEREGISTER) == 0 ||
            wcscmp(command, PARAM_NOTIFY) == 0 ||
            wcscmp(command, PARAM_TRANSPORT) == 0 ||
            wcscmp(command, PARAM_INCLUSIVE) == 0 ||
            wcscmp(command, PARAM_MANUAL) == 0 ||
            wcscmp(command, PARAM_REFRESHSERVER) == 0 ||
            wcscmp(command, PARAM_REFRESHCLIENT ) == 0 ||
            wcscmp(command, PARAM_T_REFRESHSERVER) == 0 ||
            wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0 ||
            wcscmp(command, c_szParams) == 0 ||
            wcscmp(command, PARAM_TCPIP) == 0 ||
            wcscmp(command, APP_RUN_AT_TIME) == 0  ||
            wcscmp(command, PARAM_CLEANUP) == 0 ||
            wcscmp(command, PARAM_SEND_LOG) == 0  ||
            wcscmp(command, PARAM_REMOVE_SCHEDULE) == 0 ||// used by setup to close the schedule for client push          
            wcscmp(command, PARAM_REFRESH_CONFIG) == 0 ||
            wcscmp(command, PARAM_TWO_WAY_CONFIG) == 0
            ) ) {

                quit(0);
        }


        // If APP_RUN_AT_TIME, we set the Schedule and then we start 
        // the sync at the end of the WinMain (inside the 'else' statement)
        if(wcscmp(command, PARAM_REMOVE_SCHEDULE) == 0 ){
            WCHAR appName[MAX_PATH];
            getAppName(appName);
            CeRunAppAtTime(appName, 0);
            resetPIMModifications(NULL);
            retValue = NOT_RETURN_TO_MAIN_FORM;
            return retValue;
        }

        // If APP_RUN_AT_TIME, we set the Schedule and then we start 
        // the sync at the end of the WinMain (inside the 'else' statement)
        if(wcscmp(command, APP_RUN_AT_TIME) == 0 ){
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);

            int pushValue = 0;
            string push = getRegConfig()->getPush();
            if (push.size() > 0) {
                pushValue = atoi(push.c_str());
            }
            if (pushValue > 0) {
                //
                // We have PUSH activated: it's a ClientPush sync.
                //
                sources = readPIMModifications(); 

                // Remove any schedule of startsync. It will be rescheduled at the
                // end of the sync, in case of failure.
                WCHAR appName[MAX_PATH];
                getAppName(appName);
                CeRunAppAtTime(appName, 0);    
                LOG.debug("startsync schedule removed after APP_RUN_AT_TIME");
                if (!sources) {
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                    return retValue;
                }

                delete [] command;
                command = wstrdup(TEXT("clientPush"));
            }
            else {
                // It's a normal scheduled sync. 
                // Schedule now the next sync.
                schedule();
            }
        }
        /*
        * This is to sync the configuration parameter. 
        */
        else if(wcscmp(command, PARAM_REFRESH_CONFIG) == 0) {    
            getRegConfig()->setConfigSourcesParameter("config", "sync", "refresh-from-server");
            getRegConfig()->setDirty(CS_DIRTY_SOURCE_ENABLED);
            getRegConfig()->saveDirty();            
            delete [] command;
            command = wstrdup(TEXT("manual"));            
            
        } else if(wcscmp(command, PARAM_TWO_WAY_CONFIG) == 0) {   
            getRegConfig()->setConfigSourcesParameter("config", "sync", "two-way");
            getRegConfig()->setDirty(CS_DIRTY_SOURCE_ENABLED);
            getRegConfig()->saveDirty();            
            command = wstrdup(TEXT("manual"));                
        }           

        /**
        * This function is to send the log to the Technical Support
        * It changes the parameter as a normal manual sync of the email.
        * It is not to put in else as the other
        */
        if(wcscmp(command, PARAM_SEND_LOG) == 0) {            
            int m = sendLog();
            if (m == 0) {                
                delete [] command;
                command = wstrdup(TEXT("manual"));                        
            } else {
                LOG.error("The log cannot be sent");
            }
        }  

        //-------------------------

        /*
        * These two following test are only to understand and make right 
        * modification. So the Cradle action is normalized and the RunAppAtTime
        * currently only schedules the next time sync and leaves to the flow
        * the right choice.
        */
        if(wcscmp(command, PARAM_CLEANUP) == 0) {            
            vector<const wchar_t*> ss;
            for (int i = 0; sources[i] != NULL; i++) {
                ss.push_back(sources[i]);
            }
            cleanupAllItems(ss);
            // reset the pim flag after a cleanup because it writes the registry
            resetPIMModifications((const wchar_t**)sources);
            retValue = NOT_RETURN_TO_MAIN_FORM;
        } else if (wcsicmp(command, PARAM_NOTIFY) == 0 ){
            //This if for the on cradle notification
            delete [] command;
            command = wstrdup(TEXT("cradle"));
            //command = TEXT("cradle");
            if (sources) { for (int k = 0; sources[k]; k++) { delete [] sources[k]; sources[k] = NULL; } sources = NULL;}
    
            // if (sources){ delete [] sources;}  sources = NULL;
        } else if(wcscmp(command, PARAM_AVAILABLE) == 0) {
            getAvailability();
        } else if(wcscmp(command, PARAM_REMOVENOTIF) == 0) {
            removeNotif();
            retValue = NOT_RETURN_TO_MAIN_FORM;            
        } else if(wcscmp(command, PARAM_SCHEDULE) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            schedule();                     
        } else if(wcscmp(command, PARAM_REGISTER) == 0) {            
            WCHAR c_szPath[MAX_PATH];
            getAppNameForSMS(c_szPath); // /Program files/Funambol/startsync.exe
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            hr = PushRouter_RegisterClient(c_szContentType, c_szAppId, c_szPath, c_szParams);
            if (hr == S_OK) {
                LOG.debug("Wap push client registered successfully");
            } else if (hr == PUSHRTR_E_ALREADYREGISTERED) {
                LOG.debug("Wap push client already registered");
            } else {
                LOG.error("Error to register wap push client");
            }
            retValue = NOT_RETURN_TO_MAIN_FORM;
            
        } else if(wcscmp(command, PARAM_DEREGISTER) == 0) {
            //
            // unregister the startsync from the pushrouter
            //
            hr = PushRouter_UnRegisterClient(c_szContentType, c_szAppId);
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            if (hr == S_OK) {
                LOG.debug("Wap push client unregistered successfully");
            } else if (hr == PUSHRTR_E_NOTFOUND) {
                LOG.debug("Wap push client not found. Arleady unregistered");
            } else {
                LOG.error("Error to unregister wap push client");
            }
            retValue = NOT_RETURN_TO_MAIN_FORM;
        } else if(wcscmp(command, c_szParams) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            memset(&PushMsg, 0, sizeof(PUSHMSG));
            PushMsg.cbSize = sizeof(PUSHMSG);

            hr = PushRouter_Open(c_szContentType, c_szAppId, &hevtMsgAvailable, &hPushRouter);
            if (hr != S_OK) {
                LOG.error("Wap push: PushRouter_Open - it is not possible to register the client.");
            }
            while(WaitForSingleObject(hevtMsgAvailable, 10000) != WAIT_TIMEOUT) {
                hr = PushRouter_GetMessage(hPushRouter, &PushMsg);
                if (hr != S_OK) {
                    LOG.error("Error in PushRouter_GetMessage wap push client");
                    break;
                }
                wchar_t** ss;
                wchar_t** ws = NULL;
                int ret = handleWapPushMessage(&PushMsg, &ws);

                int i = 0;
                int a = 0;
                while(ws[i]){
                    char* srctocmp = toMultibyte(ws[i]);
                    if (!(strcmp(getRegConfig()->getConfigSourcesParameter(srctocmp, "sync"), "none") == 0)){
                        a++;
                    }
                    i++;
                    if (srctocmp){
                        delete [] srctocmp;
                    }
                }

                if(a != 0){
                    ss = new wchar_t*[a+1];

                    i = 0;
                    a = 0;
                    int sizews = 0;
                    while(ws[i]){

                        char* srctocmp = toMultibyte(ws[i]);
                        if (!(strcmp(getRegConfig()->getConfigSourcesParameter(srctocmp, "sync"), "none") == 0)){
                            sizews = wcslen(ws[i])+ 1;
                            ss[a] = new wchar_t[sizews];
                            wcscpy(ss[a], ws[i]);
                            a++;
                        }
                        if ( srctocmp ){
                            delete [] srctocmp;
                        }
                        i++;
                    }
                    ss[a] = 0;
                }
                else{
                    ss = NULL;
                }
                if (ret != 0) {
                    LOG.error("Wap push: error in handleWapPushMessage message");
                    break;
                }
                if (ss) {
                    if (askConfirm()) {
                                                
                        bool found = false;
                        int i = 0;
                        for (i = 0; ss[i]; i++) {
                            if (wcsstr(ss[i], L"mail") != 0) {
                                found = true;
                            }
                        }
                        if (found) {
                             setSourceOnReg(ss);
                             MailSyncMessages(ACCOUNT_NAME, MCF_ACCOUNT_IS_NAME | MCF_RUN_IN_BACKGROUND); 
                             return retValue = NOT_RETURN_TO_MAIN_FORM;                             
                        }

                        sync((const wchar_t **)ss);
                    }
                }
                PushRouter_FreeMessage(&PushMsg);
                memset(&PushMsg, 0, sizeof(PUSHMSG));
                PushMsg.cbSize = sizeof(PUSHMSG);                
            }
        } else if (wcscmp(command, PARAM_INCLUSIVE) == 0) {
            
            // just to know the number of ids
            unsigned int i = 0;
            for (i = 0; sources[i]; i++) {
                // the position start from 0 [that is mail, the name of the source]
                // [1], [2].. contains the ids
                //
            }
            // the name of the source: it should be "mail"
            wchar_t* mailSourceName = wstrdup(sources[0]);

            // the buffer for the ids
            const wchar_t ** ids = new const wchar_t*[i];

            for (unsigned j = 0; j < i; j++) {
                ids[j] =  wstrdup(sources[j + 1]);
            }
            if (sources) { for (int k = 0; sources[k]; k++) { delete [] sources[k]; sources[k] = NULL; } sources = NULL;}
            //delete [] sources;

            sources = new wchar_t* [2];
            sources[0] = mailSourceName;
            sources[1] = 0;
            sync((const wchar_t **)sources, (const wchar_t **)ids);

        } else if (wcscmp(command, PARAM_TRANSPORT) == 0 ||
                   wcscmp(command, PARAM_T_REFRESHSERVER) == 0 ||
                   wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0) {

            if (strcmp(getRegConfig()->getConfigSourcesParameter("mail", "sync"), "none") == 0) {
                wchar_t title[128];
                wsprintf(title, TEXT("%s"), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT));
                int ret = TimedMessageBox(NULL, 
                    getLocalizationUtils()->getLocalizationString(IDS_MAIL_DISABLED),
                    title, 
                    MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);
                if(ret == IDYES){
                    getRegConfig()->setConfigSourcesParameter( "mail" , "sync" , "two-way" );
                    getRegConfig()->setDirty(CS_DIRTY_SOURCE_ENABLED);
                    getRegConfig()->saveDirty();
                } else {
                    return 1;
                }
            }
            if( wcscmp(command, PARAM_T_REFRESHSERVER) == 0) {
                if (sources == 0) { //////****************
                    LOG.error("%S call without sources", PARAM_T_REFRESHSERVER);
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                } else {
                    recover("refresh-from-server", (const wchar_t **)sources);
                }

            } else if (wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0) {
                if (sources == 0 ){ //////****************
                    LOG.error("%S call without sources", PARAM_T_REFRESHCLIENT);
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                } else {
                    recover("refresh-from-client", (const wchar_t **)sources);
                }

            } else if (sources) {
                sync((const wchar_t **)sources);

            } else if (!sources) {
                LOG.error("Transport call without sources");
                retValue = 0;
            }
        } else {
            if (!sources) {
                sources = getEnabledSourcesFromConfig();
                if (!sources) {
                    LOG.error("No sources in the config are selected to be synced");
                    quit(1);
                }                
            }
            bool flagmail = false;
            for(int i = 0; sources[i]; i++){
                if (wcsicmp(sources[i], TEXT("mail")) == 0){
                    flagmail = true;
                }
            }
            if(!flagmail) {
                if (wcsicmp(command, TEXT("refresh-from-server")) == 0) {
                    if (sources == 0) {
		                quit(1);
	                }
                    recover("refresh-from-server", (const wchar_t **)sources);

                } else if (wcscmp(command, TEXT("refresh-from-client")) == 0) {
                    if (sources == 0) {
		                quit(1);
                    }
                    recover("refresh-from-client", (const wchar_t **)sources);
                }
                else if (wcscmp(command, TEXT("clientPush")) == 0) {
                    int ret = sync((const wchar_t **)sources);
                    wchar_t** sourcesLeft = readPIMModifications();
                    if (sourcesLeft) {
                        // We still have sources to sync (sync failed), so schedule a new sync.
                        schedule();
                        if (sourcesLeft) { for (int k = 0; sourcesLeft[k]; k++) { delete [] sourcesLeft[k]; sourcesLeft[k] = NULL; } sourcesLeft = NULL;}
                        
                    }
                    // Force to return the real errorcode of the sync. To avoid NOT_RETURN_TO_MAIN_SCREEN.
                    retValue = ret;
                } 
                else { 
                    // manual parameter
                    sync((const wchar_t **)sources);
                }
            } else {
                if (wcscmp(command, TEXT("refresh-from-server")) == 0 || 
                    wcscmp(command, TEXT("refresh-from-client")) == 0){
                    
                    wstring value = TEXT("transport-");
                    value += command;
                    setValuesInRegistry(TEXT("transportCommand"), value.c_str());
                }
                setSourceOnReg(sources);
                MailSyncMessages(ACCOUNT_NAME, MCF_ACCOUNT_IS_NAME | MCF_RUN_IN_BACKGROUND);
                return retValue = NOT_RETURN_TO_MAIN_FORM;
            }
        }
        if (hPushRouter) {
            PushRouter_Close(hPushRouter);
            PushRouter_FreeMessage(&PushMsg);
        }

        PowerPolicyNotify(PPN_UNATTENDEDMODE, FALSE);

        if (command){
            delete [] command;
        }
        if (sources) { for (int k = 0; sources[k]; k++) { delete [] sources[k]; sources[k] = NULL; } sources = NULL;}
        
        //if (sources){
        //    delete [] sources;
        //}

    }//end try
    catch (OutOfMemoryException& e) {

        int k = e.getMemExcErrorCode();
        if (k == -50) {
            LOG.error("Low Storage Memory Exception.");
            quit(k);
        }
        else if (k == -1) { // error due to Out of memory
            LOG.error("Out of memory. Required %lu ", e.getMemExcMemoryRequired());
            quit(-40);
        } else { // error due to Out of memory. Not -1 set but supposed
            LOG.error("Out of memory");
            quit(-40);
        }
    }
    catch (...) {
        LOG.error("Generic failure");
        quit(-2);
    }


    if (retValue == 0) {
        LOG.info("Sync successful.");
    }
    else if (retValue == NOT_RETURN_TO_MAIN_FORM) {
        // nothing to do
    }
    else {
        LOG.error("Sync not successful.");
    }

    return retValue;
}

void setValuesInRegistry(const wchar_t* name, const wchar_t* value) {
    
    HKEY key;
    DWORD res;
    int sizew = 0;

    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        TEXT("Software\\") ROOT_CONTEXT_W,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
        );
    
    if (key == 0) {
        goto finally;
    }
        
    RegSetValueEx(
        key,
        name,
        NULL,
        REG_SZ,  // we currently support only strings
        (UCHAR*)value,
        (wcslen(value)+1)*sizeof(WCHAR)
        );

    RegFlushKey(key);

finally:

    if (key != 0) {
        RegCloseKey(key);
    }    
}


/**
* Set a list of sources that has to be synced passing through 
* the SyncMLTransport to take advantage of the mail notification.
* This is done only if the mail source is in the list of sources 
* to be synced
*/
void setSourceOnReg(wchar_t** sources) {
    
    wchar_t* value = NULL;
    int sizew = 0;
   
    for (int i = 0; sources[i]; i++ ){
        sizew += wcslen (sources[i]) + 1;
    }

    value = new wchar_t[sizew];
    
    for (int i = 0; sources[i]; i++ ){
        if (i == 0){
            wcscpy(value, sources[i]);
        } else {
            wcscat(value, TEXT(","));
            wcscat(value, sources[i]);
        }
    }
    
    setValuesInRegistry(TEXT("transportSources"), value);
   
    if (value) {
        delete [] value; value = NULL;
    }
}

/**
* Read the enabled sources from the configuration and return an array null terminated
* with the names of the sources to be synced
*/
wchar_t** getEnabledSourcesFromConfig() {
    
    int i = 0, count = 0;
    wchar_t** sourcesfr = NULL;

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("mail", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("contact", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("calendar", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("task", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("note", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("briefcase", "sync"), "none") == 0)) {
        count++;
    }
    if (count == 0) {
        return sourcesfr;
    }

    sourcesfr = new wchar_t*[count + 1];                

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("mail", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("mail"));
        i++;
    }

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("contact", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("contact"));        
        i++;
    }

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("calendar", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("calendar"));
        i++;
    }

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("task", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("task"));                         
        i++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("note", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("note"));                          
        i++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("briefcase", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("briefcase"));  
        i++;
    }
    sourcesfr[i] = 0;
    return sourcesfr;

}

/**
 * Utility function to set a desired property in Windows registry.
 * @param fullContext    the full context path of the key to set (under HKLM)
 * @param propertyName   the name of the key to set
 * @param propertyVlaue  the value to set
 */
void setProperty(const char* fullContext, const char* propertyName, const char* propertyValue) {

    if (!fullContext  || !propertyName || !propertyValue) {
        return;
    }

    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    dmt = DMTreeFactory::getDMTree(fullContext);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(fullContext);
    if (!node)  goto finally;
    node->setPropertyValue(propertyName, propertyValue);

finally:
    if (dmt)   delete dmt;
    if (node)  delete node;
}


/**
* function to try the parsign of command line
*/

void testCommandLineParsing() {

    wchar_t *com, **sou;
    parseCmdLine(TEXT("manual"), &com, &sou);
    if (sou) { for (int i = 0; sou[i]; i++) { delete [] sou[i]; sou[i] = NULL; } }
    parseCmdLine(TEXT("manual  contact, calendar  "), &com, &sou);
    if (sou) { for (int i = 0; sou[i]; i++) { delete [] sou[i]; sou[i] = NULL; } }
    parseCmdLine(TEXT("manual contact,calendar,task"), &com, &sou);
    if (sou) { for (int i = 0; sou[i]; i++) { delete [] sou[i]; sou[i] = NULL; } }
    parseCmdLine(TEXT("manual contact, calendar,   task"), &com, &sou);
    if (sou) { for (int i = 0; sou[i]; i++) { delete [] sou[i]; sou[i] = NULL; } }
    parseCmdLine(TEXT("transport-refresh-from-client mail,   task    "), &com, &sou);
    if (sou) { for (int i = 0; sou[i]; i++) { delete [] sou[i]; sou[i] = NULL; } }
    parseCmdLine(TEXT("/notify   "), &com, &sou);
    if (sou) { for (int i = 0; sou[i]; i++) { delete [] sou[i]; sou[i] = NULL; } }
    parseCmdLine(TEXT(""), &com, &sou);
    
}


    /*
int parsingCommandLine() {
 OLD PARSING COMMANDLINE

    if( wcsstr(cmdline, TEXT("debug ")) ) {
        *command = wstrdup(cmdline+wcslen(TEXT("debug ")));
        g_debug = true;
    }
    else {
        *command = wstrdup(cmdline);
        g_debug = false;
    }
   

    *sources = NULL;

    wchar_t *args = wcschr(*command, ' ');
    if(!args) {
        // in this case comand == cmdline and sources is null
        return 0;
    }
    // terminate command and point to the start of arguments
    args[0] = 0; args++;
    if (!args[0]) {
        // empty args after a space, like before
        return 0;
    }
    int argc;
    wchar_t *arg = args;
    // Count arguments
    for(argc = 1; (arg=wcschr(arg+1, ',')) != 0; argc++);

    // Get arg list
    wchar_t **srclist = new wchar_t *[argc+1];
    arg=args;
    int i;

    for(i=0; i<argc; i++) {
        if(!arg || !arg[0])
            return -1;
        srclist[i]=arg;

        arg=wcschr(arg, ',');
        if(arg) {
            arg[0]=0; arg++;
        }
    }
    srclist[i]=0;
    *sources=srclist;
}
*/
